<pre class='metadata'>
Title: CSS Line Grid Module Level 1
Status: ED
Work Status: Exploring
Shortname: css-line-grid
Level: 1
Group: csswg
TR: https://www.w3.org/TR/css-line-grid-1/
Previous Version: https://www.w3.org/TR/2014/WD-css-line-grid-1-20140916/
ED: https://drafts.csswg.org/css-line-grid/
Editor: Elika Etemad, Invited Expert, http://fantasai.inkedblade.net/contact, w3cid 35400
Editor: Koji Ishii, Invited Expert, kojiishi@gluesoft.co.jp, w3cid 45369
Editor: Alan Stearns, Adobe Systems&#44; Inc., stearns@adobe.com, w3cid 46659
Abstract: This module contains CSS features for aligning content to a baseline grid.
</pre>

<style type="text/css">
	.sidenote-block {
		border: 1px solid gray;
		margin: auto;
		text-align: left;
		padding: 4pt;
		position: relative;
		max-width: 400pt;
	}
	.sidenote-body {
		margin-left: 120pt;
		font-size: 12pt;
		line-height: 18pt;
	}
	.sidenote-note {
		font-size: 10pt;
		line-height: 18pt;
		position: absolute; left: 4pt; top: 4pt;
		width: 110pt;
	}
</style>

<h2 id='intro'>
Introduction</h2>

<h3 id='context'>
Background</h3>

<em>This section is not normative.</em>

	This specification provides features
	to align lines and blocks
	to invisible grids in the document.

	Aligning lines and blocks to grids
	provides the following benefits:

	<ul>
		<li>Vertical rhythm is kept for better readability.</li>
		<li>Lines are aligned between columns in multi-column documents.</li>
		<li>The top and the bottom margins of pictures are made equal,
			while keeping the vertical rhythm of text before and after the pictures.</li>
		<!--
		<li>Aligning to grids can sometimes be turned off for objects like tables,
			but then turned back on for the following text
			to the same grids as the one for the text before the objects.</li>
		-->
		<li>Layout lines are at the same position on every page in paged media.
			Keeping the position of the bottom line of a page has benefits for
			design and readability.
			This also improves the readability of duplex printing,
			two pages spreads,
			and displaying on slow display devices like e-ink.</li>
		<li>East Asian layouts require vertical rhythm
			more often than other scripts do,
			even in single column, non-paged media documents,
			as defined in [[JLREQ]].</li>
	</ul>

	There are several types of objects in a document
	that can break the vertical rhythm.
	Examples include lines with different sizes of text,
	pictures, and tables.

	<div class="figure">
		<img src="images/line-grid-multicol.png"
			width="480"
			alt="Vertical rhythm kept through pictures and different size of text in a multi-column document"
			/>
		<p class="caption">Vertical rhythm kept through pictures and different size of text in a multi-column document.</p>
	</div>

	<div class="example">
		<div class="sidefigure">
			<img src="images/line-grid-wrap.png"
				width="276"
				alt="Large text wraps within line grids"
				/>
			<p class="caption">Large text wraps within line grids.</p>
		</div>

		When a different size of text,
		such as a headings wraps,
		it is usually aligned to grids as a block
		and the lines within the block do not align.

		<br style="clear:both;"/>
	</div>

	<div id="ex-sidenote" class="figure">
		<div class="sidenote-block">
			<div class="sidenote-body">

				Sidenotes (and footnotes for that matter) are often set
				at a smaller size than the basic text.
				This smaller text should still line up with the basic text.
				Authors can try to achieve this effect
				by calculating appropriate font-size, line-height,
				and margins*, but lack the proper tools to get the baselines to align.
			</div>
			<div class="sidenote-note">

				Even if the author controls all this,
				the baselines won't align. And careful
				calculations can be thrown off by user stylesheets.
			</div>
		</div>
		<p class="caption">Sidenotes are set at a smaller size, and baselines don't align.</p>
	</div>

	<figure>
		<img src="images/sidenote-aligned.png"
			alt="sidenote rendering with aligned baselines"/>
		<figcaption>
			Sidenote with baselines aligned to the body text.
		</figcaption>
	</figure>

	<div id="ex-width" class="sidefigure">
		<img src="width-multiple-of-em.png"
			width="180" height="142"
			alt="East Asian layouts may require width be a multiple of em without fractions"
			/>
		<p class="caption">East Asian layouts may require width
			be a multiple of <em>em</em> without fractions.</p>
	</div>

	East Asian layouts may require grid-like features
	in inline progression direction as well.

	It is often desirable in East Asian layouts
	to make the line width
	a multiple of <em>em</em> without fractions.
	Because most East Asian characters have 1em advance
	and most East Asian documents are justified,
	this minimizes cases where justification needs to expand character spacing.

	This module provides the following capabilities:

	<ul>
		<li>Defining grids in the line progression direction.</li>
		<li>Controlling how lines and blocks align to the grids.</li>
	</ul>

	It is important to control these capabilities independently,
	so that, for example,
	aligning to grids can be turned off for tables,
	but can then be turned back on
	for aligning the following text to the grids.


<h3 id='placement'>
Module Interactions</h3>

	This module extends the line box model
	defined in [[!CSS2]] sections 9.4.2 and 10.8.

<h3 id='values'>
Values</h3>

	This specification follows the <a href="https://www.w3.org/TR/CSS21/about.html#property-defs">CSS property definition conventions</a> from [[!CSS2]].
	Value types not defined in this specification are defined in CSS Values & Units [[!CSS-VALUES-3]].
	Other CSS modules may expand the definitions of these value types.

	In addition to the property-specific values listed in their definitions,
	all properties defined in this specification
	also accept the <a>CSS-wide keywords</a> keywords as their property value.
	For readability they have not been repeated explicitly.

<h2 id='line-grid-property'>
Defining a <dfn>Line Grid</dfn>: the 'line-grid' property</h2>

	<pre class='propdef'>
		Name: line-grid
		Value: match-parent | create
		Initial: match-parent
		Applies to: block, flex and grid containers
		Inherited: no
		Percentages: N/A
		Computed Value: specified keyword
		Animation type: discrete
	</pre>

	Specifies whether this box creates
	a new baseline grid for its descendants
	or uses the same baseline grid as its parent.
	(Each box always has an associated <a>line grid</a>.
	However, whether a box or its contents snap
	to a <a>line grid</a> is determined
	by 'line-snap' and 'box-snap'.)

	The values of this property have the following meanings:

	<dl dfn-type="value" dfn-for="line-grid">
		<dt><dfn>match-parent</dfn>
		<dd>Box assumes the <a>line grid</a> of its parent
			if its <a>writing mode</a> is the same as its parent.
			If the box has a <a>writing mode</a>
			that is different than its parent,
			then the box creates a new <a>line grid</a>
			as ''create'' below.

		<dt><dfn>create</dfn>
		<dd>Box creates a new <a>line grid</a>
			using its own font and line layout settings,
			including any adjustment to the line height caused by the 'line-height-step' property.
			The <a>line grid</a> consists of a series
			of horizontal lines corresponding
			to all the baselines
			(alphabetic, text-top, text-bottom,
			mathematic, central, hanging, etc.)
			and to the line-over and line-under edges,
			positioned where they would fall
			if the contents of this element consisted entirely
			of line boxes filled with text (no sub-elements)
			using the first available font.
			If the box is paginated,
			the <a>line grid</a> is restarted on each page;
			since line boxes cannot be fragmented,
			no page begins with the bottom part of a line's grid.
	</dl>

	The initial containing block establishes a <a>line grid</a>
	using the font and line layout settings of the root element.
	If the root element (which has no parent)
	has a value of ''match-parent'',
	then it adopts the <a>line grid</a> of the initial containing block.

	In paged media,
	if the page context applying to a particular page
	specifies ''line-grid: create'',
	then a <a>line grid</a> for that page is established
	using the page context's font and line layout settings
	applied to the page box (including its margins),
	and for layout within that page the initial containing block
	is assumed to have adopted this <a>line grid</a>.

	<div class="example">
		Since the initial value of 'line-grid' is ''match-parent'',
		by default a <a>line grid</a> is automatically created
		using the root element's font and line height settings
		to fill the initial containing block / page area.
		When all pages of a document are the same size,
		this will result in a consistent <a>line grid</a> that exactly aligns
		to the top of each page area.

		It is sometimes appropriate
		for the <a>line grid</a> to begin with the content,
		after the root element's margins/padding/border
		and/or other header content.
		To do this, the root element (or some other element) can be told to establish the <a>line grid</a>.
		<pre>
			main { line-grid: create; }
		</pre>

		If this <a>line grid</a> is then paginated, the <a>line grid</a> will start at the top of each page area after the first.

		In other cases, it's desirable for
		all pages of a document share a common <a>line grid</a>,
		even if page margins, border and/or padding change.
		The following rule will instead fill the page box area
		<em>including</em> the page margin/padding/borders
		with the <a>line grid</a>:
		<pre>
			@page { line-grid: create; }
		</pre>

		As this <a>line grid</a> is paginated, it will start at the top of each page box after the first;
		however it might not align to the top of the page area
		(where the content would normally begin).
	</div>

	Issue: The names of these values is currently up for debate.
	Current suggestions for match-parent
	include match-parent and normal;
	those for create include create and new.

	<p class="note">The original proposal for line grids
	allowed an element to create a named grid.
	This property could still be extended to do this in the future.

	<p class="note">There might need to be an offset
	for more complicated designs.
	How to set this offset is problematic:
	usually it's not a fixed length,
	but the distance to clear some header content.
	This could be added to a later level of css-line-grid.

<h2 id='grid-snapping'>
Snapping to a Grid</h2>

	The 'line-snap' and 'box-snap' properties
	cause line boxes and block boxes to shift position
	in order to align to the <a>line grid</a>.
	Shifting boxes in this way affects layout –
	it is not merely a display translation.
	If a line box is shifted,
	then subsequent line boxes will be laid out
	using the new shifted position as input
	to their line stacking rules.
	If a block box is shifted,
	then subsequent boxes will be laid out
	using the new shifted position as input.

<h3 id='line-snap'>
Snapping Line Boxes: the 'line-snap' property</h3>

	<pre class='propdef'>
		Name: line-snap
		Value: none | baseline | contain
		Initial: none
		Applies to: <a>block container</a> elements
		Inherited: yes
		Percentages: N/A
		Computed Value: specified keyword
		Animation type: discrete
	</pre>

	This property applies to all the line boxes
	directly contained by the element,
	and, when not none,
	causes each line box to shift
	(usually downward, possibly by zero)
	until it snaps to the <a>line grid</a>
	specified by 'line-grid'.
	(The unshifted position is the position that
	would be determined by normal line stacking rules,
	with consideration of any new controls
	defined by other modules such as [[CSS3LINE]].)


	Values have the following meanings:

	<dl dfn-type="value" dfn-for="line-snap">
	<dt><dfn>none</dfn>
		<dd>Line boxes do not snap to the grid; they stack normally.
	<dt><dfn>baseline</dfn>
		<dd>The dominant baseline snaps with the matching baseline on the
		<a>line grid</a> applying to the element.
	<dt><dfn>contain</dfn>
		<dd>
			Two baselines are used to align the line box:
			the line box is snapped
			so that its <a href="https://drafts.csswg.org/css-inline/#central">central</a> baseline
			is centered between one
			of the line grid's <a href="https://drafts.csswg.org/css-inline/#text-over-edge">text-over-edge</a> baselines
			and a subsequent (but not necessarily consecutive)
			<a href="https://drafts.csswg.org/css-inline/#text-under-edge">text-under-edge</a> baseline.
	</dl>

	<div class="example">

	Line boxes almost always shift downward
	(towards the block-end direction)
	when snapping to a <a>line grid</a>.
	Here there are three lines
	with 20px line-height
	and line-snap:baseline
	that should snap to a 30px <a>line grid</a>.
	Each line box shifts down
	so that the baselines align
	with the grid lines.

		<figure style="float:left;">
			<img src="images/simple-no-snap.png"
				alt="line positions before snapping"/>
			<figcaption>
				Before line snapping
			</figcaption>
		</figure>
		<figure style="float:left;">
			<img src="images/simple-snapped.png"
				alt="line positions after snapping"/>
			<figcaption>
				After line snapping
			</figcaption>
		</figure>
		<p style="clear:left;">
			In the figures below,
			there are two additional lines
			from h3 elements with line-snap:none.
			These lines do not shift
			to align to the grid,
			but their positions can change
			based on the shifting of lines around them.
			In this example,
			lines 1 2 and 3 shift down
			to snap their baselines to the grid lines,
			and line B has normal line box placement
			just below the line above.
		</p>
		<figure style="float:left;">
			<img src="images/top-no-snap.png"
				alt="line positions before snapping"/>
			<figcaption>
				Before line snapping
			</figcaption>
		</figure>
		<figure style="float:left;">
			<img src="images/top-snapped.png"
				alt="line positions after snapping"/>
			<figcaption>
				After line snapping
			</figcaption>
		</figure>
	</div>

<h3 id='box-snap'>
Snapping Block Boxes: the 'box-snap' property</h3>

	Issue: This is a rough draft of trying to solve
	the box-snapping problem.

	Issue: Some optional box values
	(margin-box, border-box)
	could be added to the before and after values
	to allow snapping various box model edges
	to the <a>line grid</a>.

	Issue: An auto value could be useful -
	one that defaults to center,
	but snaps to before if it's the first block in a fragment container,
	and snaps to after if it's the last block in a fragment container.

	<pre class='propdef'>
		Name: box-snap
		Value: none | block-start | block-end | center | baseline | last-baseline
		Initial: none
		Applies to: block-level boxes and internal table elements except table cells
		Inherited: yes
		Percentages: N/A
		Computed Value: as specified
		Animation type: discrete
	</pre>

	Specifies how the block is snapped to the baseline grid.

	Values have the following meanings:

	<dl dfn-type="value" dfn-for="box-snap">
	<dt><dfn>none</dfn>
		<dd>The block is not snapped to any grid.</dd>
	<dt><dfn>block-start</dfn>
		<dd>The block-start edge is snapped to the nearest grid line.
	<dt><dfn>block-end</dfn>
		<dd>The block-end edge is snapped to the nearest grid line.
	<dt><dfn>center</dfn>
		<dd>The block is centered between
		one of the baseline grid's text-over baselines and a subsequent
		(but not necessarily consecutive) text-under baseline.
	<dt><dfn>baseline</dfn>
		<dd>The first line box's dominant baseline is snapped to the nearest grid line.
	<dt><dfn>last-baseline</dfn>
		<dd>The last line box's dominant baseline is snapped to the nearest grid line.
	</dl>

	Snapping block boxes always uses the <a>line grid</a> of the box parent
	(a block's own <a>line grid</a> has no effect on box snapping).
	If the box is an empty block that could be
	<a href="https://www.w3.org/TR/CSS21/box.html">collapsed through</a>,
	then this property has no effect. [[!CSS2]]

	To snap a block-level element to a grid line,
	take the unsnapped result of layout
	(including margin collapsing, clearance, etc.)
	as the block's initial position,
	then shift the block downward
	(towards the block-end direction)
	until it snaps to the <a>line grid</a>.

	For the ''block-start'' and ''block-end'' values,
	either the text-over-edge
	or text-under-edge baseline is chosen:
	whichever one is on the matching side
	of the central baseline.
	For example,
	when snapping the block-start edge
	in horizontal writing mode,
	the text-over-edge is chosen.
	In some cases the text-under-edge
	might be used instead for the block-start edge:
	for example,
	when the writing mode of the <a>line grid</a>
	doesn't match that of the affected element,
	or when due to the 'text-orientation' settings
	the text-under-edge corresponds to the block-start edge.

  <!--To snap an after border-box or half-border edge to a grid
	line, the effective content-box height is increased.-->

	When applied to table row group and table row boxes,
	'box-snap' only affects the before and after edges,
	and only if those edges are not
	at the beginning or end of the table, respectively.
	To snap a before edge on a table row or row group,
	the preceding row's height is increased.
	To snap an after edge on a table row or row group,
	the affected row's height is increased.

	When applied to table column group and table column boxes,
	'box-snap'only affects the start and end edges,
	and only if those edges are not
	at the start or end of the table, respectively.
	How the space is redistributed among columns
	to satisfy snapping constraints is not defined, however:
	<ul>
		<li>In an auto-sized table no column may be smaller than its minimum content width.
		<li>The resulting table must not exceed its original measure if it had a non-auto measure.
		<li>The adjusted widths must not cause the table to overflow its containing block any more than it would with ''box-snap: none''.
	</ul>

	To satisfy these constraints, some column edges may remain unsnapped.

<!--
	Line snapping for columns is processed from start to end: column widths
	are increased or decreased in order to snap the column edges.
	By preference, the widths are increased.
	However if the table has an explicit size or is auto-sized and the resulting
	table overflows its containing block, then columns whose widths were
	increased must, one by one, switch their snap to decreasing the column
	width, until the table reaches its specified size or
-->
<!--
	<p>For internal table elements in separated borders tables, ''margin-box''
	represents the edge at the midpoint of the border spacing. For collapsed
	tables, ''margin-box'' and ''border-box'' are treated as ''half-border''.
	When two coinciding edges have conflicting snap values, the later value
	wins.
-->

	<div class="issue">
		Going forward, we could add a 2-d box-snap
		with more controls
		and to also extend it to handle snapping
		to an explicit layout grid
		(not just the baseline grid).
		Current thoughts are for box-snap
		to break out into longhands;
		the current block-snap property should thus be a subset of its expected future values.
		Below is an outline of the structure.
		(This may, of course, change.)

		<pre>
			box-snap
			  +- block-snap
			  |     +- block-snap-edge
			  |     |     +- block-start-snap-edge: <<shape-box>>
			  |     |     +- block-end-snap-edge: <<shape-box>>
			  |     +- block-snap-align: <<content-position>>
			  |     +- block-snap-grid: text-grid | <<custom-ident>> /* name of grid/gridlines */
			  +- inline-snap
			        +- inline-snap-edge
			        |     +- inline-start-snap-edge: <<shape-box>>
			        |     +- inline-end-snap-edge: <<shape-box>>
			        +- inline-snap-align: <<content-position>>
			        +- inline-snap-grid: text-grid | <<custom-ident>> /* name of grid/gridlines */

			where <<shape-box>> includes
				margin-box | border-box | half-border-box | padding-box | content-box
			and <<content-position>> includes
				start | end | stretch | first-baseline | last-baseline | center
		</pre>
	</div>

<h2 id='snap-details'>
Snapping Details</h2>

<h3 id='alignment-interactions'>
Interacting with other alignments</h3>

	An element can have additional block layout constraints
	(such as centering or 'box-snap')
	that can complicate line snapping.
	In these cases,
	implementations must produce
	the same result as the steps below:

	<ul>
		<li>
			Line snap as if the layout constraint does not apply
		</li>
		<li>
			Apply the layout constraint to the block
		</li>
		<li>
			Adjust exactly one line box shift
			or space at the beginning or end of the block
			such that all of the snapping lines meet the <a>line grid</a>
			while maintaining the block layout constraint.
			The single adjustment chosen must be
			the smallest shift needed to achieve the result
			without causing overflow.
		</li>
	</ul>

	<div class="example">

		A block containing lines to snap
		might not be top-aligned within its container.
		In the figures below,
		the block containing the elements is centered.
		In a centered situation,
		you have to align baselines
		while maintaining centering.
		This can be done in two shift-and-center steps.

		First, shift the snapping lines
		as if the block was top-aligned
		(as in figure 9 above),
		then remove the shift
		for the very first snapping line.
		After removing the first shift,
		try centering the block.
		This is almost certain
		to throw the baseline alignment off.
		You can see one such result
		in the partial shifting figure below.

		Second, measure the distance
		from the first snapped line's baseline
		to the grid lines above and below,
		looking for the closest grid line
		to that baseline.

		If the closest grid line
		is in the block-start direction,
		then add space below the last line in the block
		equal to twice that distance.
		Then the block is centered again,
		which will align all of the snapped lines to the grid.

		<figure style="float:left;">
			<img src="images/first-center-part-snapped.png"
				alt="line positions at step 1"/>
			<figcaption>
				Partial shifting
			</figcaption>
		</figure>
		<figure style="float:left;">
			<img src="images/first-center-snapped.png"
				alt="line positions after full snapping"/>
			<figcaption>
				Full line snapping
			</figcaption>
		</figure>

		<p style="clear:left;">
			If the closest grid line
			is in the block-end direction,
			then the first snapped line is shifted downward
			by twice that distance.
			Then the block is centered again,
			which will again align all of the snapped lines to the grid.
		</p>
		<figure style="float:left;">
			<img src="images/second-center-part-snapped.png"
				alt="line positions at step 1"/>
			<figcaption>
				Partial shifting
			</figcaption>
		</figure>
		<figure style="float:left;">
			<img src="images/second-center-snapped.png"
				alt="line positions after full snapping"/>
			<figcaption>
				Full line snapping
			</figcaption>
		</figure>
	</div>

	<div class="example">

		An end-aligned block also uses two steps,
		but is simpler than the centered case.

		First, shift the snapping lines
		as if there were no end-alignment
		(as in figure 9 above),
		then end-align the block.

		Second, shift the entire block contents upwards
		until the last snapped line aligns to a grid line.
		In this example, the shift is very minor.

		<figure style="float:left;">
			<img src="images/bottom-no-snap.png"
				alt="line positions before snapping"/>
			<figcaption>
				Before snapping
			</figcaption>
		</figure>
		<figure style="float:left;">
			<img src="images/bottom-snapped.png"
				alt="line positions after snapping"/>
			<figcaption>
				After line snapping
			</figcaption>
		</figure>
	</div>

<h3 id='leading-adjustments'>
Leading adjustments</h3>

	In some cases lines of equal line height
	will not align perfectly to a baseline grid:
	this happens, for example,
	when fonts (of the same size)
	with different baseline tables
	are mixed on a line.
	In these cases,
	removing top half-leading can sometimes be
	the correct thing to do,
	but must be limited
	to the amount of baseline discrepancy
	so as not to trigger in other cases.

	Issue: Details to follow.
	It's complicated when you have
	more than one font-size or line-height
	in a single line box.

<h2 class=no-num id='acknowledgments'>Acknowledgments</h2>

	This module was made possible by the advice and contributions of
	Tab Atkins,
	Dave Cramer,
	Dave Hyatt,
	Bem Jones-Bey,
	Håkon Wium Lie,
	Shinyu Murakami,
	Liam Quin,
	and the CSS Working Group members.

<h2 class=no-num id='change-log'>
Change Log</h2>

<h3 class="no-num" id="20140916">
Since <a href="https://www.w3.org/TR/2014/WD-css-line-grid-1-20140916/">September 16th 2014</a></h3>
	<ul>
		<li>Refined ICB/root/@page grid creation</li>
		<li>Added flex and grid container applicability for line-grid</li>
		<li>Added normative text for line-snap examples</li>
		<li>Added @page and root element line grid handling</li>
		<li>Made the dependency on line-height-step explicit</li>
		<li>''match-parent'' creates new line grid for orthoganal flows</li>
	</ul>

<h3 class="no-num" id="20140403">
Since <a href="https://www.w3.org/TR/2014/WD-css-line-grid-1-20140403/">April 3rd 2014</a></h3>
	<ul>
		<li>Added line-snapping examples</li>
		<li>Modified box-snap values</li>
		<li>Removed line-slack</li>
		<li>Removed grid offset</li>
		<li>Removed grid units</li>
		<li>Removed 2d box snapping</li>
		<li>Removed rounding functions</li>
	</ul>
