<!--
	This document is formatted using Bikeshed.
	Roughly speaking, it is a Markdown preprocessor,
	with additional functionality for cross-spec autolinking,
	automatic generation of indexes/ToC/etc,
	and many other features.
	See https://tabatkins.github.io/bikeshed/ for detailed documentation.

	When making edits, please respect the following coding style:
	- Tabs for indentation, spaces for alignment.
	- Semantic line breaks: at phrases boundaries, each line < ~80ch
	- Indent the entire spec one level except for headings.
	- Line break after opening heading tag, so heading text
	  is easy to pick out when scanning the source.
	- Empty lines between blocks.
	- Indent contents of block-level HTML elements
	  (except <p>, which we usually imply via Markdown formatting
	  and otherwise leave inlined at the start of the paragraph).
	  Definitely leave a break and indent
	  after any block start tag with attributes, though.
	- No optional end tags.
	- Use manual IDs so that IDs remain stable as you adjust the heading text;
	  add old IDs (via empty elements with IDs, or e.g. Bikeshed's oldids attribute)
	  when removing or changing IDs so that links to your spec don't break.
-->
<pre class='metadata'>
Title: W3C Process Document
Status: w3c/ED
ED: https://w3c.github.io/w3process/
TR: https://www.w3.org/Consortium/Process/
Previous Version: https://www.w3.org/2019/Process-20190301/
Previous Version: https://www.w3.org/2018/Process-20180201/
Previous Version: https://www.w3.org/2017/Process-20170301/
Editor: Florian Rivoal, Invited Expert, https://florian.rivoal.net/
Former Editor: Natasha Rooney, Invited Expert
Former Editor: Charles McCathie Nevile, Yandex, http://yandex.com
Former Editor: Ian Jacobs, W3C, https://www.w3.org/
Level: none
Shortname: w3process
Abstract:
	The mission of the World Wide Web Consortium (<abbr>W3C</abbr>) is to lead the World Wide Web to its full potential
	by developing common protocols that promote its evolution and ensure its interoperability.
	The W3C Process Document describes the organizational structure of the W3C and processes,
	responsibilities and functions that enable W3C to accomplish its mission.
	This document does not describe the internal workings of the Team.

	For more information about the W3C mission and the history of W3C,
	please refer to <a href="https://www.w3.org/Consortium/">About W3C</a>.
Status Text:
	This document is based on the <a href="https://www.w3.org/2017/Process-20190301/">1 March 2019 Process</a>,
	which is the currently operative W3C Process.
</pre>
<!--
<pre class=biblio>
{
	"PATENT-POLICY": {
		"href": "https://patent-policy.rivoal.net",
		"title": "W3C IPR Policy (experimental version)"
	}
}
</pre>
-->

<style>
.about { margin-left: 3em; margin-right: 3em; font-size: .83em}
table { margin-left: auto; margin-right: auto }
.diagram { text-align: center; margin: 2.5em 0 }
.issue::before {content: "Issue: "}
.issue {border: 2px dashed red; background-color: #ffa;}
.issue .issue {background-color: #fcc;}
.rfc2119 {font-variant:small-caps}

/*for the SVG - navigation highlighting */

:focus path,
:focus polygon,
:focus ellipse {
	stroke-width: 4px;
}
g g:hover path,
g g:hover polygon,
g g:hover ellipse {
	stroke-width: 4px;
}
:focus text,
g g:hover text {
	stroke: black;
	stroke-width: .5px;
}

@media print {
	a[data-ref]::after { content: " [" attr(data-ref) "]"; }
}

.table-borders {
	border-collapse: collapse;
}
.table-borders td,
.table-borders th {
	border: 1px solid black;
	padding: 0.5ex;
}

</style>
<!--[if lt IE 9]><script src='undefined://www.w3.org/2008/site/js/html5shiv.js'></script><![endif]-->

<h2 class="no-num no-toc no-ref" id="pp">
Relation of Process Document to Patent Policy</h2>

	W3C Members' attention is called to the fact
	that provisions of the Process Document are binding on Members
	per the <a href="https://www.w3.org/Consortium/Agreement/Member-Agreement">Membership Agreement</a> [[MEMBER-AGREEMENT]].
	The W3C Patent Policy</a> [[!PATENT-POLICY]]
	is incorporated by normative reference as a part of the Process Document,
	and is thus equally binding.

	The Patent Policy places additional obligations on Members, Team, and other participants in W3C.
	The Process Document does not restate those requirements but includes references to them.
	The Process Document and Patent Policy have been designed to allow them to evolve independently.

	In the Process Document, the term “participant” refers to an individual, not an organization.

<h2 class="no-num no-toc no-ref" id=terms>
Conformance and specialized terms</h2>

	The terms
	<em class="rfc2119">must</em>,
	<em class="rfc2119">must not</em>,
	<em class="rfc2119">should</em>,
	<em class="rfc2119">should not</em>,
	<em class="rfc2119">required</em>,
	and <em class="rfc2119">may</em>
	are used in accordance with <a href="https://www.ietf.org/rfc/rfc2119.txt">RFC 2119</a>.
	The term <dfn id="not-required" noexport><em class="rfc2119">not required</em></dfn>
	is equivalent to the term <em class="rfc2119">may</em>
	as defined in RFC2119
	[[!RFC2119]].

	Some terms have been capitalized in this document (and in other W3C materials)
	to indicate that they are entities with special relevance to the W3C Process.
	These terms are defined within this document,
	and readers are reminded that the ordinary English definitions are insufficient
	for the purpose of understanding this document.

<nav data-fill-with="table-of-contents" id="toc"></nav>
<main>

<h2 id="Intro">
Introduction</h2>

	Most W3C work revolves around the standardization of Web technologies.
	To accomplish this work, W3C follows processes that promote the development of high-quality standards
	based on the <a href="#Consensus">consensus</a> of the Membership, Team, and public.
	W3C processes promote fairness, responsiveness, and progress:
	all facets of the W3C mission.
	This document describes the processes W3C follows in pursuit of its mission.

	Here is a general overview of how W3C standardizes a Web technology.
	In many cases, the goal of this work is a [=W3C Recommendation=]--
	a Web standard.

	<ol>
		<li>
			People generate interest in a particular topic.
			For instance, Members express interest in the form of <a href="#Submission">Member Submissions</a>,
			and the <a href="#Team">Team</a> monitors work inside and outside of W3C for signs of interest.
			Also, W3C is likely to organize a <a href="#GAEvents">Workshop</a> to bring people together
			to discuss topics that interest the W3C community.

		<li>
			When there is enough interest in a topic
			(e.g., after a successful Workshop and/or discussion on an <a href="#ACCommunication">Advisory Committee mailing list</a>),
			the Director announces the development of a proposal
			for one or more new <a href="#WGCharterDevelopment">Interest Group or Working Group charters</a>,
			depending on the breadth of the topic of interest.
			W3C Members <a href="#CharterReview">review</a> the proposed charters.
			When there is support within W3C for investing resources in the topic of interest,
			the Director approves the group(s) and they begin their work.

		<li>
			There are three types of Working Group participants:
			<a href="#member-rep">Member representatives</a>,
			<a href="#invited-expert-wg">Invited Experts</a>,
			and <a href="#Team">Team representatives</a>.
			Team representatives both contribute to the technical work
			and help ensure the group's proper integration with the rest of W3C.
			The <a href="#WGCharter">Working Group charter</a> sets expectations about each group's deliverables
			(e.g., <a href="#Reports">technical reports</a>, test suites, and tutorials).

		<li>
			Working Groups generally create specifications and guidelines
			that undergo cycles of revision and review
			as they <a href="#rec-advance">advance to W3C Recommendation</a> status.
			The W3C process for producing these technical reports
			includes significant review by the Members and public,
			and requirements that the Working Group be able to show implementation and interoperability experience.
			At the end of the process,
			the Advisory Committee reviews the mature technical report,
			and if there is support,
			W3C publishes it as a <a href="#RecsW3C">Recommendation</a>.
	</ol>

	The Process Document promotes the goals of quality and fairness in technical decisions
	by encouraging <a href="#Consensus">consensus</a>,
	requiring reviews (by both Members and public) as part of the <a href="#Reports">technical report development process</a>,
	and through an <a href="#ACAppeal">Advisory Committee Appeal process</a>.

	The other sections of the Process Document:

	<ol>
		<li>
			set forth <a href="#Policies">policies</a> for participation in W3C groups,

		<li>
			establish two permanent groups within W3C:
			the <a href="#TAG">Technical Architecture Group (TAG)</a>,
			to help resolve Consortium-wide technical issues;
			and the <a href="#AB">Advisory Board (AB)</a>,
			to help resolve Consortium-wide non-technical issues,
			and to manage the <a href="#GAProcess">evolution of the W3C process</a>,
			and

		<li>
			describe other interactions between the <a href="#Members">Members</a>
			(as represented by the <a href="#AC">W3C Advisory Committee</a>),
			the Team,
			and the general public.
	</ol>

	The W3C also operates Community and Business Groups,
	which are separately described in <a href="https://www.w3.org/community/about/agreements/">their own process document</a> [[BG-CG]].

<h2 id="Organization">
Members, Advisory Committee, Team, Advisory Board, Technical Architecture Group</h2>

	W3C's mission is to lead the Web to its full potential.
	W3C <a href="#Members">Member</a> organizations provide resources to this end,
	and the W3C <a href="#Team">Team</a> provides the technical leadership
	and organization to coordinate the effort.

<h3 id="Members">
Members</h3>

	W3C Members are primarily represented in W3C processes as follows:

	<ol>
		<li>
			The <dfn export lt="Advisory Committee|AC">Advisory Committee</dfn>
			is composed of one <dfn export lt="Advisory Committee representative| AC representative">representative</dfn>
			from each Member organization
			(refer to the [=Member-only=] list
			of <a href="https://www.w3.org/Member/ACList">current Advisory Committee representatives</a>. [[CURRENT-AC]])
			The Advisory Committee:

			<ul>
				<li>
					reviews plans for W3C at each <a href="#ACMeetings">Advisory Committee meeting</a>;

				<li>
					reviews formal proposals from the W3C Director:
					<a href="#CharterReview">Charter Proposals</a>,
					<a href="#RecsPR">Proposed Recommendations</a>,
					and <a href="#GAProcess">Proposed Process Documents</a>.

				<li>
					elects the <a href="#AB">Advisory Board</a> participants other than the Advisory Board Chair.

				<li>
					elects a majority (6) of the participants on the <a href="#TAG">Technical Architecture Group</a>.
			</ul>

			[=Advisory Committee representatives=] <em class="rfc2119">may</em> initiate
			an <a href="#ACAppeal">Advisory Committee Appeal</a> in some cases described in this document.

		<li>
			  Representatives of Member organizations participate
			  in <a href="#GAGeneral">Working Groups and Interest Groups</a> and
			  author and review <a href="#Reports">technical reports</a>.
	</ol>

	<p id="MemberSubscription">W3C membership is open to all entities,
	as described in “<a href="https://www.w3.org/Consortium/join">How to Join W3C</a>” [[JOIN]];
	(refer to the public list of <a href="https://www.w3.org/Consortium/Member/List">current W3C Members</a> [[MEMBER-LIST]]).
	Organizations subscribe according to the <a href="https://www.w3.org/Consortium/Agreement/Member-Agreement">Membership Agreement</a> [[MEMBER-AGREEMENT]].
	The <a href="#Team">Team</a> <em class="rfc2119">must</em> ensure
	that Member participation agreements remain <a href="#Team-only">Team-only</a>
	and that no Member receives preferential treatment within W3C.

	<p id="IndividualParticipation">While W3C does not have a class of membership tailored to individuals,
	individuals <em class="rfc2119">may</em> join W3C.
	Restrictions pertaining to <a href="#MemberRelated">related Members</a> apply
	when the individual also <a href="#member-rep">represents</a> another W3C Member.

<h4 id="MemberBenefits">
Rights of Members</h4>

	Each Member organization enjoys the following rights and benefits:

	<ul>
		<li>
			A seat on the <a href="#AC">Advisory Committee</a>;

		<li>
			Access to <a href="#Member-only">Member-only</a> information;

		<li>
			The <a href="#Submission">Member Submission</a> process;

		<li>
			Use of the W3C Member logo on promotional material and to publicize the Member's participation in W3C.
			For more information, please refer to the Member logo usage policy
			described in the <a href="https://www.w3.org/Member/Intro">New Member Orientation</a> [[INTRO]].
	</ul>

	Furthermore, subject to further restrictions included in the Member Agreement,
	representatives of Member organizations participate in W3C as follows:

	<ul>
		<li>
			In <a href="#GAGeneral">Working Groups and Interest Groups</a>.

		<li>
			In <a href="#GAEvents">Workshops and Symposia</a>;

		<li>
			On the Team, as <a href="#fellows">W3C Fellows</a>.
	</ul>

	The rights and benefits of W3C membership are contingent upon conformance to the processes described in this document.
	The vast majority of W3C Members faithfully follow the spirit as well as the letter of these processes.
	When serious and/or repeated violations do occur,
	and repeated attempts to address these violations have not resolved the situation,
	the [=Director=] <em class="rfc2119">may</em> take disciplinary action.
	Arbitration in the case of further disagreement is governed by paragraph 19 of the Membership Agreement [[MEMBER-AGREEMENT]].
	Refer to the <a href="https://www.w3.org/2002/09/discipline">Guidelines for Disciplinary Action</a> [[DISCIPLINARY-GL]].

<h4 id="RelatedAndConsortiumMembers">
Member Consortia and Related Members</h4>

<h5 id="MemberConsortia">
Membership Consortia</h5>

	A “<dfn id=fdn-member-consortium lt="Member Consortium | Member Consortia">Member Consortium</dfn>” means a consortium,
	user society,
	or association of two or more individuals,
	companies,
	organizations or governments,
	or any combination of these entities
	which has the purpose of participating in a common activity
	or pooling resources to achieve a common goal other than participation in,
	or achieving certain goals in,
	W3C.
	A joint-stock corporation or similar entity is not a [=Member Consortium=]
	merely because it has shareholders or stockholders.
	If it is not clear whether a prospective Member qualifies as a [=Member Consortium=],
	the Director may reasonably make the determination.
	For a [=Member Consortium=], the rights and privileges of W3C Membership
	described in the W3C Process Document extend to the [=Member Consortium=]'s paid staff
	and Advisory Committee representative.

	[=Member Consortia=] <em class="rfc2119">may</em> also designate
	up to four (or more at the Team's discretion) individuals
	who, though not employed by the organization,
	<em class="rfc2119">may</em> exercise the rights of <a href="#member-rep">Member representatives</a>.

	For [=Member Consortia=] that have individual people as members,
	these individuals <em class="rfc2119">must</em> disclose their employment affiliation
	when participating in W3C work.
	Provisions for <a href="#MemberRelated">related Members</a> apply.
	Furthermore, these individuals <em class="rfc2119">must</em> represent the broad interests of the W3C Member organization
	and not the particular interests of their employers.

	For Member Consortia that have organizations as Members,
	all such designated representatives <em class="rfc2119">must</em> be an official representative of the Member organization
	(e.g. a Committee or Task Force Chairperson)
	and <em class="rfc2119">must</em> disclose their employment affiliation when participating in W3C work.
	Provisions for <a href="#MemberRelated">related Members</a> apply.
	Furthermore, these individuals <em class="rfc2119">must</em> represent the broad interests of the W3C Member organization
	and not the particular interests of their employers.

	For all representatives of a Member Consortium,
	IPR commitments are made on behalf of the Member Consortium,
	unless a further IPR commitment is made by the individuals' employers.

<h5 id="MemberRelated">
Related Members</h5>

	In the interest of ensuring the integrity of the consensus process,
	Member involvement in some of the processes in this document is affected by related Member status.
	As used herein, two Members are <dfn export lt="Related Member">related</dfn> if:

	<ol>
		<li>
			Either Member is a [=subsidiary=] of the other, or

		<li>
			Both Members are [=subsidiaries=] of a common entity, or

		<li>
			The Members have an employment contract or consulting contract that affects W3C participation.
	</ol>

	A <dfn>subsidiary</dfn> is an organization of which effective control and/or majority ownership rests with another,
	single organization.

	[=Related Members=] <em class="rfc2119">must</em> disclose these relationships
	according to the mechanisms described in the <a href="https://www.w3.org/Member/Intro">New Member Orientation</a> [[INTRO]].

<h4 id="AC">
Advisory Committee (AC)</h4>

	When an organization joins W3C
	(see “<a href="https://www.w3.org/Consortium/join">How to Join W3C</a>” [[JOIN]]),
	it <em class="rfc2119">must</em> name its Advisory Committee representative as part of the Membership Agreement.
	The <a href="https://www.w3.org/Member/Intro">New Member Orientation</a> [[INTRO]]
	explains how to subscribe or unsubscribe to Advisory Committee mailing lists,
	provides information about <a href="#ACMeetings">Advisory Committee Meetings</a>,
	explains how to name a new [=Advisory Committee representative=],
	and more.
	[=Advisory Committee representatives=] <em class="rfc2119">must</em> follow the <a href="#coi">conflict of interest policy</a>
	by disclosing information according to the mechanisms described in the New Member Orientation.
	See also the additional roles of [=Advisory Committee representatives=]
	described in the W3C Patent Policy [[PATENT-POLICY]].

	Additional information for Members is available at the <a href="https://www.w3.org/Member/">Member Web site</a> [[MEMBER-HP]].

<h5 id="ACCommunication">
Advisory Committee Mailing Lists</h5>

	The [=Team=] <em class="rfc2119">must</em> provide two mailing lists for use by the [=Advisory Committee=]:

	<ol>
		<li>
			One for official announcements (e.g., those required by this document) from the [=Team=] to the [=Advisory Committee=].
			This list is read-only for Advisory Committee representatives.

		<li>
			One for discussion among [=Advisory Committee representatives=].
			Though this list is primarily for Advisory Committee representatives,
			the [=Team=] <em class="rfc2119">must</em> monitor discussion
			and <em class="rfc2119">should</em> participate in discussion when appropriate.
			Ongoing detailed discussions <em class="rfc2119">should</em> be moved to other appropriate lists
			(new or existing, such as a mailing list created for a <a href="#GAEvents">Workshop</a>).
	</ol>

	An [=Advisory Committee representative=] <em class="rfc2119">may</em> request
	that additional individuals from their organization be subscribed to these lists.
	Failure to contain distribution internally
	<em class="rfc2119">may</em> result in suspension of additional email addresses,
	at the discretion of the Team.

<h5 id="ACMeetings">
Advisory Committee Meetings</h5>

	The [=Team=] organizes a <a href="#ftf-meeting">face-to-face meeting</a> for the [=Advisory Committee=]
	<span class="time-interval">twice a year</span>.
	The [=Team=] appoints the Chair of these meetings (generally the CEO).
	At each Advisory Committee meeting,
	the Team <em class="rfc2119">should</em> provide an update to the Advisory Committee about:

	<dl>
		<dt><em>Resources</em>
		<dd>
			<ul>
				<li>
					The number of W3C Members at each level.

				<li>
					An overview of the financial status of W3C.
			</ul>

		<dt><em>Allocations</em>
		<dd>
			<ul>
				<li>
					The allocation of the annual budget, including size of the Team and their approximate deployment.

				<li>
					A list of all activities (including but not limited to Working and Interest Groups)
					and brief status statement about each,
					in particular those started or terminated since the previous Advisory Committee meeting.

				<li>
					The allocation of resources to pursuing <a href="#Liaisons">liaisons</a> with other organizations.
			</ul>
		</dd>
	</dl>

	Each Member organization <em class="rfc2119">should</em> send one <a href="#member-rep">representative</a>
	to each Advisory Committee Meeting.
	In exceptional circumstances
	(e.g., during a period of transition between representatives from an organization),
	the meeting Chair <em class="rfc2119">may</em> allow a Member organization to send two representatives to a meeting.

	The [=Team=] <em class="rfc2119">must</em> announce the date and location of each Advisory Committee meeting
	no later than at the end of the previous meeting;
	<span class="time-interval">one year's</span> notice is preferred.
	The Team <em class="rfc2119">must</em> announce the region of each Advisory Committee meeting
	at least <span class="time-interval">one year</span> in advance.

	More information about <a href="https://www.w3.org/Member/Meeting/">Advisory Committee meetings</a> [[AC-MEETING]]
	is available at the Member Web site.

<h3 id="Team">
The W3C Team</h3>

	The <dfn export lt="Team | W3C Team">Team</dfn> consists of
	the Director,
	CEO,
	W3C paid staff,
	unpaid interns,
	and W3C Fellows.
	<dfn id="fellows">W3C Fellows</dfn> are Member employees working as part of the Team;
	see the <a href="https://www.w3.org/Consortium/Recruitment/Fellows">W3C Fellows Program</a> [[FELLOWS]].
	The Team provides technical leadership about Web technologies,
	organizes and manages W3C activities to reach goals
	within practical constraints (such as resources available),
	and communicates with the Members and the public
	about the Web and W3C technologies.

	The [=Director=] and CEO <em class="rfc2119">may</em> delegate responsibility
	(generally to other individuals in the Team)
	for any of their roles described in this document,
	except <a href="#tag-participation">participation in the TAG</a>.</p>

	The <dfn id="def-Director" export lt="Director|W3C Director">Director</dfn> is the lead technical architect at W3C,
	whose responsibilities are identified throughout this document in relevant places.
	Some key ones include:
	assessing <a href="#def-Consensus" id="DirectorDecision">consensus</a> within W3C for architectural choices,
	publication of <a href="#Reports">technical reports</a>,
	and chartering new Groups;
	appointing group <a href="#GeneralChairs">Chairs</a>,
	adjudicating as "tie-breaker" for [=Group decision appeals=],
	and deciding on the outcome of formal objections;
	the Director is generally Chair of the <a href="#TAG">TAG</a>.

	Team administrative information such as Team salaries,
	detailed budgeting,
	and other business decisions
	are <a href="#Team-only">Team-only</a>,
	subject to oversight by the [=Host institutions=].

	Note: W3C is not currently incorporated.
	For legal contracts, W3C is represented by four <dfn id="hosts" lt="Hosts|Host institutions">“Host” institutions</dfn>:
	Beihang University,
	the European Research Consortium for Informatics and Mathematics (<abbr>ERCIM</abbr>),
	Keio University,
	and the Massachusetts Institute of Technology (<abbr>MIT</abbr>).
	Within W3C, the Host institutions are governed by hosting agreements;
	the [=Hosts=] themselves are not W3C Members.

<h3 id="AB">
Advisory Board (AB)</h3>

	Created in March 1998,
	the <dfn export lt="Advisory Board|AB">Advisory Board</dfn> provides ongoing guidance to the Team
	on issues of strategy,
	management,
	legal matters,
	process,
	and conflict resolution.
	The Advisory Board also serves the Members
	by tracking issues raised between Advisory Committee meetings,
	soliciting Member comments on such issues,
	and proposing actions to resolve these issues.
	The Advisory Board manages the <a href="#GAProcess">evolution of the Process Document</a>.
	The Advisory Board hears a <a href="#SubmissionNo">Submission Appeal</a>
	when a Member Submission is rejected
	for reasons unrelated to Web architecture;
	see also the <a href="#TAG">TAG</a>.

	The [=Advisory Board=] is <strong>not</strong> a board of directors
	and has no decision-making authority within W3C;
	its role is strictly advisory.

	The [=Team=] <em class="rfc2119">must</em> make available a mailing list,
	confidential to the Advisory Board and Team,
	for the Advisory Board to use for its communication.

	The [=Advisory Board=] <em class="rfc2119">should</em> send a summary of each of its meetings
	to the Advisory Committee and other group Chairs.
	The Advisory Board <em class="rfc2119">should</em> also report on its activities
	at each <a href="#ACMeetings">Advisory Committee meeting</a>.

	Details about the Advisory Board
	(e.g., the list of Advisory Board participants,
	mailing list information, and summaries of Advisory Board meetings)
	are available at the <a href="https://www.w3.org/2002/ab/">Advisory Board home page</a> [[AB-HP]].

<h4 id="ABParticipation">
Advisory Board Participation</h4>

	The [=Advisory Board=] consists of nine to eleven elected participants and a Chair.
	The [=Team=] appoints the Chair of the <a href="#AB">Advisory Board</a>,
	who is generally the CEO.
	The team also appoints a <a href="https://www.w3.org/Guide/staff-contact">Team Contact</a> for the [=AB=],
	as described in <a href="#ReqsAllGroups"></a>.

	The remaining nine to eleven [=Advisory Board=] participants are elected by the W3C [=Advisory Committee=]
	following the <a href="#AB-TAG-elections">AB/TAG nomination and election process</a>.

	With the exception of the Chair,
	the terms of all Advisory Board participants are for <span class="time-interval">two years</span>.
	Terms are staggered so that each year,
	either five or six terms expire.
	If an individual is elected to fill an incomplete term,
	that individual's term ends at the normal expiration date of that term.
	Regular Advisory Board terms begin on 1 July and end on 30 June.</p>

<h3 id="TAG">
Technical Architecture Group (TAG)</h3>

	Created in February 2001,
	the mission of the <dfn export lt="Technical Architecture Group|TAG">TAG</dfn> is stewardship of the Web architecture.
	There are three aspects to this mission:

	<ol>
		<li>
			to document and build consensus around principles of Web architecture
			and to interpret and clarify these principles when necessary;

		<li>
			to resolve issues involving general Web architecture brought to the TAG;

		<li>
			to help coordinate cross-technology architecture developments inside and outside W3C.
	</ol>

	The [=TAG=] hears a <a href="#SubmissionNo">Submission Appeal</a>
	when a Member Submission is rejected for reasons related to Web architecture;
	see also the <a href="#AB">Advisory Board</a>.

	The [=TAG=]'s scope is limited to technical issues about Web architecture.
	The TAG <em class="rfc2119">should not</em> consider
	administrative,
	process,
	or organizational policy issues of W3C,
	which are generally addressed by
	the W3C Advisory Committee,
	Advisory Board,
	and Team.
	Please refer to the <a href="https://www.w3.org/2004/10/27-tag-charter.html">TAG charter</a> [[TAG-CHARTER]]
	for more information about the background and scope of the TAG,
	and the expected qualifications of TAG participants.

	The [=Team=] <em class="rfc2119">must</em> make available two mailing lists for the TAG:

	<ul>
		<li>
			a public discussion (not just input) list for issues of Web architecture.
			The [=TAG=] will conduct its public business on this list.

		<li>
			a <a href="#Member-only">Member-only</a> list for discussions within the TAG
			and for requests to the TAG that,
			for whatever reason, cannot be made on the public list.
	</ul>

	The [=TAG=] <em class="rfc2119">may</em> also request the creation of additional topic-specific, public mailing lists.
	For some TAG discussions (e.g.,  a <a href="#SubmissionNo">Submission Appeal</a>),
	the TAG <em class="rfc2119">may</em> use a list that will be <a href="#Member-only">Member-only</a>.

	The [=TAG=] <em class="rfc2119">should</em> send a summary of each of its <a href="#GeneralMeetings">meetings</a>
	to the Advisory Committee and other group Chairs.
	The [=TAG=] <em class="rfc2119">should</em> also report on its activities at each <a href="#ACMeetings">Advisory Committee meeting</a>.

	When the [=TAG=] votes to resolve an issue,
	each TAG participant
	(whether appointed, elected, or the Chair)
	has one vote;
	see also the section on <a href="https://www.w3.org/2004/10/27-tag-charter.html#Voting">voting in the TAG charter</a> [[TAG-CHARTER]]
	and the general section on <a href="#Votes">votes</a> in this Process Document.

	Details about the [=TAG=]
	(e.g., the list of TAG participants, mailing list information, and summaries of TAG meetings)
	are available at the <a href="https://www.w3.org/2001/tag/">TAG home page</a> [[TAG-HP]].

<h4 id="tag-participation">
Technical Architecture Group Participation</h4>

	The [=TAG=] consists of:

	<ul>
		<li>
			Tim Berners-Lee, who is a life member;

		<li>
			The [=Director=],
			sitting <i>ex officio</i>;

		<li>
			Three participants appointed by the [=Director=];

		<li>
			Six participants elected by the [=Advisory Committee=]
			following the <a href="#AB-TAG-elections">AB/TAG nomination and election process</a>.
	</ul>

	The [=Team=] appoints the Chair of the TAG,
	who <em class="rfc2119">must</em> be one of the participants.
	The team also appoints a <a href="https://www.w3.org/Guide/teamcontact/role.html">Team Contact</a> [[TEAM-CONTACT]] for the TAG,
	as described in <a href="#ReqsAllGroups"></a>.

	The terms of elected and Director-appointed TAG participants are for <span class="time-interval">two years</span>.
	Terms are staggered so that each year three elected terms,
	and either one or two appointed terms expire.
	If an individual is appointed or elected to fill an incomplete term,
	that individual's term ends at the normal expiration date of that term.
	Regular TAG terms begin on 1 February and end on 31 January.

	The [=Director=] <em class="rfc2119">may</em> announce the appointed participants
	after the results for the Advisory Committee election of participants have been announced.

<h3 id="AB-TAG-participation">
Advisory Board and Technical Architecture Group Participation</h3>

	[=Advisory Board=] and [=TAG=] participants have a special role within W3C:
	they are elected by the Membership and appointed by the Director
	with the expectation that they will use their best judgment
	to find the best solutions for the Web,
	not just for any particular network,
	technology,
	vendor,
	or user.
	Advisory Board and TAG participants are expected to participate regularly and fully.
	Advisory Board and TAG participants <em class="rfc2119">should</em> attend <a href="#ACMeetings">Advisory Committee meetings</a>.

	Individuals elected or appointed to the Advisory Board or TAG act in their personal capacity,
	to serve the needs of the W3C membership as a whole,
	and the Web community.
	Whether they are Member representatives or Invited Experts,
	their activities in those roles are separate and distinct from their activities on the Advisory Board or TAG.

	An individual participates on the Advisory Board or TAG
	from the moment the individual's term begins until the seat is <a href="#AB-TAG-vacated">vacated</a>
	(e.g. because the term ends).
	Although Advisory Board and TAG participants do not advocate for the commercial interests of their employers,
	their participation does carry the responsibilities associated with Member representation,
	Invited Expert status,
	or Team representation
	(as described in the section on the <a href="#AB-TAG-elections">AB/TAG nomination and election process</a>).

	Participation in the TAG or AB is afforded to the specific individuals elected or appointed to those positions,
	and a participant's seat <em class="rfc2119">must not</em> be delegated to any other person.

<h4 id="AB-TAG-constraints">
Advisory Board and Technical Architecture Group Participation Constraints</h4>

	Given the few seats available on the [=Advisory Board=] and the [=TAG=],
	and in order to ensure that the diversity of W3C Members is represented:

	<ul>
		<li>
			Two participants with the same [=primary affiliation=] per <a href="#AB-TAG-elections"></a>
			must not both occupy elected seats on the TAG
			except when this is caused by a change of affiliation of an existing participant.
			At the completion of the next regularly scheduled election for the TAG,
			the organization <em class="rfc2119">must</em> have returned to having at most one seat.

		<li>
			Two participants with the same [=primary affiliation=] per <a href="#AB-TAG-elections"></a>
			must not both occupy elected seats on the AB.
			If, for whatever reason, these constraints are not satisfied
			(e.g., because an [=AB=] participant changes jobs),
			one participant <em class="rfc2119">must</em> cease [=AB=] participation
			until the situation has been resolved.
			If after <span class="time-interval">30 days</span> the situation has not been resolved,
			the Chair will apply
			the <a href="#random">verifiable random selection procedure</a> described below
			to choose one person for continued participation and declare the other seat(s) vacant.
		<li>
			An individual <em class="rfc2119">must not</em> participate on both the TAG and the AB.
	</ul>

<h4 id="AB-TAG-elections">
Advisory Board and Technical Architecture Group Elections</h4>

	The [=Advisory Board=] and a portion of the [=Technical Architecture Group=] are elected by the [=Advisory Committee=],
	using a Single Transferable Vote system.
	An election begins when the Team sends a Call for Nominations to the Advisory Committee.
	Any Call for Nominations specifies the minimum and maximum number of available seats,
	the deadline for nominations,
	details about the specific vote tabulation system selected by the Team for the election,
	and operational information such as how to nominate a candidate.
	The [=Team=] <em class="rfc2119">may</em> modify the tabulation system after the Call for Nominations
	but <em class="rfc2119">must</em> stabilize it no later than the Call for Votes.
	The [=Director=] <em class="rfc2119">should</em> announce appointments
	no later than the start of a nomination period as part of the Call for Nominations.

	In the case of regularly scheduled elections of the [=TAG=],
	the minimum and maximum number of avaiable seats are the same:
	the 3 seats of the terms expiring that year,
	plus the number of other seats that are vacant or will be vacant by the time the newly elected members take their seats.

	In the case of regularly scheduled elections of the [=AB=],
	the minimum and maximum number of avaiable seats differ:
	The maximum number is the 5 or 6 seats of the terms expiring that year,
	plus the number of other seats that are vacant or will be vacant by the time the newly elected members take their seats;
	the minimum number is such that when added to the occupied seats from the prior year,
	the minimum size of the AB (9) is reached.

	Each Member (or group of [=related Members=])
	<em class="rfc2119">may</em> nominate one individual.
	A nomination <em class="rfc2119">must</em> be made with the consent of the nominee.
	In order for an individual to be nominated as a Member representative,
	the individual <em class="rfc2119">must</em> qualify for <a href="#member-rep">Member representation</a>
	and the Member's [=Advisory Committee representative=] <em class="rfc2119">must</em> include in the nomination
	the (same) <a href="#member-rep-info">information required for a Member representative in a Working Group</a>.
	In order for an individual to be nominated as an Invited Expert,
	the individual <em class="rfc2119">must</em> provide
	the (same) <a href="#inv-expert-info">information required for an Invited Expert in a Working Group</a>
	and the nominating [=Advisory Committee representative=] <em class="rfc2119">must</em> include that information in the nomination.
	In order for an individual to be nominated as a [=Team=] representative,
	the nominating [=Advisory Committee representative=]
	<em class="rfc2119">must</em> first secure approval from [=Team=] management.
	A nominee is <em class="rfc2119">not required</em> to be an employee of a Member organization,
	and <em class="rfc2119">may</em> be a <a href="#fellows">W3C Fellow</a>.
	The nomination form <em class="rfc2119">must</em> ask for the nominee's [=primary affiliation=],
	and this will be reported on the ballot.
	For most nominees,
	the <dfn>primary affiliation</dfn> is their employer and will match their affiliation in the W3C database.
	For contractors and invited experts,
	this will normally be their contracting company
	or their invited expert status;
	in some cases
	(e.g. where a consultant is consulting for only one organization)
	this <em class="rfc2119">may</em> be the organization for whom the nominee is consulting.
	A <a href="#AB-TAG-constraints">change of affiliation</a> is defined
	such that this field would carry a different answer
	if the nominee were to be re-nominated
	(therefore,
	terminating employment,
	or accepting new employment,
	are changes of affiliation).
	(Other formal relationships such as other contracts should be disclosed as potential conflicts of interest.)
	Each nomination <em class="rfc2119">should</em> include
	a few informative paragraphs about the nominee.

	If, after the deadline for nominations, the number of nominees is:

	<ul>
		<li>
			Greater than or equal to the minimum number of available seats
			and less than or equal to the maximum number of available seats,
			those nominees are thereby elected.
			This situation constitutes a tie for the purpose of assigning <a href="#short-term">short terms</a>.
			Furthermore, if the number is less than the maximum number of available seats,
			the longest terms are filled first.

		<li>
			Less than the minimum number of available seats,
			Calls for Nominations are issued until a sufficient number of people have been nominated.
			Those already nominated do not need to be renominated after a renewed call.

		<li>
			Greater than the maximum number of available seats,
			the [=Team=] issues a Call for Votes
			that includes the names of all candidates,
			the (maxiumum) number of available seats,
			the deadline for votes,
			details about the vote tabulation system selected by the Team for the election,
			and operational information.
	</ul>

	When there is a vote,
	each Member
	(or group of [=related Members=])
	<em class="rfc2119">may</em> submit one ballot that ranks candidates in the Member's preferred order.
	Once the deadline for votes has passed,
	the [=Team=] announces the results to the [=Advisory Committee=].
	In case of a tie the <a href="#random">verifiable random selection procedure</a> described below
	will be used to fill the available seats.

	<p id="short-term">The shortest term is assigned to the elected candidate ranked lowest by the tabulation of votes,
	the next shortest term to the next-lowest ranked elected candidate,
	and so on.
	In the case of a tie among those eligible for a short term,
	the <a href="#random">verifiable random selection procedure</a> described below
	will be used to assign the short term.

	Refer to <a href="https://www.w3.org/2002/10/election-howto">How to Organize an Advisory Board or TAG election</a> [[ELECTION-HOWTO]]
	for more details.

<h5 id="random">
Verifiable Random Selection Procedure</h5>

	When it is necessary to use a verifiable random selection process
	(e.g., in an [=AB=] or [=TAG=] election,
	to “draw straws” in case of a tie
	or to fill a short term),
	W3C uses the random and verifiable procedure defined in <a href="https://www.ietf.org/rfc/rfc3797.txt">RFC 3797</a> [[!RFC3797]].
	The procedure orders an input list of names
	(listed in alphabetical order by family name unless otherwise specified)
	into a “result order”.

	W3C applies this procedure as follows:</p>

	<ol>
		<li>
			When N people have tied for M (less than N) seats.
			In this case, only the names of the N individuals who tied
			are provided as input to the procedure.
			The M seats are assigned in result order.

		<li>
			After all elected individuals have been identified,
			when N people are eligible for M (less than N) short terms.
			In this case, only the names of those N individuals are provided as input to the procedure.
			The short terms are assigned in result order.
	</ol>

<h4 id="AB-TAG-vacated">
Advisory Board and Technical Architecture Group Vacated Seats</h4>

	An [=Advisory Board=] or [=TAG=] participant's seat is vacated when:

	<ul>
		<li>
			the participant resigns, or

		<li>
			an Advisory Board or TAG participant changes affiliations
			such that the <a href="#AB-TAG-constraints">Advisory Board and TAG participation constraints</a> are no longer met,
			or

		<li>
			the Director dismisses the participant
			for failing to meet the criteria in section <a href="#ParticipationCriteria"></a>,
			or

		<li>
			their term ends.
	</ul>

	If a participant changes affiliation,
	but the <a href="#AB-TAG-constraints">participation constraints</a> are met,
	that participant's seat becomes vacant at the next regularly scheduled election for that group.

	Vacated seats are filled according to this schedule:

	<ul>
		<li>
			When an appointed [=TAG=] seat is vacated,
			the Director <em class="rfc2119">may</em> re-appoint someone immediately,
			but no later than the next regularly scheduled election.

		<li>
			When an elected seat on either the [=AB=] or [=TAG=] is vacated,
			the seat is filled at the next regularly scheduled election for the group
			unless the group Chair requests that W3C hold an election before then
			(for instance, due to the group's workload).

			<ul>
				<li>
					The group Chair <em class="rfc2119">should not</em> request such an election
					if the next regularly scheduled election is fewer than three months away.

				<li>
					The group Chair <em class="rfc2119">may</em> request an election,
					and the election may begin, as soon as a current member gives notice of a resignation,
					including a resignation effective as of a given date in the future.
			</ul>

			When such an election is held,
			the minimum number of available seat is such that
			when added to the number of continuing participants,
			the minimum total number of elected seats is met
			(6 for the [=TAG=], 9 for the [=AB=]);
			and the maximum number corresponds to all unoccupied seats.
			Except for the number of available seats and the length of the terms,
			the <a href="#AB-TAG-elections">usual rules for Advisory Board and Technical Architecture Group Elections</a> apply.
	</ul>

<h2 id="Policies">
General Policies for W3C Groups</h2>

	This section describes general policies for W3C groups regarding participation,
	meeting requirements,
	and decision-making.
	These policies apply to <span id="participant">participants</span> in the following groups:
	<a href="#AC">Advisory Committee</a>,
	<a href="#ABParticipation">Advisory Board</a>,
	<a href="#tag-participation">TAG</a>,
	<a href="#wgparticipant">Working Groups</a>,
	and <a href="#igparticipant">Interest Groups</a>.

<h3 id="ParticipationCriteria">
Individual Participation Criteria</h3>

	There are three qualities an individual is expected to demonstrate in order to participate in W3C:

	<ol>
		<li>
			Technical competence in one's role;

		<li>
			The ability to act fairly;

		<li>
			Social competence in one's role.
	</ol>

	[=Advisory Committee representatives=] who nominate individuals from their organization for participation in W3C activities
	are responsible for assessing and attesting to the qualities of those nominees.

	<p>Participants in any W3C activity <em class="rfc2119">must</em> abide
	by the terms and spirit of the <a href="https://www.w3.org/Consortium/cepc/">W3C Code of Ethics and Professional Conduct</a> [[!CEPC]]
	and the participation requirements described in
	“Disclosure”
	in the W3C Patent Policy [[!PATENT-POLICY]].

	The [=Director=] <em class="rfc2119">may</em> suspend or remove for cause
	a participant in any group (including the [=AB=] and [=TAG=]),
	where cause includes failure to meet the requirements of this process,
	the membership agreement, or applicable laws.

<h4 id="coi">
Conflict of Interest Policy</h4>

	Individuals participating materially in W3C work <em class="rfc2119">must</em> disclose significant relationships
	when those relationships might reasonably be perceived as creating a conflict of interest with the individual's role at W3C.
	These disclosures <em class="rfc2119">must</em> be kept up-to-date
	as the individual's affiliations change and W3C membership evolves
	(since, for example, the individual might have a relationship with an organization that joins or leaves W3C).
	Each section in this document that describes a W3C group
	provides more detail about the disclosure mechanisms for that group.

	The ability of an individual to fulfill a role within a group
	without risking a conflict of interest depends on the individual's affiliations.
	When these affiliations change,
	the individual's assignment to the role <em class="rfc2119">must</em> be evaluated.
	The role <em class="rfc2119">may</em> be reassigned according to the appropriate process.
	For instance,
	the [=Director=] <em class="rfc2119">may</em> appoint a new group [=Chair=]
	when the current Chair changes affiliations
	(e.g., if there is a risk of conflict of interest,
	or if there is risk that the Chair's new employer will be over-represented within a W3C activity).

	The following are some scenarios where disclosure is appropriate:

	<ul>
		<li>
			Paid consulting for an organization whose activity is relevant to W3C,
			or any consulting compensated with equity
			(shares of stock, stock options, or other forms of corporate equity).

		<li>
			A decision-making role/responsibility
			(such as participating on the Board) in other organizations relevant to W3C.

		<li>
			A position on a publicly visible advisory body,
			even if no decision making authority is involved.
	</ul>

	Individuals seeking assistance on these matters <em class="rfc2119">should</em> contact the [=Team=].

	[=Team=] members are subject to the <a href="https://www.w3.org/2000/09/06-conflictpolicy">W3C Team conflict of interest policy</a> [[!CONFLICT-POLICY]].

<h4 id="member-rep">
Individuals Representing a Member Organization</h4>

	Generally, individuals representing a Member in an official capacity within W3C
	are employees of the Member organization.
	However, an [=Advisory Committee representative=] <em class="rfc2119">may</em> designate a non-employee
	to represent the Member.
	Non-employee Member representatives <em class="rfc2119">must</em> disclose
	relevant affiliations to the Team and to any group in which the individual participates.

	In exceptional circumstances
	(e.g., situations that might jeopardize the progress of a group or create a <a href="#coi">conflict of interest</a>),
	the <a href="#def-Director">Director</a> <em class="rfc2119">may</em> decline
	to allow an individual designated by an Advisory Committee representative to participate in a group.

	A group charter <em class="rfc2119">may</em> limit
	the number of individuals representing a W3C Member
	(or group of <a href="#MemberRelated">related Members</a>).

<h3 id="GeneralMeetings">
Meetings</h3>

	W3C groups
	(including the <a href="#ACMeetings">Advisory Committee</a>,
	<a href="#AB">Advisory Board</a>,
	<a href="#TAG">TAG</a>,
	and <a href="#GroupsWG">Working Groups</a>)
	<em class="rfc2119">should</em> observe the meeting requirements in this section.

	<p>W3C distinguishes two types of meetings:</p>

	<ol>
		<li>
			A <dfn id="ftf-meeting">face-to-face meeting</dfn> is one
			where most of the attendees are expected to participate in the same physical location.

		<li>
			A <dfn id="distributed-meeting">distributed meeting</dfn> is one
			where most of the attendees are expected to participate from remote locations
			(e.g., by telephone, video conferencing, or <abbr title="Internet Relay Chat">IRC</abbr>).
	</ol>

	A [=Chair=] <em class="rfc2119">may</em> invite an individual with a particular expertise
	to attend a meeting on an exceptional basis.
	This person is a meeting guest,
	not a group <a href="#participant">participant</a>.
	Meeting guests do not have <a href="#Votes">voting rights</a>.
	It is the responsibility of the Chair to ensure
	that all meeting guests respect the chartered <a href="#confidentiality-levels">level of confidentiality</a>
	and other group requirements.

	Meeting announcements <em class="rfc2119">should</em> be sent to all appropriate group mailing lists,
	i.e. those most relevant to the anticipated meeting participants.

	The following table lists requirements for organizing a meeting:

	<table class="data">
	<thead>
		<tr>
			<td>
			<th scope=col>Face-to-face meetings
			<th scope=col>Distributed meetings

	<tbody>
		<tr>
			<th scope=row>Meeting announcement (before)
			<td><span class="time-interval">eight weeks<sup>*</sup></span>
			<td><span class="time-interval">one week<sup>*</sup></span>
		<tr>
			<th scope=row>Agenda available (before)
			<td><span class="time-interval">two weeks</span>
			<td><span class="time-interval">24 hours</span> (or longer if a meeting is scheduled after a weekend or holiday)
		<tr>
			<th scope=row>Participation confirmed (before)
			<td><span class="time-interval">three days</span>
			<td><span class="time-interval">24 hours</span>
		<tr>
			<th scope=row>Action items available (after)
			<td><span class="time-interval">three days</span>
			<td><span class="time-interval">24 hours</span>
		<tr>
			<th scope=row>Minutes available (after)
			<td><span class="time-interval">two weeks</span>
			<td><span class="time-interval">48 hours</span>
	</table>

	<sup>*</sup> To allow proper planning (e.g., travel arrangements),
	the Chair is responsible for giving sufficient advance notice
	about the date and location of a meeting.
	Shorter notice for a meeting is allowed
	provided that there are no objections from group participants.

<h3 id="Consensus">
Consensus</h3>

	Consensus is a core value of W3C.
	To promote consensus,
	the W3C process requires Chairs to ensure
	that groups consider all legitimate views and objections,
	and endeavor to resolve them,
	whether these views and objections are expressed by the active participants of the group
	or by others
	(e.g., another W3C group,
	a group in another organization,
	or the general public).
	Decisions <em class="rfc2119">may</em> be made during meetings
	(<a href="#ftf-meeting">face-to-face</a>
	or <a href="#distributed-meeting">distributed</a>)
	as well as through email.

	Note: The Director, CEO, and COO have the role of
	assessing consensus within the Advisory Committee.

	The following terms are used in this document
	to describe the level of support for a decision among a set of eligible individuals:

	<dl>
		<dt><dfn id="def-Consensus">Consensus</dfn>:
		<dd>
			A substantial number of individuals in the set
			support the decision
			and nobody in the set registers a <a href="#FormalObjection">Formal Objection</a>.
			Individuals in the set <em class="rfc2119">may</em> abstain.
			Abstention is either an explicit expression of no opinion
			or silence by an individual in the set.

		<dt><dfn id="def-Unanimity">Unanimity</dfn>:
		<dd>
			The particular case of [=consensus=]
			where all individuals in the set support the decision
			(i.e., no individual in the set abstains).

		<dt><dfn id="def-Dissent">Dissent</dfn>:
		<dd>
			At least one individual in the set registers a <a href="#FormalObjection">Formal Objection</a>.
	</dl>

	By default, the set of individuals eligible to participate in a decision is the set of group participants.
	The Process Document does not require a quorum for decisions
	(i.e., the minimal number of eligible participants required to be present before the Chair can call a question).
	A charter <em class="rfc2119">may</em> include a quorum requirement for consensus decisions.

	Where [=unanimity=] is not possible,
	a group <em class="rfc2119">should</em> strive to make [=consensus=] decisions
	where there is significant support and few abstentions.
	The Process Document does not require a particular percentage of eligible participants
	to agree to a motion in order for a decision to be made.
	To avoid decisions where there is widespread apathy,
	(i.e., little support and many abstentions),
	groups <em class="rfc2119">should</em> set minimum thresholds of active support before a decision can be recorded.
	The appropriate percentage <em class="rfc2119">may</em> vary depending on the size of the group
	and the nature of the decision.
	A charter <em class="rfc2119">may</em> include threshold requirements for consensus decisions.
	For instance, a charter might require a supermajority of eligible participants
	(i.e., some established percentage above 50%)
	to support certain types of consensus decisions.

	<div class=note>
	Note: Chairs have substantial flexibility
	in how they obtain and assess consensus among their groups.
	Unless otherwise constrained by charter,
	they may use modes including but not limited to explicit calls for consensus,
	polls of participants,
	“lazy consensus” in which lack of objection after sufficient notice is taken as assent;
	or they may also delegate and empower a document editor
	to assess consensus on their behalf,
	whether in general
	or for specific pre-determined circumstances
	(e.g. in non-controversial situations, for specific types of issues, etc).

	If questions or disagreements arise,
	the final determination of consensus remains with the chair.
	</div>

<h4 id="managing-dissent">
Managing Dissent</h4>

	In some cases, even after careful consideration of all points of view,
	a group might find itself unable to reach consensus.
	The [=Chair=] <em class="rfc2119">may</em> record a decision where there is [=dissent=]
	(i.e., there is at least one <a href="#FormalObjection">Formal Objection</a>)
	so that the group can make progress
	(for example, to produce a deliverable in a timely manner).
	Dissenters cannot stop a group's work
	simply by saying that they cannot live with a decision.
	When the Chair believes that the Group has duly considered
	the legitimate concerns of dissenters as far as is possible and reasonable,
	the group <em class="rfc2119">should</em> move on.

	Groups <em class="rfc2119">should</em> favor proposals that create the weakest objections.
	This is preferred over proposals that are supported by a large majority
	but that cause strong objections from a few people.
	As part of making a decision where there is [=dissent=],
	the [=Chair=] is expected to be aware of which participants work for the same
	(or <a href="#MemberRelated">related</a>)
	Member organizations and weigh their input accordingly.

<h4 id="WGArchiveMinorityViews">
Recording and Reporting Formal Objections</h4>

	In the W3C process,
	an individual <em class="rfc2119">may</em> register a Formal Objection to a decision.
	A <dfn id="FormalObjection">Formal Objection</dfn> to a group decision
	is one that the reviewer requests that the Director consider
	as part of evaluating the related decision
	(e.g., in response to a <a href="#rec-advance">request to advance</a> a technical report).

	Note: In this document, the term “Formal Objection” is used to emphasize this process implication:
	Formal Objections receive Director consideration.
	The word “objection” used alone has ordinary English connotations.

	An individual who registers a [=Formal Objection=] <em class="rfc2119">should</em> cite technical arguments
	and propose changes that would remove the [=Formal Objection=];
	these proposals <em class="rfc2119">may</em> be vague or incomplete.
	[=Formal Objections=] that do not provide substantive arguments
	or rationale are unlikely to receive serious consideration by the Director.

	A record of each [=Formal Objection=] <em class="rfc2119">must</em> be <a href="#confidentiality-change">publicly available</a>.
	A Call for Review (of a document) to the Advisory Committee <em class="rfc2119">must</em> identify any [=Formal Objections=].

<h4 id="formal-address">
Formally Addressing an Issue</h4>

	In the context of this document,
	a group has <dfn export>formally addressed</dfn> an issue when it has sent a public, substantive response
	to the reviewer who raised the issue.
	A substantive response is expected to include rationale for decisions
	(e.g., a technical explanation, a pointer to charter scope, or a pointer to a requirements document).
	The adequacy of a response is measured
	against what a W3C reviewer would generally consider to be technically sound.
	If a group believes that a reviewer's comments result from a misunderstanding,
	the group <em class="rfc2119">should</em> seek clarification before reaching a decision.

	As a courtesy,
	both Chairs and reviewers <em class="rfc2119">should</em> set expectations
	for the schedule of responses and acknowledgments.
	The group <em class="rfc2119">should</em> reply to a reviewer's initial comments
	in a timely manner.
	The group <em class="rfc2119">should</em> set a time limit
	for acknowledgment by a reviewer of the group's substantive response;
	a reviewer cannot block a group's progress.
	It is common for a reviewer to require a week or more
	to acknowledge and comment on a substantive response.
	The group's responsibility to respond to reviewers
	does not end once a reasonable amount of time has elapsed.
	However, reviewers <em class="rfc2119">should</em> realize
	that their comments will carry less weight
	if not sent to the group in a timely manner.

	Substantive responses <em class="rfc2119">should</em> be recorded.
	The group <em class="rfc2119">should</em> maintain an accurate summary
	of all substantive issues and responses to them
	(e.g., in the form of an issues list with links to mailing list archives).

<h4 id="WGChairReopen">
Reopening a Decision When Presented With New Information</h4>

	The [=Chair=] <em class="rfc2119">may</em> reopen a decision
	when presented with new information, including:

	<ul>
		<li>
			additional technical information,
		<li>
			comments by email from participants who were unable to attend a scheduled meeting,

		<li>
			comments by email from meeting attendees
			who chose not to speak out during a meeting
			(e.g., so they could confer later with colleagues or for cultural reasons).
	</ul>

	The [=Chair=] <em class="rfc2119">should</em> record
	that a decision has been reopened,
	and <em class="rfc2119">must</em> do so upon request from a group participant.

<h3 id="Votes">
Votes</h3>

	A group <em class="rfc2119">should</em> only conduct a vote to resolve a <em>substantive issue</em>
	after the [=Chair=] has determined that all available means of <a href="#Consensus">reaching consensus</a>
	through technical discussion and compromise have failed,
	and that a vote is necessary to break a deadlock.
	In this case the [=Chair=] <em class="rfc2119">must</em> record
	(e.g., in the minutes of the meeting or in an archived email message):

	<ul>
		<li>
			an explanation of the issue being voted on;

		<li>
			the decision to conduct a vote
			(e.g., a simple majority vote) to resolve the issue;

		<li>
			the outcome of the vote;

		<li>
			any Formal Objections.
	</ul>

	In order to vote to resolve a substantive issue,
	an individual <em class="rfc2119">must</em> be a group <a href="#participant">participant</a>.
	Each organization represented in the group <em class="rfc2119">must</em> have at most one vote,
	even when the organization is represented by several participants in the group
	(including Invited Experts).
	For the purposes of voting:

	<ul>
		<li>
			A Member or group of <a href="#MemberRelated">related Members</a> is considered a single organization.

		<li>
			The <a href="#Team">Team</a> is considered an organization.
	</ul>

	Unless the charter states otherwise,
	<a href="#invited-expert-wg">Invited Experts</a> <em class="rfc2119">may</em> vote.

	If a participant is unable to attend a vote,
	that individual <em class="rfc2119">may</em> authorize anyone at the meeting
	to act as a <dfn id="proxy">proxy</dfn>.
	The absent participant <em class="rfc2119">must</em> inform the Chair in writing
	who is acting as proxy, with written instructions on the use of the proxy.
	For a Working Group or Interest Group,
	see the related requirements regarding an individual
	who attends a meeting as a <a href="#mtg-substitute">substitute</a> for a participant.

	A group <em class="rfc2119">may</em> vote for other purposes than to resolve a substantive issue.
	For instance, the Chair often conducts a “straw poll” vote
	as a means of determining whether there is consensus about a potential decision.

	A group <em class="rfc2119">may</em> also vote to make a process decision.
	For example,
	it is appropriate to decide by simple majority
	whether to hold a meeting in San Francisco or San Jose
	(there's not much difference geographically).
	When simple majority votes are used to decide minor issues,
	voters are <em class="rfc2119">not required</em> to state the reasons for votes,
	and the group is <em class="rfc2119">not required</em> to record individual votes.

	A group charter <em class="rfc2119">may</em> include formal voting procedures
	(e.g., quorum or threshold requirements)
	for making decisions about substantive issues.

	Procedures for <a href="#ACVotes">Advisory Committee votes</a> are described separately.

<h3 id="WGAppeals">
Appeal of [=Chair Decisions=] and [=Group Decisions=]</h3>

	Groups resolve issues through dialog.
	Individuals who disagree strongly with a decision
	<em class="rfc2119">should</em> register with the Chair any [=Formal Objections=]
	(e.g., to a decision made as the result of a <a href="#Votes">vote</a>).

	As detailed in other parts of this document,
	the [=Chair=] of a [=Working Group=] or [=Interest Group=] has the prerogative
	to make certain decisions based on their own judgement.
	Such decisions are called <dfn export>Chair Decisions</dfn>.
	In contrast,
	decisions taken by the [=Chair=] of a [=Working Group=] or [=Interest Group=]
	on the basis of having assessed the [=consensus=] of the group
	or following a vote (see [[#Votes]])
	are called <dfn export>Group Decisions</dfn>.

	When group participants believe that their concerns are not being duly considered by the group or the [=Chair=],
	they <em class="rfc2119">may</em> ask the <a href="#def-Director">Director</a>
	(for representatives of a Member organization, via their Advisory Committee representative)
	to confirm or deny the decision.
	This is a <dfn export id="wg-decision-appeal">Group Decision Appeal</dfn>
	or a <dfn export id="chair-decision-appeal">Chair Decision Appeal</dfn>.
	The participants <em class="rfc2119">should</em> also make their requests known
	to the <a href="#TeamContact">Team Contact</a>.
	The Team Contact <em class="rfc2119">must</em> inform the Director
	when a group participant has raised concerns about due process.

	Any requests to the Director to confirm a decision
	<em class="rfc2119">must</em> include a summary of
	the issue (whether technical or procedural),
	decision,
	and rationale for the objection.
	All counter-arguments,
	rationales,
	and decisions <em class="rfc2119">must</em> be recorded.

	Procedures for <a href="#ACAppeal">Advisory Committee appeals</a> are described separately.

<h3 id="resignation">
Resignation from a Group</h3>

	A W3C Member or Invited Expert <em class="rfc2119">may</em> resign from a group.
	On written notification from an Advisory Committee representative
	or Invited Expert
	to the team,
	the Member and their representatives
	or the Invited Expert
	will be deemed to have resigned from the relevant group.
	The team <em class="rfc2119">must</em> record the notification.
	See “Exclusion and Resignation from the Working Group” in the W3C Patent Policy [[PATENT-POLICY]]
	for information about obligations remaining after resignation from certain groups.

<h2 id="dissemination"> <span id="chapterDissemination"></span>
Dissemination Policies</h2>

	The [=Team=] is responsible for managing communication within W3C
	and with the general public
	(e.g., news services, press releases, managing the Web site and access privileges, and managing calendars).
	Members <em class="rfc2119">should</em> solicit review by the Team
	prior to issuing press releases about their work within W3C.

	The [=Team=] makes every effort to ensure the persistence and availability of the following public information:

	<ul>
		<li>
			<a href="#Reports">W3C technical reports</a> whose publication has been approved by the Director.
			Per the Membership Agreement,
			W3C technical reports (and software) are available free of charge to the general public;
			(refer to the <a href="https://www.w3.org/Consortium/Legal/copyright-documents">W3C Document License</a> [[DOC-LICENSE]]).

		<li>
			A <a href="https://www.w3.org/Consortium/mission">mission statement</a> [[MISSION]]
			that explains the purpose and mission of W3C,
			the key benefits for Members,
			and the organizational structure of W3C.

		<li>
			Legal documents,
			including the <a href="https://www.w3.org/Consortium/Agreement/Member-Agreement">Membership Agreement</a> [[MEMBER-AGREEMENT]]
			and documentation of any legal commitments W3C has with other entities.

		<li>
			The Process Document.

		<li>
			Public results of W3C activities and <a href="#GAEvents">Workshops</a>.
	</ul>

	To keep the Members abreast of W3C meetings,
	[=Workshops=],
	and review deadlines,
	the Team provides them with a regular (e.g., weekly) news service
	and maintains a <a href="https://www.w3.org/participate/eventscal">calendar</a> [[CALENDAR]]. 
	of official W3C events.
	Members are encouraged to send schedule and event information to the Team for inclusion on this calendar.

<h3 id="confidentiality-levels">
Confidentiality Levels</h3>

	There are three principal levels of access to W3C information
	(on the W3C Web site, in W3C meetings, etc.):
	public,
	Member-only,
	and Team-only.

	While much information made available by W3C is public,
	“<dfn export id="Member-only" lt="Member-Only">Member-only</dfn>” information
	is available to authorized parties only,
	including representatives of Member organizations,
	<a href="#invited-expert-wg">Invited Experts</a>,
	the Advisory Board,
	the TAG,
	and the Team.
	For example,
	the <a href="#WGCharter">charter</a> of some Working Groups
	<em class="rfc2119">may</em> specify a [=Member-only=] confidentiality level for group proceedings.

	“<dfn export id="Team-only" lt="Team-Only">Team-only</dfn>” information
	is available to the Team and other authorized parties.

	Those authorized to access [=Member-only=] and [=Team-only=] information:

	<ul>
		<li>
			<em class="rfc2119">must</em> treat the information as confidential within W3C,

		<li>
			<em class="rfc2119">must</em> use reasonable efforts to maintain the proper level of confidentiality, and

		<li>
			<em class="rfc2119">must not</em> release this information to the general public or press.
	</ul>

	The [=Team=] <em class="rfc2119">must</em> provide mechanisms
	to protect the confidentiality of [=Member-only=] information
	and ensure that authorized parties have proper access to this information.
	Documents <em class="rfc2119">should</em> clearly indicate
	whether they require [=Member-only=] confidentiality.
	Individuals uncertain of the confidentiality level of a piece of information
	<em class="rfc2119">should</em> contact the Team.

	[=Advisory Committee representatives=] <em class="rfc2119">may</em> authorize
	[=Member-only=] access to <a href="#member-rep">Member representatives</a>
	and other individuals employed by the Member
	who are considered appropriate recipients.
	For instance,
	it is the responsibility of the [=Advisory Committee representative=]
	and other employees
	and official representatives of the organization
	to ensure that Member-only news announcements
	are distributed for internal use only within their organization.
	Information about Member mailing lists is available
	in the <a href="https://www.w3.org/Member/Intro">New Member Orientation</a> [[INTRO]].

<h4 id="confidentiality-change">
Changing Confidentiality Level</h4>

	As a benefit of membership,
	W3C provides some [=Team-only=] and [=Member-only=] channels
	for certain types of communication.
	For example, [=Advisory Committee representatives=]
	can send <a href="#ACReview">reviews</a> to a [=Team-only=] channel.
	However, for W3C processes with a significant public component,
	such as the technical report development process,
	it is also important for information that affects decision-making to be publicly available.
	The Team <em class="rfc2119">may</em> need to communicate [=Team-only=] information to a Working Group or the public.
	Similarly, a Working Group whose proceedings are [=Member-only=]
	<em class="rfc2119">must</em> make public
	information pertinent to the technical report development process.

	This document clearly indicates which information <em class="rfc2119">must</em> be available to Members or the public,
	even though that information was initially communicated on [=Team-only=] or [=Member-only=] channels.
	Only the [=Team=] and parties authorized by the Team
	may change the level of confidentiality of this information.
	When doing so:

	<ol>
		<li>
			The [=Team=] <em class="rfc2119">must</em> use a version of the information
			that was expressly provided by the author for the new confidentiality level.
			In Calls for Review and other similar messages,
			the Team <em class="rfc2119">should</em> remind recipients to provide such alternatives.

		<li>
			The [=Team=] <em class="rfc2119">must not</em> attribute the version
			for the new confidentiality level to the author without the author's consent.

		<li>
			If the author has not conveyed to the [=Team=] a version
			that is suitable for another confidentiality level,
			the Team <em class="rfc2119">may</em> make available a version that reasonably communicates what is required,
			while respecting the original level of confidentiality,
			and without attribution to the original author.
	</ol>


<h2 id="GAGeneral"> <span id="ChapterGroups"></span>
Working Groups and Interest Groups</h2>

	<p id="GAGroups">This document defines two types of groups:

	<dl>
		<dt>
			<a href="#GroupsWG">Working Groups</a>.
		<dd>
			[=Working Groups=] typically produce deliverables
			(e.g., <a href="#rec-advance">Recommendation Track technical reports</a>,
			software,
			test suites,
			and reviews of the deliverables of other groups).
			There are additional participation requirements
			described in the W3C Patent Policy [[!PATENT-POLICY]].

		<dt>
			<a href="#GroupsIG">Interest Groups</a>.
		<dd>
			The primary goal of an [=Interest Group=]
			is to bring together people who wish to evaluate potential Web technologies and policies.
			An Interest Group is a forum for the exchange of ideas.
	</dl>

	[=Interest Groups=] do not publish <a href="#RecsW3C">Recommendation Track technical reports</a>;
	see information about <a href="#WGNote">maturity levels for Interest Groups</a>.

<h3 id="ReqsAllGroups">
Requirements for All Working and Interest Groups</h3>

	Each group <em class="rfc2119">must</em> have a charter.
	Requirements for the charter depend on the group type.
	All group charters <em class="rfc2119">must</em> be public
	(even if other proceedings of the group are <a href="#Member-only">Member-only</a>).

	Each group <em class="rfc2119">must</em> have a <dfn id="GeneralChairs" export>Chair</dfn> (or co-Chairs)
	to coordinate the group's tasks.
	The Director appoints (and re-appoints) Chairs for all groups.
	The Chair is a <a href="#member-rep">Member representative</a>,
	a <a href="#Team">Team representative</a>,
	or an <a href="#invited-expert-wg">Invited Expert</a>
	(invited by the Director).
	The requirements of this document that apply to those types of participants apply to Chairs as well.
	The <a href="https://www.w3.org/Guide/chair/role.html">role of the Chair</a> [[CHAIR]] is described
	in the <a href="https://www.w3.org/Guide/">Art of Consensus</a> [[GUIDE]].

	Each group <em class="rfc2119">must</em> have a <dfn id="TeamContact" export>Team Contact</dfn>,
	who acts as the interface between the [=Chair=],
	group participants,
	and the rest of the Team.
	The <a href="https://www.w3.org/Guide/teamcontact/role.html">role of the Team Contact</a> [[TEAM-CONTACT]] is described in the Member guide [[GUIDE]].
	The [=Chair=] and the [=Team Contact=] of a group <em class="rfc2119">should not</em> be the same individual.

	Each group <em class="rfc2119">must</em> have an archived mailing list
	for formal group communication
	(e.g., for meeting announcements and minutes,
	documentation of decisions,
	and [=Formal Objections=] to decisions).
	It is the responsibility of the [=Chair=] and [=Team Contact=]
	to ensure that new participants are subscribed to all relevant mailing lists.
	Refer to the list of <a href="https://www.w3.org/Member/Mail/">group mailing lists</a> [[GROUP-MAIL]].

	<p>A [=Chair=] <em class="rfc2119">may</em> form task forces
	(composed of group participants)
	to carry out assignments for the group.
	The scope of these assignments <em class="rfc2119">must not</em> exceed the scope of the group's charter.
	A group <em class="rfc2119">should</em> document the process it uses
	to create task forces
	(e.g., each task force might have an informal "charter").
	Task forces do not publish <a href="#Reports">technical reports</a>;
	the Working Group <em class="rfc2119">may</em> choose to publish their results as part of a technical report.

<h3 id=WG-and-IG>
Working Groups and Interest Groups</h3>

	Although <dfn export id="GroupsWG">Working Groups</dfn>
	and <dfn export id="GroupsIG">Interest Groups</dfn>
	have different purposes,
	they share some characteristics,
	and so are defined together in the following sections.

<h4 id="group-participation">
Working Group and Interest Group Participation Requirements</h4>

	There are three types of individual <dfn id="wgparticipant">participants in a Working Group</dfn>:
	<a href="#member-rep">Member representatives</a>,
	<a href="#invited-expert-wg">Invited Experts</a>,
	and <a href="#Team">Team representatives</a>
	(including the <a href="#TeamContact">Team Contact</a>).

	There are four types of individual <dfn id="igparticipant">participants in an Interest Group</dfn>:
	the same three types as for Working Groups plus,
	for an Interest Group where the only <a href="#ig-mail-only">participation requirement is mailing list subscription</a>,
	<dfn id="public-participant-ig">public participants</dfn>.

	Except where noted in this document or in a group charter,
	all participants share the same rights and responsibilities in a group;
	see also the <a href="#ParticipationCriteria">individual participation criteria</a>.

	A participant <em class="rfc2119">may</em> represent more than one organization
	in a [=Working Group=] or [=Interest Group=].
	Those organizations <em class="rfc2119">must</em> all be members of the group.

	An individual <em class="rfc2119">may</em> become
	a Working or Interest Group participant
	at any time during the group's existence.
	See also relevant requirements in
	“Joining an Already Established Working Group”
	in the W3C Patent Policy [[PATENT-POLICY]].

	On an exceptional basis,
	a Working or Interest Group participant <em class="rfc2119">may</em> designate
	a <dfn id="mtg-substitute">substitute</dfn>
	to attend a <a href="#GeneralMeetings">meeting</a>
	and <em class="rfc2119">should</em> inform the [=Chair=].
	The substitute <em class="rfc2119">may</em> act on behalf of the participant,
	including for <a href="#Votes">votes</a>.
	For the substitute to vote,
	the participant <em class="rfc2119">must</em> inform the [=Chair=] in writing in advance.
	As a courtesy to the group,
	if the substitute is not well-versed in the group's discussions,
	the regular participant <em class="rfc2119">should</em> authorize another participant to act as <a href="#proxy">proxy</a> for votes.

	To allow rapid progress,
	Working Groups are intended to be small
	(typically fewer than 15 people)
	and composed of experts in the area defined by the charter.
	In principle,
	Interest Groups have no limit on the number of participants.
	When a Working Group grows too large to be effective,
	W3C <em class="rfc2119">may</em> split it into an Interest Group
	(a discussion forum)
	and a much smaller Working Group
	(a core group of highly dedicated participants).

	See also the licensing obligations on Working Group participants
	in “Licensing Obligations”
	in the W3C Patent Policy [[!PATENT-POLICY]],
	and the patent claim exclusion process
	in “Exclusion From W3C RF Licensing Requirements” in the Patent Policy.

<h5 id="member-rep-wg">
Member Representative in a Working Group</h5>

	An individual is a <dfn export>Member representative in a Working Group</dfn>
	if all of the following conditions are satisfied:

	<ul>
		<li>
			the Advisory Committee representative of the Member in question
			has designated the individual as a Working Group participant, and

		<li>
			the individual qualifies for <a href="#member-rep">Member representation</a>.
	</ul>

	To designate an individual as a Member representative in a [=Working Group=],
	an [=Advisory Committee representative=] <em class="rfc2119">must</em> provide the [=Chair=] and [=Team Contact=]
	with all of the following <span id=member-rep-info>information</span>,
	in addition to any other information required by the <a href="#cfp">Call for Participation</a>
	and charter
	(including the participation requirements of the W3C Patent Policy [[!PATENT-POLICY]]):

	<ol>
		<li>
			The name of the W3C Member the individual represents
			and whether the individual is an employee of that Member organization;

		<li>
			A statement that the individual accepts the participation terms
			set forth in the charter
			(with an indication of charter date or version);

		<li>
			A statement that the Member will provide the necessary financial support for participation
			(e.g., for travel, telephone calls, and conferences).
	</ol>

	A Member participates in a [=Working Group=]
	from the moment the first Member representative joins the group
	until either of the following occurs:

	<ul>
		<li>
			the group closes, or
		<li>
			the Member <a href="#resignation">resigns</a> from the Working Group;
			this is done through the Member's Advisory Committee representative.
	</ul>

<h5 id="member-rep-ig">
Member Representative in an Interest Group</h5>

	When the participation requirements exceed <a href="#ig-mail-only">Interest Group mailing list subscription</a>,
	an individual is a <dfn export>Member representative in an Interest Group</dfn>
	if all of the following conditions are satisfied:

	<ul>
		<li>
			the [=Advisory Committee representative=] of the Member in question has designated the individual
			as an Interest Group participant, and

		<li>
			the individual qualifies for <a href="#member-rep">Member representation</a>.
	</ul>

	To designate an individual as a [=Member representative in an Interest Group=],
	the Advisory Committee representative <em class="rfc2119">must</em> follow the instructions
	in the <a href="#cfp">Call for Participation</a> and charter.

	Member participation in an Interest Group ceases under the same conditions as for a Working Group.

<h5 id="invited-expert-wg">
Invited Expert in a Working Group</h5>

	The [=Chair=] <em class="rfc2119">may</em> invite an individual with a particular expertise
	to participate in a Working Group.
	This individual <em class="rfc2119">may</em> represent an organization in the group
	(e.g., if acting as a liaison with another organization).

	An individual is an <dfn export>Invited Expert in a Working Group</dfn> if all of the following conditions are satisfied:

	<ul>
		<li>
			the Chair has designated the individual as a group participant,

		<li>
			the Team Contact has agreed with the Chair's choice, and

		<li>
			the individual has provided the <a href="#inv-expert-info">information required of an Invited Expert</a>
			to the Chair and Team Contact.
	</ul>

	To designate an individual as an Invited Expert in a Working Group,
	the [=Chair=] <em class="rfc2119">must</em> inform the Team Contact
	and provide rationale for the choice.
	When the [=Chair=] and the [=Team Contact=] disagree about a designation,
	the <a href="#def-Director">Director</a> determines
	whether the individual will be invited to participate in the Working Group.

	To <dfn id="inv-expert-info">participate in a [=Working Group=] as an Invited Expert</dfn>,
	an individual <em class="rfc2119">must</em>:

	<ul>
		<li>
			identify the organization, if any, the individual represents as a participant in this group,

		<li>
			agree to the terms of the
			<a href="https://www.w3.org/Consortium/Legal/collaborators-agreement">invited expert and collaborators agreement</a> [[!COLLABORATORS-AGREEMENT]],

		<li>
			accept the participation terms set forth in the charter,
			including the participation requirements of
			the W3C Patent Policy [[!PATENT-POLICY]],
			especially in “Note on Licensing Commitments for Invited Experts”
			and in “Disclosure”,
			indicating a specific charter date or version,

		<li>
			disclose whether the individual is an employee of a W3C Member;
			see the <a href="#coi">conflict of interest policy</a>,

		<li>
			provide a statement of who will provide the necessary financial support
			for the individual's participation
			(e.g., for travel, telephone calls, and conferences), and

		<li>
			if the individual's employer (including a self-employed individual)
			or the organization the individual represents
			is not a W3C Member,
			indicate whether that organization intends to join W3C.
			If the organization does not intend to join W3C,
			indicate reasons the individual is aware of for this choice.
	</ul>

	The [=Chair=] <em class="rfc2119">should not</em> designate as an [=Invited Expert in a Working Group=]
	an individual who is an employee of a W3C Member.
	The Chair <em class="rfc2119">must not</em> use Invited Expert status
	to circumvent participation limits imposed by the <a href="#WGCharter">charter</a>.

	An Invited Expert participates in a Working Group
	from the moment the individual joins the group
	until any of the following occurs:

	<ul>
		<li>
			the group closes, or

		<li>
			the Chair or Director withdraws the invitation to participate, or

		<li>
			the individual <a href="#resignation">resigns</a>.
	</ul>

<h5 id="invited-expert-ig">
Invited Expert in an Interest Group</h5>

	When the participation requirements exceed <a href="#ig-mail-only">Interest Group mailing list subscription</a>,
	the participation requirements for an <dfn export>Invited Expert in an Interest Group</dfn>
	are the same as those for an [=Invited Expert in a Working Group=].

<h5 id="team-rep-wg">
Team Representative in a Working Group</h5>

	An individual is a <dfn export>Team representative in a Working Group</dfn>
	when so designated by W3C management.

	A Team representative participates in a Working Group
	from the moment the individual joins the group
	until any of the following occurs:

	<ul>
		<li>
			the group closes, or

		<li>
			W3C management changes Team representation by sending email to the Chair,
			copying the group mailing list.
	</ul>

	The Team participates in a Working Group
	from the moment the Director announces the creation of the group
	until the group closes.

<h5 id="team-rep-ig">
Team Representative in an Interest Group</h5>

	When the participation requirements exceed <a href="#ig-mail-only">Interest Group mailing list subscription</a>,
	an individual is a <dfn export>Team representative in an Interest Group</dfn> when so designated by W3C management.

<h4 id="WGCharterDevelopment">
Working Group and Interest Group Charter Development</h4>

	W3C creates a charter based on interest from the Members and Team.
	The Team <em class="rfc2119">must</em> notify the Advisory Committee
	when a charter for a new Working Group or Interest Group is in development.
	This is intended to raise awareness,
	even if no formal proposal is yet available.
	Advisory Committee representatives <em class="rfc2119">may</em> provide
	feedback on the <a href="#ACCommunication">Advisory Committee discussion list</a>.

	W3C <em class="rfc2119">may</em> begin work
	on a Working Group or Interest Group charter
	at any time.

<h4 id="CharterReview">
Advisory Committee Review of a Working Group or Interest Group Charter</h4>

	The Director <em class="rfc2119">must</em> solicit <a href="#ACReview">Advisory Committee review</a>
	of every new or substantively modified Working Group or Interest Group charter,
	except for either:
	* a charter extension
	* substantive changes to a charter that do not affect the way the group functions in any significant way.

	The review period <em class="rfc2119">must</em> be at least 28 days.
	The following are examples of substantive changes that would not require an [=Advisory Committee Review=]:
	the addition of an in-scope deliverable,
	a change of [=Team Contact=],
	or a change of [=Chair=].
	Such changes must nonetheless be announced
	to the [=Advisory Committee=]
	and to <a href="#wgparticipant"> participants in the Working</a> or <a href="#igparticipant">in the Intrest Group</a>,
	and a rationale must be provided.

	The Director's Call for Review of a substantively modified charter
	<em class="rfc2119">must</em> highlight important changes
	(e.g., regarding deliverables or resource allocation)
	and include rationale for the changes.

	As part of the Advisory Committee review of any new or substantively modified Working Group charter,
	any Advisory Committee representative <em class="rfc2119">may</em> request an extended review period.

	Such a request must be submitted with a Member's comments
	in response to the Call for Review.
	Upon receipt of any such request,
	the Director <em class="rfc2119">must</em> ensure
	that the Call for Participation for the Working Group
	occurs at least 60 days
	after the Call for Review of the charter.

<h4 id="cfp">
Call for Participation in a Working Group or Interest Group</h4>

	After [=Advisory Committee review=] of a [=Working Group=] or [=Interest Group=] [=charter=],
	the Director <em class="rfc2119">may</em> issue a Call for Participation to the Advisory Committee.
	Charters <em class="rfc2119">may</em> be amended based on review comments
	before the Director issues a Call for Participation.

	For a new group, this announcement officially creates the group.
	The announcement <em class="rfc2119">must</em> include a reference to the <a href="#WGCharter">charter</a>,
	the name(s) of the group's [=Chair=](s),
	and the name(s) of the [=Team Contact=](s).

	After a Call for Participation,
	any <a href="#member-rep">Member representatives</a>
	and <a href="#invited-expert-wg">Invited Experts</a>
	<em class="rfc2119">must</em> be designated (or re-designated).

	Advisory Committee representatives <em class="rfc2119">may</em> initiate
	an <a href="#ACAppeal">Advisory Committee Appeal</a>
	against a Director's decision to create
	or substantially modify
	a Working Group or Interest Group charter.

<h4 id="charter-extension">
Working Group and Interest Group Charter Extension</h4>

	To extend a Working Group or Interest Group charter
	with no other substantive modifications,
	the Director announces the <dfn id=dfn-charter-extension export lt="charter extension">extension</dfn> to the Advisory Committee.
	The announcement <em class="rfc2119">must</em> indicate the new duration.
	The announcement <em class="rfc2119">must</em> also include rationale for the extension,
	a reference to the <a href="#WGCharter">charter</a>,
	the name(s) of the group's [=Chair=](s),
	the name of the [=Team Contact=],
	and instructions for joining the group.

	After a charter extension,
	Advisory Committee representatives
	and the [=Chair=] are <em class="rfc2119">not required</em> to re-designate <a href="#member-rep">Member representatives</a>
	and <a href="#invited-expert-wg">Invited Experts</a>.

	Advisory Committee representatives <em class="rfc2119">may</em> initiate
	an <a href="#ACAppeal">Advisory Committee Appeal</a> against a Director's decision
	regarding the extension of a Working Group or Interest Group charter.

<h4 id="WGCharter">
Working Group and Interest Group Charters</h4>

	A Working Group or Interest Group <dfn export>charter</dfn>
	<em class="rfc2119">must</em> include all of the following information.

	<ul>
		<li>
			The group's mission
			(e.g., develop a technology or process, review the work of other groups);

		<li>
			The scope of the group's work and criteria for success;

		<li>
			The duration of the group (typically from six months to two years);

		<li>
			The nature of any deliverables (technical reports, reviews of the deliverables of other groups, or software);

		<li>
			For each technical report,
			a designation whether it may,
			when sufficiently mature,
			be published as a [=Expandable Recommendation=],
			as an ordinary [=Recommendation=],
			or as either at the discretion of the [=Working Group=].
			If this information is left out,
			only publishing as an ordinary [=Recommendation=] is allowed.

			Note: Revising the Recommendation using [=proposed additions=]
			(see [[#revised-rec-features]])
			is only allowed for [=Expandable Recommendations=].


		<li>
			Expected milestone dates where available.

			Note: A charter is <em class="rfc2119">not required</em>
			to include schedules for review of other group's deliverables;

		<li>
			The process for the group to approve the release of deliverables
			(including intermediate results);

		<li>
			Any dependencies by groups within or outside of W3C on the deliverables of this group.
			For any dependencies, the charter <em class="rfc2119">must</em> specify
			the mechanisms for communication about the deliverables;

		<li>
			Any dependencies of this group on other groups within or outside of W3C.
			Such dependencies include interactions with
			<a href="https://www.w3.org/Guide/process/charter.html#horizontal-review">W3C Horizontal Groups</a> [[CHARTER]];

		<li>
			The <a href="#confidentiality-levels">level of confidentiality</a>
			of the group's proceedings and deliverables;

		<li>
			Meeting mechanisms and expected frequency;

		<li>
			If known,
			the date of the first <a href="#ftf-meeting">face-to-face meeting</a>.
			The date of the first face-to-face meeting of a proposed group
			<em class="rfc2119">must not</em> be sooner than <span class="time-interval">eight weeks</span>
			after the date of the proposal.

		<li>
			Communication mechanisms to be employed within the group,
			between the group and the rest of W3C,
			and with the general public;

		<li>
			Any voting procedures or requirements
			other than those specified in <a href="#Votes"></a>;

		<li>
			An estimate of the expected time commitment from participants;

		<li>
			The expected time commitment and level of involvement by the Team
			(e.g., to track developments,
			write and edit technical reports,
			develop code,
			or organize pilot experiments).

		<li>
			Intellectual property information.
			What are the intellectual property (including patents and copyright)
			considerations affecting the success of the Group?
			In particular, is there any reason to believe
			that it will be difficult to meet the Royalty-Free licensing goals
			in “Licensing Goals for W3C Specifications”
			in the W3C Patent Policy [[PATENT-POLICY]]?
	</ul>

	See also the charter requirements in “Licensing Goals for W3C Specifications”
	in the W3C Patent Policy [[!PATENT-POLICY]].

	For every Recommendation Track deliverable
	that continues work on a [=Working Draft=] (WD)
	published under any other Charter (including a predecessor group of the same name),
	for which there is an existing [=Patent Review Draft=]
	the description of that deliverable in the proposed charter of the adopting Working Group
	<em class="rfc2119">must</em> provide the following information:

	<ul>
		<li>
			The title,
			stable URL,
			and publication date of the [=Working Draft=]
			or other Recommendation-track document
			that will serve as the basis for work on the deliverable
			(labeled “<dfn export id="adopted-draft">Adopted Draft</dfn>”);

		<li>
			The title,
			stable URL,
			and publication date of the most recent [=Patent Review Draft=]
			that triggered an Exclusion Opportunity
			as per
			<!--[[PATENT-POLICY#lrd-exclusion-pediods|section 5.4.1]] of-->
			the W3C Patent Policy [[!PATENT-POLICY]].
			(labeled “<dfn export id="exclusion-draft">Exclusion Draft</dfn>”); and

		<li>
			The stable URL of the Working Group charter
			under which the Exclusion Draft was published
			(labeled the “<dfn export id="other-charter">Other Charter</dfn>”).
	</ul>

	All of the above data <em class=rfc2119>must</em> be identified
	in the adopting Working Group's charter using the labels indicated.

	The [=Adopted Draft=] and the [=Exclusion Draft=]
	<em class="rfc2119">must</em> each be adopted in their entirety and without any modification.
	The proposed charter <em class="rfc2119">must</em> state
	the dates on which the Exclusion Opportunity
	that arose on publishing the [=Exclusion Draft=]
	began and ended.
	As per “Joining an Already Established Working Group”
	in the W3C Patent Policy [[!PATENT-POLICY]],
	this potentially means that exclusions can only be made immediately on joining a Working Group.

	<p id="ig-charter-participation">An Interest Group charter
	<em class="rfc2119">may</em> include provisions regarding participation,
	including specifying
	that the <span id="ig-mail-only">only requirement for participation (by anyone) in
	the Interest Group is subscription to the Interest Group mailing list</span>.
	This type of Interest Group
	<em class="rfc2119">may</em> have <a href="#public-participant-ig">public participants</a>.

	A charter <em class="rfc2119">may</em> include
	provisions other than those required by this document.
	The charter <em class="rfc2119">should</em> highlight
	whether additional provisions impose constraints
	beyond those of the W3C Process Document
	(e.g., limits on the number of individuals in a Working Group
	who represent the same Member organization or group of <a href="#MemberRelated">related Members</a>).

<h4 id="GeneralTermination">
Working Group and Interest Group Closure</h4>

	A [=Working Group=] or [=Interest Group=] charter specifies a duration for the group.
	The [=Director=] <em class="rfc2119">may</em> decide to close a group
	prior to the date specified in the charter in any of the following circumstances:

	<ul>
		<li>
			There are insufficient resources to produce chartered deliverables
			or to maintain the group,
			according to priorities established within W3C.

		<li>
			The group produces chartered deliverables ahead of schedule.
	</ul>

	The [=Director=] closes a [=Working Group=] or [=Interest Group=]
	by announcement to the [=Advisory Committee=].
	[=Advisory Committee representatives=] <em class="">may</em> initiate
	an <a href="#ACAppeal">Advisory Committee Appeal</a>.

	Closing a Working Group has implications
	with respect to the W3C Patent Policy [[!PATENT-POLICY]].

<h2 id="Reports">
W3C Technical Report Development Process</h2>

	The W3C technical report development process is the set of steps and requirements
	followed by W3C [=Working Groups=] to standardize Web technology.
	The W3C technical report development process is designed to:

	<ul>
		<li>
			support multiple specification development methodologies

		<li>
			maximize [=consensus=] about the content of stable technical reports

		<li>
			ensure high technical and editorial quality

		<li>
			promote consistency among specifications

		<li>
			facilitate royalty-free, interoperable implementations of Web Standards, and

		<li>
			earn endorsement by W3C and the broader community.
	</ul>

	See also “licensing goals for W3C Specifications”
	in the W3C Patent Policy [[PATENT-POLICY]].

<h3 id="rec-advance">
W3C Technical Reports</h3>

	<dfn id="publishing" lt="publish">Publishing</dfn> as used in this document
	refers to producing a version which is listed as a W3C <dfn export>Technical Report</dfn>
	on its <a href="https://www.w3.org/TR/">Technical Reports page https://www.w3.org/TR</a> [[TR]].

	This chapter describes the formal requirements
	for [=publishing=] and maintaining a [=W3C Recommendation=] or [=Note=].

	<dl>
		<dt>Recommendations
		<dd>
			[=Working Groups=] develop technical reports on the [=W3C Recommendation Track=]
			in order to produce normative specifications or guidelines
			as standards for the Web.
			The [=Recommendation Track=] process incorporates requirements for [=wide review=],
			[=adequate implementation experience=],
			and [=consensus=]-building,
			and is subject to the W3C Patent Policy [[PATENT-POLICY]]
			which grants Royalty-Free IPR licenses to implementations.
			See [[#rec-track]] for details.

		<dt>Notes
		<dd>
			Groups can also publish documents as [=W3C Notes=],
			typically either to document information
			other than technical specifications,
			such as use cases motivating a specification
			and best practices for its use,
			or to clarify the status of work that is abandoned.
			See [[#Note]] for details.

		<!--
		<dt>Registries
		-->
	</dl>


	Individual [=Working Groups=] and [=Interest Groups=]
	<em class="rfc2119">should</em> adopt additional processes
	for developing publications,
	so long as they do not conflict with the requirements in this chapter.

<h4 id="general-requirements">
General requirements for Technical Reports</h3>

	Every document published as part of the technical report development process
	<em class="rfc2119 old">must</em> be a public document.
	The <a href="https://www.w3.org/TR/">index of W3C technical reports</a> [[TR]] is available at the W3C Web site.
	W3C strives to make archival documents indefinitely available
	at their original address in their original form.

	Every document published as part of the technical report development process
	<em class="rfc2119 old">must</em> clearly indicate its <a href="#maturity-levels">maturity level</a>,
	and <em id="DocumentStatus" class="rfc2119">must</em> include information about the status of the document.
	This status information:

	<ul>
		<li>
			<em class="rfc2119">must</em> be unique each time a specification is [=published=],

		<li>
			<em class="rfc2119">must</em> state which [=Working Group=] developed the specification,

		<li>
			<em class="rfc2119">must</em> state how to send comments or file bugs,
			and where these are recorded,

		<li>
			<em class="rfc2119">must</em> include expectations about next steps,

		<li>
			<em class="rfc2119">should</em> explain how the technology relates to existing international standards
			and related work inside or outside W3C,
			and

		<li>
			<em class="rfc2119">should</em> explain
			or link to
			an explanation of significant changes from the previous version.
	</ul>

	Every Technical Report published
	as part of the Technical Report development process
	is edited by one or more editors
	appointed by a Group [=Chair=].
	It is the responsibility of these editors to ensure that the decisions of the Group are
	correctly reflected in subsequent drafts of the technical report.
	An editor <em class="rfc2119">must</em> be a participant,
	per <a href="#group-participation"></a>
	in the Group responsible for the document(s) they are editing.

	The Team is <em class="rfc2119">not required</em> to publish a [=Technical Report=]
	that does not conform to the Team's <a href="https://www.w3.org/pubrules/">Publication Rules</a> [[!PUBRULES]]
	(e.g., for <span id="DocumentName">naming</span>,
	status information,
	style,
	and <span id="document-copyright">copyright requirements</span>).
	These rules are subject to change by the Team from time to time.
	The Team <em class="rfc2119">must</em> inform group [=Chairs=]
	and the [=Advisory Committee=]
	of any changes to these rules.

	The primary language for W3C [=Technical Reports=] is English.
	W3C encourages the translation of its [=Technical Reports=].
	<a href="https://www.w3.org/Consortium/Translation/">Information about translations of W3C technical reports</a> [[TRANSLATION]]
	is available at the W3C Web site.

<h4 id="doc-reviews">
Reviews and Review Responsibilities</h4>

	A document is available for review
	from the moment it is first [=published=].
	Working Groups <em class="rfc2119">should</em> [=formally address=]
	<em>any</em> substantive review comment
	about a technical report in a timely manner.

	Reviewers <em class="rfc2119">should</em> send substantive technical reviews as early as possible.
	[=Working Groups=] are often reluctant to make [=substantive changes=] to a mature document,
	particularly if this would cause significant compatibility problems
	due to existing implementation.
	[=Working Groups=] <em class="rfc2119">should</em> record substantive
	or interesting proposals raised by reviews
	but not incorporated into a current specification.

<h5 id="wide-review">
Wide Review</h5>

	The requirements for <dfn id=dfn-wide-review export>wide review</dfn> are not precisely defined by the W3C Process.
	The objective is to ensure that the entire set of stakeholders of the Web community,
	including the general public,
	have had adequate notice of the progress of the [=Working Group=]
	(for example through notices posted to <a href="mailto:public-review-announce@w3.org">public-review-announce@w3.org</a>)
	and were able to actually perform reviews of and provide comments on the specification.
	A second objective is to encourage groups to request reviews
	early enough that comments and suggested changes
	can still be reasonably incorporated in response to the review.
	Before approving transitions,
	the [=Director=] will consider who has been explicitly offered
	a reasonable opportunity to review the document,
	who has provided comments,
	the record of requests to and responses from reviewers,
	especially <a href="https://www.w3.org/Guide/process/charter.html#horizontal-review">W3C Horizontal Groups</a> [[CHARTER]]
	and groups identified as dependencies in the charter
	or identified as <a href="https://www.w3.org/2001/11/StdLiaison.html">liaisons</a> [[LIAISON]],
	and seek evidence of clear communication to the general public
	about appropriate times and which content to review
	and whether such reviews actually occurred.

	For example,
	inviting review of new or significantly revised sections published in Working Drafts,
	and tracking those comments
	and the [=Working Group=]'s responses,
	is generally a good practice which would often be considered positive evidence of wide review.
	[=Working Groups=] <em class="rfc2119">should</em> announce to other W3C Working Groups
	as well as the general public,
	especially those affected by this specification,
	a proposal to enter [=Candidate Recommendation=]
	(for example in approximately 28 days).
	By contrast a generic statement in a document
	requesting review at any time
	is likely not to be considered as sufficient evidence
	that the group has solicited wide review.

	A [=Working Group=] could present evidence that wide review has been received,
	irrespective of solicitation.
	But it is important to note that receiving many detailed reviews
	is not necessarily the same as wide review,
	since they might only represent comment
	from a small segment of the relevant stakeholder community.

<h4 id="correction-classes">
Classes of Changes</h4>

	This document distinguishes the following 4 classes of changes to a specification.
	The first two classes of change are considered <dfn id="editorial-change">editorial changes</dfn>,
	the latter two <dfn id="substantive-change">substantive changes</dfn>.

	<dl>
		<dt>
			1. No changes to text content
		<dd>
			These changes include fixing broken links, style sheets or invalid markup.

		<dt>
			2. Corrections that do not affect conformance
		<dd>
			Changes that reasonable implementers
			would not interpret as changing architectural
			or interoperability requirements
			or their implementation.
			Changes which resolve ambiguities in the specification
			are considered to change (by clarification) the implementation requirements
			and do not fall into this class.
		<dd>
			Examples of changes in this class include
			correcting non-normative code examples
			where the code clearly conflicts with normative requirements,
			clarifying informative use cases or other non-normative text,
			fixing typos or grammatical errors
			where the change does not change implementation requirements.
			If there is any doubt or disagreement
			as to whether requirements are changed,
			such changes do not fall into this class.

		<dt>
			3. Corrections that do not add new features
		<dd>
			These changes <em class="rfc2119">may</em> affect conformance to the specification.
			A change that affects conformance is one that:

			<ul>
				<li>
					makes conforming data, processors, or other conforming agents become non-conforming according to the new version,
					or
				<li>
					makes non-conforming data, processors, or other agents become conforming,
					or
				<li>
					clears up an ambiguity or under-specified part of the specification
					in such a way that data,
					a processor,
					or an agent
					whose conformance was once unclear
					becomes clearly either conforming or non-conforming.
			</ul>

		<dt>
			4. New features
		<dd>
			Changes that add a new functionality, element, etc.
	</dl>

<h4 id="errata">
Errata Management</h4>

	Tracking errors is an important part of a [=Working Group=]'s ongoing care of a [=technical report=];
	for this reason,
	the scope of a [=Working Group=] charter generally allows time
	for work after publication of a [=Recommendation=].
	In this Process Document,
	the term “<dfn lt="erratum | errata" export>erratum</dfn>”
	(plural “errata”) refers to any error
	that can be resolved by one or more changes in classes 1-3
	of section [[#correction-classes]].</a>.

	[=Working Groups=] <em class="rfc2119">must</em> keep
	a public record of errors
	that are reported by readers and implementers
	for [=Recommendations=].
	Such error reports <em class="rfc2119">should</em> be compiled
	no less frequently than quarterly.

	[=Working Groups=] decide how to document errata.
	Such documentation <em class="rfc2119">must</em> identify
	the affected [=technical report=] text
	and describe the error;
	it <em class="rfc2119">may</em> also describe some possible solution(s).
	Readers of the [=technical report=] <em class="rfc2119">should</em> be able easily
	to find and see the errata
	that apply to that specific [=technical report=] with their associated tests.
	Errata <em class="rfc2119">may</em> be documented
	in a separate errata page or tracking system.
	They <em class="rfc2119">may</em>,
	in addition or alternatively,
	be annotated inline
	alongside the affected [=technical report=] text
	or at the start or end of the most relevant section(s).

<h4 id=proposed-changes>
Proposed Changes</h4>

	An [=erratum=] <em class="rfc2119">may</em> be accompanied by an informative,
	<dfn>proposed correction</dfn> approved by the consensus of the [=Working Group=].
	When annotated inline,
	errata--
	including their [=proposed corrections=]--
	must be marked as such,
	are treated as <a href="#correction-classes">class 2 changes</a>,
	and are published accordingly.

	Note: Annotating changes in this way allows more mature documents
	such as [=Recommendations=] and [=Candidate Recommendations=]
	to be updated quickly with the Working Group's most current thinking,
	even when the proposed changes have not yet received
	sufficient review or implementation experience
	to be normatively incorporated into the specification proper.

	A <dfn>proposed addition</dfn> is similar to a [=proposed correction=],
	except that it proposes a new feature
	rather than an error correction.

	[=Proposed corrections=] and [=proposed additions=] are collectively known as
	<dfn lt="proposed change">proposed changes</dfn>.

<h4 id="contributor-license">
License Grants from Non-Participants</h4>

	When a party who is not already obligated under the Patent Policy
	offers a change in class 3 or 4
	(as described in [[#correction-classes]]) to a technical report under this process
	the [=Team=] <em class=rfc2119>must</em> request
	a recorded royalty-free patent commitment;
	for a change in class 4, the Team <em class=rfc2119>must</em> secure such commitment.
	Such commitment <em class=rfc2119>should</em> cover,
	at a minimum,
	all the party's Essential Claims both in the contribution,
	and that become Essential Claims as a result of incorporating the contribution into the draft
	that existed at the time of the contribution,
	on the terms specified in the “W3C Royalty-Free (RF) Licensing Requirements” section of the W3C Patent Policy [[PATENT-POLICY]].


<h3 id=rec-track oldids="recs-and-notes">
The W3C Recommendation Track</h3>

	When advancing a technical report to Recommendation,
	typically a series of [=Working Drafts=] are [=published=],
	each of which refines a document under development
	to complete the scope of work envisioned by a [=Working Group=]'s [=charter=].
	For a technical specification,
	once review suggests the Working Group has met their requirements satisfactorily for a new standard,
	there is a [=Candidate Recommendation=] phase.
	This allows the entire W3C membership to provide feedback
	on whether the specification is appropriate as a [=W3C Recommendation=],
	while the [=Working Group=] formally collects <a href="implementation-experience">implementation experience</a>
	to demonstrate that the specification works in practice.
	The next phase is a [=Proposed Recommendation=],
	to finalize the review of W3C Members.
	If the Director determines that W3C Member review
	supports a specification becoming a standard,
	W3C publishes it as a [=Recommendation=].

	In summary, the <dfn lt="W3C Recommendation Track | REC Track | Recommendation Track">W3C Recommendation Track</dfn> consists of:

	<ol>
		<li>Publication of the [=First Public Working Draft=].
		<li>Publication of zero or more revised [=Working Drafts=].
		<li>Publication of one or more [=Candidate Recommendations=].
		<li>Publication of a [=Proposed Recommendation=].
		<li>Publication as a [=W3C Recommendation=].
		<li>Possibly, publication as an [=Edited Recommendation|Edited=] or [=Amended Recommendation=].
	</ol>

	<div role="region" aria-label="Basic W3C Recommendation Track">
		<pre class=include-raw>
			path: basic-rec-track.svg
		</pre>
	</div>

	This Process defines certain [=Recommendation Track=] publications as <dfn>Patent Review Drafts</dfn>.
	These correspond to the “Last Call Working Draft” of the Patent Policy [[!PATENT-POLICY]].

	W3C <em class="rfc2119">may</em> <a href="#tr-end">end work on a technical report</a> at any time.

	The Director <em class="rfc2119">may</em> decline a request to advance in maturity level,
	requiring a [=Working Group=] to conduct further work,
	and <em class="rfc2119">may</em> require
	the specification to return to a lower <a href="#maturity-levels">maturity level</a>.
	The Director <em class="rfc2119">must</em> inform the <a href="#AC">Advisory Committee</a>
	and [=Working Group=] [=Chairs=] when a [=Working Group=]'s request
	for a specification to advance in maturity level is declined
	and the specification is returned to a [=Working Group=] for further work.

<h4 id="maturity-levels">
Maturity Levels on the Recommendation Track</h4>

	<dl>
		<dt>
			<dfn id="RecsWD" export lt="W3C Working Draft | Working Draft | WD">Working Draft</dfn> (<abbr>WD</abbr>)
		<dd>
			A Working Draft is a document that W3C has [=published=]
			on the <a href="https://www.w3.org/TR/">W3C's Technical Reports page</a> [[TR]]
			for review by the community (including W3C Members), the public,
			and other technical organizations,
			and for simple historical reference.
			Some, but not all, Working Drafts are meant to advance to [=Recommendation=];
			see the <a href="#DocumentStatus">document status section</a> of a Working Draft
			for the group's expectations.
			[=Working Drafts=] do not necessarily represent a [=consensus=] of the [=Working Group=] with respect to their content,
			and do not imply any endorsement by W3C
			or its members beyond agreement to work on a general area of technology.
			Nevertheless the [=Working Group=] decided to adopt the [=Working Draft=]
			as the basis for their work at the time of adoption.
			A [=Working Draft=] is suitable for gathering [=wide review=]
			prior to advancing to the next stage of maturity.

			For all Working Drafts a Working Group:

			<ul>
				<li>
					<em class="rfc2119">should</em> document outstanding issues,
					and parts of the document on which the Working Group does not have consensus,
					and

				<li>
					<em class="rfc2119">may</em> request publication of a Working Draft
					even if its content is considered unstable
					and does not meet all Working Group requirements.
			</ul>

			The first Working Draft of a technical report is called the <dfn export id=fpwd>First Public Working Draft</dfn> (<abbr>FPWD</abbr>),
			and has patent implications as defined in the W3C Patent Policy [[!PATENT-POLICY]].

		<dt>
			<dfn id="RecsCR" export lt="W3C Candidate Recommendation | Candidate Recommendation | CR">Candidate Recommendation</dfn> (<abbr>CR</abbr>)
		<dd>
			A Candidate Recommendation is a document that satisfies the technical
			requirements of the Working Group that produced it and their dependencies,
			or makes substantive corrections
			to a [=Recommendation=] that is not maintained by a [=Working Group=],
			and has already received wide review.
			W3C publishes a Candidate Recommendation to

			<ul>
				<li>
					signal to the wider community that it is time to do a final review
				<li>
					gather <a href="#implementation-experience">implementation experience</a>
			</ul>

			Note: Advancing to [=Candidate Recommendation=] indicates
			that the document is considered complete and fit for purpose,
			and that no further refinement to the text is expected
			without additional implementation experience and testing;
			additional features in a later revision may however be expected.
			A [=Candidate Recommendation=] is expected to be as well-written,
			detailed,
			self-consistent,
			and technically complete
			as a [=Recommendation=],
			and acceptable as such
			if and when the requirements for further advancement are met.

			Candidate Recommendation publications take one of two forms:

			<dl>
				<dt>
					<dfn>Candidate Recommendation Snapshot</dfn>
				<dd>
					A [=Candidate Recommendation Snapshot|Review Snapshot=]
					corresponds to a [=Patent Review Draft=]
					as used in the W3C Patent Policy [[!PATENT-POLICY]].
					Publishing a [=Patent Review Draft=] triggers a Call for Exclusions,
					per [[PATENT-POLICY#patent-exclusions|section 5.5]]
					of the W3C Patent Policy.

					Publication as a Candidate Recommendation Snapshot
					requires approval of either a [=Transition Request=]
					(for the first Candidate Recommendation publication from another maturity level)
					or an [=Update Request=]
					(for subsequent Candidate Recommendation Snapshots).
				<dt>
					<dfn>Candidate Recommendation Draft</dfn>
				<dd>
					An [=Candidate Recommendation Draft=]
					is published on the <a href="https://www.w3.org/TR/">W3C's Technical Reports page</a> [[TR]]
					to integrate changes from the previous Candidate Recommendation Snapshot
					that the Working Group intends to include
					in a subsequent Candidate Recommendation Snapshot.
					This allows for wider review of the changes
					and for ease of reference to the integrated specification.

					Any changes published directly into a Candidate Recommendation Draft
					should be at the same level of quality as a Candidate Recommendation Reveiw Snapshot.
					However, the process requirements are minimized
					so that the Working Group can easily keep the specification up to date.

					A Candidate Recommendation Draft
					<em>does not</em> provide an exclusion opportunity
					instead, it is considered a [=Working Draft=]
					for the purpose of the <a href="https://www.w3.org/Consortium/Patent-Policy">W3C Patent Policy</a> [[PATENT-POLICY]].
			</dl>

			A <dfn>Rescinded Candidate Recommendation</dfn>
			is a [=Candidate Recommendation=] in which significant problems have been discovered
			such that W3C cannot endorse it or continue work on it,
			for example due to burdensome patent claims that affect implementers and cannot be resolved
			(see [[PATENT-POLICY#sec-PAG-conclude|section 5.5.5.4]] of the W3C Patent Policy [[PATENT-POLICY]]).
			There is no path to restoration for a [=Rescinded Candidate Recommendation=].
			See [[PATENT-POLICY#283|section 2.8.3]]
			of the W3C Patent Policy [[!PATENT-POLICY]]
			for implication on patent licensing obligations.

		<dt>
			<dfn export id="RecsPR" lt="W3C Proposed Recommendation | Proposed Recommendation | PR">Proposed Recommendation</dfn> (<abbr>PR</abbr>)
		<dd>
			A Proposed Recommendation is a document
			that has been accepted by the W3C
			as of sufficient quality to become a [=W3C Recommendation=].
			This phase triggers formal review by the [=Advisory Committee=],
			who <em class="rfc2119">may</em> recommend
			that the document be [=published=] as a [=W3C Recommendation=],
			returned to the [=Working Group=] for further work,
			or abandoned.
			[=Substantive changes=] <em class="rfc2119">must not</em> be made to a [=Proposed Recommendation=]
			except by [=publishing=] a new [=Working Draft=] or [=Candidate Recommendation=].

			When the intended next step is a [=Expandable Recommendation=],
			the [=Proposed Recommendation=] is deemed to be a <dfn>Proposed Expandable Recommendation</dfn>,
			and <em class=rfc2119>must</em> identify itself as such.
			In all other respects,
			[=Proposed Recommendations=] and [=Proposed Expandable Recommendations=] are identical.

		<dt>
			<dfn id="RecsW3C" export lt="W3C Recommendation | Recommendation | REC">W3C Recommendation</dfn> (<abbr>REC</abbr>)
		<dd>
			A W3C Recommendation is a specification
			or set of guidelines
			or requirements that,
			after extensive [=consensus=]-building,
			has received the endorsement of the W3C and its Members.
			W3C recommends the wide deployment
			of its Recommendations as standards for the Web.
			The W3C Royalty-Free IPR licenses
			granted under the W3C Patent Policy [[PATENT-POLICY]]
			apply to [=W3C Recommendations=].
			As technology evolves,
			a [=W3C Recommendation=] may become:
			<dl>
				<dt>
					An <dfn export>Edited Recommendation</dfn>
				<dd>
					A [=Working Group=] may make <a href="#editorial-change">editorial</a>
					to a [=Recommendation=],
					and produce a new version
					which W3C [=publishes=] as a <a href="#revising-rec">revised edition</a>
					of the [=Recommendation=].
				<dt>
					An <dfn id="rec-amended" export>Amended Recommendation</dfn>
				<dd>
					An Amended Recommendation is a [=Recommendation=] that is amended to include
					<a href="#correction-classes">substantive changes that do not add new features</a>,
					and is produced by the W3C
					at a time when the [=Recommendation=] does not fit within the charter of any active Working Group.
					Since the [=W3C team=] rather than a [=Working Group=] moves it through the Process,
					there are implications regarding the scope of Royalty-Free IPR licenses
					granted under the W3C Patent Policy [[PATENT-POLICY]].

				<dt>
					A <dfn export id="RecsSup">Superseded Recommendation</dfn>
				<dd>
					A Superseded Recommendation is a specification
					that has been replaced by a newer version
					that the W3C recommends for new adoption.
					An [=Obsolete Recommendation|Obsolete=] or Superseded specification
					has the same status as a [=W3C Recommendation=]
					with regards to W3C Royalty-Free IPR Licenses granted under the Patent Policy.

					Note: When a Technical Report which had previously been published as a [=Recommendation=]
					is again published as a [=Recommendation=]
					after following the necessary steps to <a href="#revising-rec">revise</a> it,
					the latest version replaces the old one,
					without the need to invoke the the steps of [[#rec-rescind]]:
					it is the same document, updated.
					Explicitly declaring a documented superseded, using the process documented in [[#rec-rescind]],
					is intended for cases where a [=Recommendation=] is superseded by a separate [=Technical Report=]
					(or a by a document managed outside of W3C).

				<dt>
					An <dfn export id="RecsObs">Obsolete Recommendation</dfn>
				<dd>
					An Obsolete Recommendation is a specification
					that the W3C has determined lacks sufficient market relevance
					to continue recommending it for implementation,
					but which does not have fundamental problems
					that would require it to be [=Rescinded Recommendation|Rescinded=].
					If an Obsolete specification gains sufficient market relevance,
					the W3C may decide to restore it to [=Recommendation=] status.

				<dt>
					<dfn export id="RescindedRec">Rescinded Recommendation</dfn>
				<dd>
					A Rescinded Recommendation is an entire Recommendation that W3C no longer endorses,
					and believes is unlikely to ever be restored to [=Recommendation=] status.
					See also [[PATENT-POLICY#283|section 2.8.3]]
					of the W3C Patent Policy [[!PATENT-POLICY]].
			</dl>

		<dt>
			<dfn id="Extensible" export lt="W3C Expandable Recommendation | Expandable Recommendation | Extensible REC">W3C Expandable Recommendation</dfn> (<abbr>Extensible REC</abbr>)
		<dd>
			A Expandable Recommendation is identical to a [=Recommendation=] in most respects.
			All statements about [=Recommendations=] in this document
			apply to [=Expandable Recommendations=] as well.
			For instance, the criteria for advancement are the same,
			the steps to <a href="#rec-rescind">declare a W3C Recommendation Rescinded, Obsolete or Superseded</a> are the same,
			a [=Expandable Recommendation=] is a [=Recommendation=] for the purpose of the Patent Policy [[!PATENT-POLICY]],
			etc.

			However, new features (i.e. <a href="#correction-classes">class 4 changes</a>)
			can be incrementally added to a [=Expandable Recommendation=],
			which is not possible for an ordinary [=Recommendation=].
			This is enabled by the mechanism
			described in <a href="#revised-rec-features"></a>.
			Publishing a technical report as a [=Expandable Recommendation=]
			is only possible when explicitly allowed in the [=Working Group=]'s [=Charter=].
	</dl>

	Only sufficiently technically mature work should be advanced.

	Note: Should faster advancement to meet scheduling considerations be desired,
	this can be achieved by reducing the scope of the technical report to a subset that is adequately mature and deferring
	less stable features to other technical reports.

	When publishing an updated version of an existing [=Candidate Recommendation=] or [=Recommendation=],
	technical reports are expected to meet the same maturity criteria as when they are first published under that status.
	However, in the interest of replacing stale documents with improved ones in a timely manner,
	if flaws have been discovered in the technical report after its initial publication as a [=CR=] or [=REC=]
	that would have been severe enough to reject that publication had they be known in time,
	it is also permissible to publish an updated [=CR=] or [=REC=] following the usual process,
	even if only some of these flaws have been satisfactorily addressed.

	[=Working Groups=] and [=Interest Groups=] <em class="rfc2119">may</em> make available [=Editor's drafts=].
	<dfn export lt="Editor's Draft | Editor Draft | ED">Editor's drafts</dfn> (<abbr>ED</abbr>) have no official standing whatsoever,
	and do not necessarily imply consensus of a [=Working Group=] or [=Interest Group=],
	nor are their contents endorsed in any way by W3C.

<h4 id="implementation-experience">
Implementation Experience</h4>

	Implementation experience is required to show that a specification is sufficiently clear,
	complete,
	and relevant to market needs,
	to ensure that independent interoperable implementations
	of each feature of the specification will be realized.
	While no exhaustive list of requirements is provided here,
	when assessing that there is <dfn id="adequate-implementation">adequate implementation experience</dfn>
	the [=Director=] will consider (though not be limited to):

	<ul>
		<li>
			is each feature of the current specification implemented,
			and how is this demonstrated?

		<li>
			are there independent interoperable implementations of the current specification?

		<li>
			are there implementations created by people other than the authors of the specification?

		<li>
			are implementations publicly deployed?

		<li>
			is there implementation experience
			at all levels of the specification's ecosystem
			(authoring, consuming, publishing…)?

		<li>
			are there reports of difficulties or problems with implementation?
	</ul>

	Planning and accomplishing a demonstration of (interoperable) implementations can be very time consuming.
	Groups are often able to work more effectively
	if they plan how they will demonstrate interoperable implementations
	early in the development process;
	for example, developing tests in concert with implementation efforts.


<h4 id="transition-reqs">
Advancement on the Recommendation Track</h4>

	For <em>all</em> requests to advance a specification
	to a new maturity level other than [=Note=]
	(called <dfn id=trans-req export>Transition Requests</dfn>),
	the Working Group:

	<ul>
		<li>
			<em class="rfc2119">must</em> record the group's decision to request advancement.

		<li>
			<em class="rfc2119">must </em> obtain [=Director=] approval.

		<li>
			<em class="rfc2119 ">must</em> publicly document all new features
			(<a href="#correction-classes">class 4 changes</a>) to the technical report
			since the previous publication.

		<li>
			<em class="rfc2119">must</em> publicly document if other substantive changes
			(<a href="#correction-classes">class 3 changes</a>) have been made,
			and <em class="rfc2119">should</em> document the details of such changes.

		<li>
			<em class="rfc2119">should</em> publicly document if [=editorial changes=] changes have been made,
			and <em class="rfc2119">may</em> document the details of such changes.
		<li>
			<em class="rfc2119">must</em> [=formally address=] all issues
			raised about the document since the previous <a href="#maturity-levels">maturity level</a>.

		<li>
			<em class="rfc2119">must</em> provide public documentation of any [=Formal Objections=].

		<li>
			<em class="rfc2119">should</em> report which, if any, of the [=Working Group=]'s requirements
			for this document have changed since the previous step.

		<li>
			<em class="rfc2119">should</em> report any changes in dependencies with other groups.

		<li>
			<em class="rfc2119">should</em> provide information about implementations known to the [=Working Group=].
	</ul>

	For a [=First Public Working Draft=] there is no “previous maturity level”,
	so many requirements do not apply,
	and approval is normally fairly automatic.
	For later stages,
	especially transition to [=Candidate Recommendation|Candidate=] or [=Proposed Recommendation=],
	there is usually a formal review meeting
	to ensure the requirements have been met before [=Director=]'s approval is given.

	[=Transition Requests=] to [=First Public Working Draft=]
	or [=Candidate Recommendation=]
	will not normally be approved
	while a [=Working Group=]'s [=charter=] is undergoing or awaiting a [=Director=]'s decision
	on an [=Advisory Committee Review=].</p>

<h4 id="update-reqs">
Updating Mature Publications on the Recommendation Track</h4>

	Certain requests to re-publish a specification
	within its current maturity level
	(called <dfn export>Update Requests</dfn>)
	require [=Director=] approval.
	For such [=update requests=], the Working Group:

	<ul>
		<li>
			<em class="rfc2119">must</em> record the group's decision to request the update.

		<li>
			<em class="rfc2119">must</em> show that the changes have received [=wide review=].

		<li>
			<em class="rfc2119">must </em> obtain [=Director=] approval,
			or fulfill the criteria for [[#streamlined-update]].

		<li>
			<em class="rfc2119">must</em> provide public documentation of any [=Formal Objections=].

		<li>
			<em class="rfc2119 ">must</em> publicly document of all new features
			(<a href="#correction-classes">class 4 changes</a>) to the technical report
			since the previous publication.

		<li>
			<em class="rfc2119">must</em> publicly document if other substantive changes
			(<a href="#correction-classes">class 3 changes</a>) have been made,
			and <em class="rfc2119">should</em> document the details of such changes.

		<li>
			<em class="rfc2119">should</em> publicly document if [=editorial changes=] changes have been made,
			and <em class="rfc2119">may</em> document the details of such changes.

		<li>
			<em class="rfc2119">must</em> show that the revised specification
			meets all [=Working Group=] requirements,
			or explain why the requirements have changed or been deferred,

		<li>
			<em class="rfc2119">should</em> report which, if any, of the [=Working Group=]'s requirements
			for this document have changed since the previous step.

		<li>
			<em class="rfc2119">should</em> report any changes in dependencies with other groups.

		<li>
			<em class="rfc2119">should</em> provide information about implementations known to the [=Working Group=].
	</ul>

	There is usually a formal review meeting
	to ensure the requirements have been met before [=Director=]'s approval is given.

	Note: [=Update request=] approval is expected to be fairly simple
	compared to getting approval for a [=transition request=].

	The [=Director=] <em class="rfc2119">must</em> announce the publication
	of the revised specification
	to other W3C groups and the Public.


<h5 id="streamlined-update">
Streamlined Publication Approval</h5>

	Note: These criteria are intentionally stricter than
	the general requirements for an [=update request=].
	This is in order to minimize ambiguities and the need for expert judgement,
	and to make self-evaluation practical.

	In order to streamline the publication process in non-controversial cases,
	approval to an [=update request=] is automatically granted without formal review
	when the following <em>additional</em> criteria are fulfilled:

	<ul>
		<li>
			There <em class=rfc2119>must</em> have been no changes to [=Working Group=] requirements about this document.

		<li>
			For each of the
			<a href="https://www.w3.org/Guide/process/charter.html#horizontal-review">W3C Horizontal Groups</a> [[CHARTER]],
			if the Horizontal Review Group has made available a set criteria
			under which their review is not necessary,
			the [=Working Group=] <em class=rfc2119>must</em> document that these criteria have been fulfilled.
			Otherwise, the [=Working Group=] <em class=rfc2119>must</em> show
			that review from that group has been solicited and received.

		<li>
			No [=Formal Objection=] has been registered against the document.

		<li>
			The [=Working Group=] <em class=rfc2119>must</em>
			have [=formally addressed=]:
			* all issues raised against the document that resulted in changes since the previous publication
			* all issues raised against changes since the previous publication
			* all issues raised against the document that were closed since the previous publication with no change to the document

			The response to each of these issues <em class=rfc2119>must</em> be to the satisfaction
			of the person who raised it:
			their proposal has been accepted,
			or a compromise has been found,
			or they accepted the Working Group's rationale for rejecting it.

			Note: This is stricter than the general Transition Request criteria.
	</ul>

	Additionally, for updates to [=Recommendations=]
	with <a href="#revised-rec-substantive">substative changes</a>
	or <a href="#revised-rec-features">with new features</a>:

	<ul>

		<li>
			Changes to the document are limited to
			[=proposed corrections=] that were included in a [=Last Call for Review of Proposed Corrections=]
			possibly combined with <a href=#correction-classes>class 1 or 2 changes</a>,
			and/or (in the case of an [=Expandable Recommendation=])
			[=proposed additions=] that were included in a [=Last Call for Review of Proposed Additions=].

		<li>
			The [=Working Group=] <em class=rfc2119>must</em> show
			that all changes have been implemented in at least 2 distinct products by 2 different implementers,
			as evidenced by passing tests of a test suite
			providing extensive coverage of the changes,
			or an alternative streamlined approval implementation requirement
			described in the [=working Group=] charter has been met.

			Note: This is stricter than the general criteria for [=adequate implementation experience=].
	</ul>

	The [=Working Group=] must provide written evidence for these claims,
	and the [=Team=] must make these answers publicly and permanently available.

	After publication,
	if an AC Representative
	or Team member
	doubts that the evidence presented supports the claims,
	they <em class=rfc2119>may</em> request that a formal review meeting be convened post facto.
	If that review finds that the requirements were not fulfilled,
	the Team <em class=rfc2119>may</em> revert the changes
	by updating in place the status section to indicate that it has been reverted,
	and by republishing the previously approved version of the technical report.

<h4 id="first-wd">
Publishing a First Public Working Draft</h4>

	To publish the [=First Public Working Draft=] of a document,
	a [=Working Group=] <em class="rfc2119">must</em> meet the applicable <a href="#transition-reqs">requirements for advancement</a>.

	The [=Director=] <em class="rfc2119">must</em> announce
	the publication of a [=First Public Working Draft=]
	to other W3C groups and to the public.

<h4 id="revising-wd" oldids="revised-wd">
Revising a Working Draft</h4>

	A [=Working Group=] <em class="rfc2119">should</em> [=publish=] a [=Working Draft=]
	to the W3C Technical Reports page
	when there have been significant changes
	to the previous published document
	that would benefit from review beyond the Working Group.

	If 6 months elapse without significant changes to a specification,
	a Working Group <em class="rfc2119">should</em> publish a revised [=Working Draft=],
	whose status section <em class="rfc2119">should</em> indicate reasons for the lack of change.

	To publish a revision of a Working draft, a Working Group:

	<ul>
		<li>
			<em class="rfc2119">must</em> record the group's decision to request publication.
			[=Consensus=] is not required,
			as this is a procedural step,

		<li>
			<em class="rfc2119">must</em> provide public documentation
			of [=substantive changes=] to the technical report
			since the previous [=Working Draft=],

		<li>
			<em class="rfc2119">should</em> provide public documentation
			of significant [=editorial changes=] to the technical report
			since the previous step,

		<li>
			<em class="rfc2119">should</em> report which,
			if any,
			of the Working Group's requirements for this document
			have changed since the previous step,

		<li>
			<em class="rfc2119">should</em> report any changes in dependencies with other groups,
	</ul>

	Possible next steps for any Working Draft:

	<ul>
		<li>Revised <a href="#revising-wd">Working Draft</a>
		<li><a href="#transition-cr">Candidate Recommendation</a>
		<li><a href="#Note">Working Group Note</a>
	</ul>

<h4 id="transition-cr" oldids="candidate-rec last-call">
Transitioning to Candidate Recommendation</h4>

	To publish a [=Candidate Recommendation=],
	in addition to meeting the <a href="#transition-reqs">requirements for advancement</a>
	a [=Working Group=],
	or in the case of a candidate [=Amended Recommendation=]
	(a document intended to become an [=Amended Recommendation=]),
	the W3C:

	<ul>
		<li>
			<em class="rfc2119">must</em> show that the specification
			has met all [=Working Group=] requirements,
			or explain why the requirements have changed or been deferred,

		<li>
			<em class="rfc2119">must</em> document changes to dependencies during the development of the specification,

		<li>
			<em class="rfc2119">must</em> document
			how [=adequate implementation experience=] will be demonstrated,

		<li>
			<em class="rfc2119">must</em> specify the deadline for comments,
			which <em class="rfc2119">must</em> be <strong>at least</strong> 28 days after publication,
			and <em class="rfc2119">should</em> be longer for complex documents,

		<li>
			<em class="rfc2119">must</em> show that the specification has received [=wide review=], and

		<li>
			<em class="rfc2119">may</em> identify features in the document as <dfn export>at risk</dfn>.
			These features <em class="rfc2119">may</em> be removed
			before advancement to [=Proposed Recommendation=]
			without a requirement to publish a new [=Candidate Recommendation=].
	</ul>

	The first Candidate Recommendation publication
	after approval of a [=Transition Request=]
	is always a [=Candidate Recommendation Snapshot=].
	The [=Director=] <em class="rfc2119">must</em> announce
	the publication of the [=Candidate Recommendation Snapshot=]
	to other W3C groups
	and to the public.

	Possible next steps after a [=Candidate Recommendation Snapshot=]:

	<ul>
		<li>Return to <a href="#revising-wd">Working Draft</a></li>
		<li>A revised <a href="#publishing-crrs">Candidate Recommendation Snapshot</a></li>
		<li>A revised <a href="#publishing-crud">Candidate Recommendation Draft</a></li>
		<li><a href="#transition-pr">Proposed Recommendation</a></li>
		<li><a href="#Note">Working Group Note</a></li>
	</ul>

	[=Advisory Committee=] representatives <em class="rfc2119">may</em> initiate an
	<a href="#ACAppeal">Advisory Committee Appeal</a> of the decision to advance the technical report.

<h4 id=revising-cr oldids="revised-cr">
Revising a Candidate Recommendation</h4>

<h5 id="publishing-crrs">
Publishing a [=Candidate Recommendation Snapshot=]</h5>

	If there are any [=substantive changes=] made to a [=Candidate Recommendation=]
	since the previous [=Candidate Recommendation Snapshot=]
	other than to remove features explicitly identified as [=at risk=],
	the [=Working Group=] <em class="rfc2119">must</em>
	meet the requirements of an [=update request=] in order to republish.

	In addition the Working Group:

	<ul>
		<li>
			<em class="rfc2119">must</em> specify the deadline for further comments,
			which <em class="rfc2119">must</em> be <strong>at least</strong> 28 days after publication,
			and <em class="rfc2119">should</em> be longer for complex documents,

		<li>
			<em class="rfc2119">may</em> identify features in the document as [=at risk=].
			These features <em class="rfc2119">may</em> be removed
			before advancement to [=Proposed Recommendation=]
			without a requirement to publish a new [=Candidate Recommendation=].
	</ul>

	The [=Director=] <em class="rfc2119">must</em> announce
	the publication of a revised [=Candidate Recommendation Snapshot=]
	to other W3C groups
	and to the public.

	To provide timely updates and patent protection,
	a [=Candidate Recommendation Snapshot=]
	<em class=rfc2119>should</em> be published
	within 24 months of the Working Group accepting
	any proposal for a substantive change
	(and preferably sooner).
	To make scheduling reviews easier,
	a [=Candidate Recommendation Snapshot=] <em class=rfc2119>should not</em> be published
	more often than approximately once every 6 months.

	Note: [=Substantive changes=] trigger a new Exclusion Opportunity
	per [[PATENT-POLICY#patent-exclusions|section 5.5]]
	of the W3C Patent Policy [[!PATENT-POLICY]].

<h5 id="publishing-crud">
Publishing a [=Candidate Recommendation Draft=]</h5>

	A [=Working Group=] <em class="rfc2119">should</em> [=publish=] a [=Candidate Recommendation Draft|Update Draft=]
	to the W3C Technical Reports page
	when there have been significant changes
	to the previous published document
	that would benefit from review beyond the Working Group.


	To publish a revision of a [=Candidate Recommendation Draft=],
	a Working Group:

	<ul>
		<li>
			<em class="rfc2119">must</em> record the group's decision to request publication,

		<li>
			<em class="rfc2119">must</em> provide public documentation
			of [=substantive changes=] to the technical report
			since the previous [=Candidate Recommendation Snapshot=],

		<li>
			<em class="rfc2119">should</em> provide public documentation
			of significant [=editorial changes=] to the technical report
			since the previous [=Candidate Recommendation Snapshot=],

		<li>
			<em class="rfc2119">should</em> document outstanding issues,
			and parts of the document on which the Working Group does not have consensus,

		<li>
			<em class="rfc2119">should</em> report which,
			if any,
			of the Working Group's requirements for this document
			have changed since the previous step,

		<li>
			<em class="rfc2119">should</em> report any changes in dependencies with other groups.

	</ul>

	Note: A Working Group <em>does not</em> need to
	meet the requirements of a [=Candidate Recommendation Snapshot=] [=update request=]
	in order to publish a [=Candidate Recommendation Draft=].

	Possible next steps after a [=Candidate Recommendation Draft=]:

	<ul>
		<li>Return to <a href="#revising-wd">Working Draft</a></li>
		<li>A revised <a href="#publishing-crrs">Candidate Recommendation Snapshot</a></li>
		<li>A revised <a href="#publishing-crud">Candidate Recommendation Draft</a></li>
		<li><a href="#transition-pr">Proposed Recommendation</a>,
		if there are no [=substantive change=] other than dropping [=at risk=] features</li>
		<li><a href="#Note">Working Group Note</a></li>
	</ul>


<h4 id=transition-pr oldids="rec-pr">
Transitioning to Proposed Recommendation</h4>

	In addition to meeting the <a href="#transition-reqs">requirements for advancement</a>,

	<ul>
		<li>
			The status information <em class="rfc2119">must</em> specify the deadline for [=Advisory Committee review=],
			which <em class="rfc2119">must</em> be <strong>at least</strong> 28 days
			after the publication of the [=Proposed Recommendation=]
			and <em class="rfc2119">should</em> be at least 10 days
			after the end of the last Exclusion Opportunity
			per [[PATENT-POLICY#patent-exclusions|section 5.5]]
			of the W3C Patent Policy [[!PATENT-POLICY]].
	</ul>

	A Working Group,
	or for a proposed [=Amended Recommendation=], the W3C:

	<ul>
		<li>
			<em class="rfc2119">must</em> show [=adequate implementation experience=]
			except where an exception is approved by the [=Director=],
		<li>
			<em class="rfc2119">must</em> show that the document has received [=wide review=],

		<li>
			<em class="rfc2119">must</em> show that all issues
			raised during the [=Candidate Recommendation=] review period
			other than by [=Advisory Committee representatives=]
			acting in their formal [=AC representative=] role
			have been [=formally addressed=],

		<li>
			<em class="rfc2119">must </em>identify any substantive issues
			raised since the close of the [=Candidate Recommendation=] review period,

		<li>
			<em class="rfc2119">must</em> indicate whether the intended next step is [=Expandable Recommendation=],
			in which case the [=Proposed Recommendation=] <em class=rfc2119>must</em> be identified as a
			[=Proposed Expandable Recommendation=].
			This is only possible when the [=Charter=] allows it for that Technical Report.

		<li>
			<em class=rfc2119>must not</em> have any [=substantive changes=]
			since the most recent [=Candidate Recommendation Snapshot=],
			other than dropping features identified [=at risk=].

		<li>
			<em class="rfc2119">may</em> have removed features
			identified in the [=Candidate Recommendation Snapshot=] document as [=at risk=]
			without republishing the specification as a [=Candidate Recommendation Snapshot=].
	</ul>

	The Director:

	<ul>
		<li>
			<em class="rfc2119">must</em> announce the publication of a [=Proposed Recommendation=]
			to the <a href="#AC">Advisory Committee</a>,
			and <em class="rfc2119">must</em> begin an [=Advisory Committee Review=]
			on the question of whether the specification is appropriate to [=publish=] as a [=W3C Recommendation=].

		<li>
			<em class="rfc2119">may</em> approve a [=Proposed Recommendation=]
			with minimal <a href="#implementation-experience">implementation experience</a>
			where there is a compelling reason to do so.
			In such a case, the [=Director=] <em class="rfc2119">should</em> explain the reasons for that decision.
	</ul>

	Since a [=W3C Recommendation=] <em class="rfc2119">must not</em> include any [=substantive changes=]
	from the [=Proposed Recommendation=] it is based on,
	to make any [=substantive change=] to a [=Proposed Recommendation=]
	the [=Working Group=] <em class="rfc2119">must</em> return the specification to [=Candidate Recommendation=]
	or [=Working Draft=].

	Possible Next Steps:

	<ul>
		<li>
			Return to <a href="#revising-wd">Working Draft</a>

		<li>
			Return to <a href="#transition-cr">Candidate Recommendation</a>

		<li>
			<a href="#transition-rec">Recommendation status</a>,
			including [=Edited Recommendation|Edited=] or [=Amended Recommendation=]
			(The expected next step).

		<li>
			<a href="#Note">Working Group Note</a>
	</ul>

	[=Advisory Committee representatives=] <em class="rfc2119">may</em> initiate an [=Advisory Committee Appeal=]
	of the decision to advance the technical report.

<h4 id="transition-rec" oldids="rec-publication">
Transitioning to W3C Recommendation</h4>

	The decision to advance a document to [=Recommendation=]
	is a [=W3C Decision=].

	In addition to meeting the <a href="#transition-reqs">requirements for advancement</a>,

	<ul>
		<li>
			A [=Recommendation=] <em class="rfc2119">must</em> identify where errata are tracked, and

		<li>
			A [=Recommendation=] <em class="rfc2119">must not</em> include any [=substantive changes=]
			from the [=Proposed Recommendation=]
			on which it is based.

		<li>
			If there was any [=dissent=] in Advisory Committee reviews,
			the [=Director=] <em class="rfc2119">must</em> publish the substantive content of the dissent
			to W3C and the general public,
			and <em class="rfc2119">must</em> [=formally address=] the comment
			at least 14 days before publication as a [=W3C Recommendation=].

		<li>
			[=Advisory Committee representatives=] <em class="rfc2119">may</em> initiate
			an [=Advisory Committee Appeal=]
			of the [=W3C decision=]

		<li>
			The [=Director=] <em class="rfc2119">must</em> announce the publication of a [=W3C Recommendation=]
			to <a href="#AC">Advisory Committee</a>,
			other W3C groups
			and to the public.
	</ul>

	Possible next steps:
	A [=W3C Recommendation=] normally retains its status indefinitely.
	However it <em class="rfc2119">may</em> be:

	<ul>
		<li>
			republished as an <a href="#revising-rec">Edited or Amended Recommendation</a>, or

		<li>
			republished as a <a href="#transition-cr">Candidate Recommendation</a>
			to be developed as an [=Edited Recommendation|Edited=] or [=Amended Recommendation=], or

		<li>
			declared <a href="#rec-rescind">superseded or obsolete</a>, or

		<li>
			<a href="#rec-rescind">rescinded</a>.
	</ul>

<h4 id=revising-rec oldids="rec-modify revised-rec">
Revising a W3C Recommendation</h4>

	This section details the process for making changes to a [=Recommendation=].

	<div role="region" aria-label="Revising a W3C Recommendation">
		<pre class=include-raw>
			path: revising-rec.svg
		</pre>
	</div>

<h5 id="revised-rec-markup">
Revising a Recommendation: Markup Changes</h5>

	A [=Working group=] <em class="rfc2119">may</em> request republication of a [=Recommendation=],
	or if there is no [=Working Group=] chartered to maintain a [=Recommendation=]
	W3C <em class="rfc2119">may</em> republish the [=Recommendation=],
	to make corrections that do not result
	in any changes to the text of the specification.
	(See <a href="#correction-classes">class 1 changes</a>.)

<h5 id="revised-rec-editorial">
Revising a Recommendation: Editorial Changes</h5>

	[=Editorial changes=] to a [=Recommendation=] require no technical review of the proposed changes.
	A [=Working Group=],
	provided there are no votes against the resolution to publish,
	<em class="rfc2119">may</em> request publication of a [=Recommendation=]
	or W3C <em class="rfc2119">may</em> publish a [=Recommendation=]
	to make this class of change without passing through earlier maturity levels.
	Such publications are called [=Edited Recommendations=].
	(See <a href="#correction-classes">class 2 changes</a>.)

<h5 id="revised-rec-substantive">
Revising a Recommendation: Substantive Changes</h5>

	A [=proposed correction=] can be made normative
	and be folded into the main text of the [=Recommendation=],
	once it has satisfied all the same criteria
	as the rest of the [=Recommendation=],
	including review by the community to ensure
	the technical and editorial soundness of the proposed change.
	To validate this, the [=Working Group=] must request
	a [=Last Call for Review of Proposed Changes=],
	followed by an [=update request=].
	See [[#change-review]]

	Alternatively,
	a [=Working Group=] <em class="rfc2119">may</em> incorporate the changes
	and <a href="#transition-cr">publish as a Candidate Recommendation</a>,
	or if no [=Working Group=] is chartered to maintain a [=Recommendation=]
	W3C <em class="rfc2119">may</em> publish a <a href="#transition-cr">candidate Amended Recommendation</a>,
	and advance the specification from that state.
	If the publication was requested by the [=W3C team=] in the absence of a [=Working Group=],
	the resulting [=Recommendation=] will be called an [=Amended Recommendation=].
	(See <a href="#correction-classes">class 3 changes</a>.)

<h5 id="revised-rec-features">
Revising a Recommendation: New Features</h5>

	To make changes which introduce a new feature or features
	(see <a href="#correction-classes">class 4 changes</a>)
	to a [=Recommendation=] which is not a [=Expandable Recommendation=],
	W3C <em class="rfc2119">must</em>
	follow the full process of <a href="#rec-advance">advancing a technical report to Recommendation</a>
	beginning with a new [=First Public Working Draft=].

	However,
	new features can also be added to a [=Expandable Recommendation=]
	using [=proposed additions=].
	A [=proposed addition=] can be made normative
	and be folded into the main text of an [=Expandable Recommendation=],
	once it has satisfied all the same criteria
	as the rest of the [=Recommendation=],
	including review by the community to ensure
	the technical and editorial soundness of the proposed change.
	To validate this, the [=Working Group=] must request
	a [=Last Call for Review of Proposed Changes=],
	followed by an [=update request=].
	See [[#change-review]]

<h5 id=change-review>
Incorporating Proposed Changes</h5>

	A <dfn id=last-call>Last Call for Review of Proposed Changes</dfn>
	verifies acceptance by the W3C community of [=proposed changes=]
	by combining an [=AC Review=] with a patent exclusion opportunity.

	The combination of the existing [=Recommendation=]
	with the [=proposed changes=] included in the [=Last Call for Review of Proposed Changes=]
	is considered a [=Patent Review Draft=] of the proposed changes
	for the purposes of the Patent Policy [[PATENT-POLICY]].
	Also, the review initiated by the [=Last Call for Review of Proposed Additions=]
	is an [=Advisory Committee Review=].

	The <a>Last Call for Review of Proposed Changes</a>
	must be announced to other W3C groups, the public, and the [=Advisory Committee=].
	The announcement <em class="rfc2119">must</em>:
	<ul>
		<li>
			Identify whether this is a
			<dfn>Last Call for Review of Proposed Corrections</dfn>,
			<dfn>Last Call for Review of Proposed Additions</dfn>,
			or <dfn>Last Call for Review of Proposed Corrections and Additions</dfn>.

		<li>
			Identify the specific proposed changes under review.

		<li>
			Specify the deadline for review comments,
			which must not be any sooner than 60 days from the Call for Review.

		<li>
			Solicit review and, if it does not already have it, implementation experience.
	</ul>

	Note: [=Last Call for Review of Proposed Additions=] and
	[=Last Call for Review of Proposed Corrections and Additions=]
	can only be issued for [=Expandable Recommendations=].

	A [=Working Group=] <em class="rfc2119">may</em> batch
	multiple [=proposed changes=] into a single <a>Last Call for Review of Proposed Changes</a>.
	To facilitate review,
	a <a>Last Call for Review of Proposed Changes</a> on a given specification
	<em class=rfc2119>should not</em> be issued more frequently
	than approximately once every six months.

	At the end of the <a>Last Call for Review of Proposed Changes</a>,
	the [=W3C Decision=] may either be
	to reject the [=Proposed Change=],
	or to clear the [=Proposed Change=] for advancement as is,
	or to return the proposal to the [=Working Group=] with a request to [=formally address=]
	comments made on the changes under review.
	If the [=Working Group=] needs to amend a [=Proposed Change=]
	in response to review feedback
	it must issue another <a>Last Call for Review of Proposed Change</a> on the revised change
	before it can be incorporated into the main text.

	Once all comments on a [=Proposed Change=] have been [=formally addressed=],
	and after the [=Working Group=] can show [=adequate implementation experience=]
	and the fulfillment of all other requirements of Recommendation text,
	it may incorporate the [=Proposed Change=] into the normative [=Recommendation=]
	by issuing an [=update request=] for publication of the updated [=Recommendation=].

	To ensure adequate review of [=Proposed Change=] combinations,
	only [=Proposed Changes=] included in the most recent
	[=Last Call for Review of Proposed Changes=]
	can be incorporated into the normative [=Recommendation=] text.
	(Thus if incorporation of a [=Proposed Change=] is postponed,
	it may need be included in multiple Last Calls for Review of Proposed Changes.)

<h5 id=rec-conversion>
Transitoning between Expandable Recommendation and Recommendation</h5>

	If the [=Working Group=]'s [=Charter=] allows a Technical Report
	to be published either as a [=Recommendation=] or as a [=Expandable Recommendation=],
	it is possible to republish a [=Recommendation=] as a [=Expandable Recommendation=]:
	There <em class=rfc2119>must</em> be a [=Working Group=] decision to do so,
	after which the [=Team=] <em class=rfc2119>must</em> solicit
	an [=Advisory Committee Review=] of the proposed change of status.
	The review period <em class="rfc2119">must</em> last at least <span class="time-interval">28 days</span>.
	If the proposal is approved,
	the [=Recommendation=] is republished as a [=Expandable Recommendation=].
	Republishing a [=Expandable Recommendation=] as a [=Recommendation=],
	if allowed by the relevant [=Working Group=]'s charter,
	follows the same process.

	Note: An AC Review is necessary to make sure the change from
	[=Recommendation=] to [=Expandable Recommendation=] is not made lightly,
	as there may be stakeholders who rely on a [=Recommendation=] having a fixed scope.

<h4 id="tr-end">
Retiring Recommendation Track Documents</h4>

	Work on a technical report <em class="rfc2119">may</em> cease at any time.
	Work <em class="rfc2119 new">should</em> cease
	if W3C or a [=Working Group=] determines
	that it cannot productively carry the work any further.

<h5 id=abandon-draft>
Abandoning an Unfinished Technical Report</h5>

	Any [=technical report=] no longer intended
	to advance or to be maintained,
	and that is not being rescinded,
	<em class="rfc2119">should</em> be [=published=] as a [=Working Group Note=].
	This can happen if
	the [=Working Group=] decided
	to abandon work on the report,
	or the [=Director=] required the [=Working Group=]
	to discontinue work on the technical report before completion.
	If the [=Director=] <a href="#GeneralTermination">closes a Working Group</a>
	W3C <em class="rfc2119">must</em> [=publish=] any unfinished [=technical report=]
	on the Recommendation track as [=Working Group Notes=].

<h5 id=rescind-cr>
Rescinding a Candidate Recommendation</h5>

	The process for rescinding a [=Candidate Recommendation=]
	is the same as for rescinding a [=Recommendation=].

<h5 id="rec-rescind">
Abandoning a W3C Recommendation</h5>

	It is possible that W3C decides
	that implementing a particular [=Recommendation=] is no longer recommended.
	There are three designations for such specifications,
	chosen depending on the advice W3C wishes to give about further use of the specification.

	W3C <em class="rfc2119">may</em> obsolete a [=Recommendation=],
	for example if the W3C Community decides that the [=Recommendation=] no longer represents best practices,
	or is not adopted and is not apparently likely to be adopted.
	An [=Obsolete Recommendation=] <em class="rfc2119">may</em> be restored to normal [=Recommendation=],
	for example because despite marking it Obsolete the specification is later more broadly adopted.

	W3C <em class="rfc2119">may</em> declare a Recommendation Superseded
	if a newer version exists which the W3C recommends for new adoption.
	The process for declaring a Recommendation Superseded is the same as for declaring it Obsolete, below;
	only the name and explanation change.

	W3C <em class="rfc2119">may</em> rescind a Recommendation
	if W3C believes there is no reasonable prospect of it being restored
	for example due to burdensome patent claims that affect implementers and cannot be resolved;
	see the W3C Patent Policy [[!PATENT-POLICY]]
	and in particular [[PATENT-POLICY#283|section 2.8.3]]
	and [[PATENT-POLICY#sec-PAG-conclude|section 5.5.5.4]].

	W3C only rescinds, supersedes, or obsoletes entire Recommendations.
	A [=Recommendation=] can be both superseded and obsolete.
	To rescind, supersede, or obsolete some part of a Recommendation,
	W3C follows the process for <a href="#revising-rec">modifying a Recommendation</a>.


	Note: For the purposes of the W3C Patent Policy
	[[PATENT-POLICY]]
	an [=Obsolete Recommendation|Obsolete=] or [=Superseded Recommendation=] has the status of an active [=Recommendation=],
	although it is not recommended for future implementation;
	a [=Rescinded Recommendation=] ceases to be in effect
	and no new licenses are granted under the Patent Policy.

	<div role="region" aria-label="Retiring a W3C Recommendation">
		<pre class=include-raw>
			path: retiring-rec.svg
		</pre>
	</div>

<h5 id="deactivation">
Process for Rescinding, Obsoleting, Superseding a Recommendation</h5>

	The process of obsoleting,
	rescinding,
	superseding,
	or restoring
	a [=Recommendation=] can be initiated
	either by a request from the [=Director=]
	or via a request from any of the following:

	<ul>
		<li>
			The [=Working Group=] who produced,
			or is chartered to maintain,
			the [=Recommendation=].

		<li>
			The [=TAG=], if there is no such [=Working Group=]

		<li>
			Any individual who made a request to the relevant [=Working Group=] as described above,
			or the [=TAG=] if such a group does not exist, to obsolete, rescind, supersede, or restore a [=Recommendation=],
			where the request was not answered within 90 days

		<li>
			5% of the members of the Advisory Committee
	</ul>

	<p id="proposed-rescinded-rec">The [=Director=] <em class="rfc2119">must</em> then
	submit the request to the <a href="#AC">Advisory Committee</a> for review.
	For any [=Advisory Committee review=] of a proposal to
	obsolete,
	rescind,
	supersede,
	or restore
	a [=Recommendation=] the [=Director=] <em class="rfc2119">must</em>:

	<ul>
		<li>
			announce the proposal to all [=Working Group=] [=Chairs=],
			and to the Public,
			as well as to the [=Advisory Committee=]

		<li>
			indicate that this is a proposal to
			Rescind,
			Obsolete,
			Supersede,
			or restore,
			a [=Recommendation=] as appropriate

		<li>
			identify the [=Recommendation=] by URL

		<li>
			publish a rationale for the proposal

		<li>
			identify known dependencies
			and solicit review from all dependent [=Working Groups=]

		<li>
			solicit public review

		<li>
			specify the deadline for review comments,
			which <em class="rfc2119">must</em> be at least 28 days
			after the [=Director=]'s announcement
	</ul>

	and <em class="rfc2119">should</em>

	<ul>
		<li>
			identify known implementations.
	</ul>

	If there was any [=dissent=] in the [=Advisory Committee review=],
	the [=Director=] <em class="rfc2119">must</em> publish
	the substantive content of the dissent to W3C
	<strong>and the public</strong>,
	and <em class="rfc2119">must</em> [=formally address=] the dissent
	at least 14 days
	before publication as an [=Obsolete Recommendation|Obsolete=] or [=Rescinded Recommendation=].

	The <a href="#AC">Advisory Committee</a> <em class="rfc2119">may</em> initiate an [=Advisory Committee Appeal=]
	of the [=Director=]'s decision.

	W3C <em class="rfc2119">must</em> publish an [=Obsolete Recommendation|Obsolete=] or [=Rescinded Recommendation=]
	with up to date status.
	The updated version <em class="rfc2119">may</em> remove the main body of the document.
	The Status of this Document section <em class="rfc2119">should</em> link
	to the explanation of <a href="https://www.w3.org/2016/11/obsoleting-rescinding/">Obsoleting and Rescinding W3C Specifications</a> [[OBS-RESC]] as appropriate.

	Once W3C has published a Rescinded Recommendation,
	future W3C technical reports <em class="rfc2119">must not</em> include normative references
	to that technical report.

	Note: W3C strives to ensure that all Technical Reports
	will continue to be available at their version-specific URL.


<h3 id="Note">
Working Group and Interest Group Notes</h3>

	A <dfn export id="WGNote" lt="W3C Note | WG Note | Working Group Note | IG Note | Interest Group Note | Note">Working Group Note or Interest Group Note</dfn> (<abbr>NOTE</abbr>)
	is published
	by a chartered [=Working Group=] or [=Interest Group=]
	to provide a stable reference for a useful document
	that is not intended to be a formal standard,
	or to document work that was abandoned without producing a [=Recommendation=].

	[=Working Groups=] and [=Interest Groups=] <em class="rfc2119">may</em> publish work as [=W3C Notes=].
	Examples include:

	<ul>
		<li>
			supporting documentation for a specification,
			such as explanations of design principles
			or use cases and requirements,

		<li>
			non-normative guides to good practices,

		<li>
			specifications where work has been stopped
			and there is no longer consensus for making them a new standard.
	</ul>

	Some [=W3C Notes=] are developed through successive [=Working Drafts=],
	with an expectation that they will become [=Notes=],
	while others are simply [=published=].
	There are few formal requirements to [=publish=] a document as a [=W3C Note=],
	and they have no standing as a recommendation of W3C
	but are simply documents preserved for historical reference.

	In order to publish a [=Note=],
	a [=Working Group=] or [=Interest Group=]:

	<ul>
		<li>
			<em class="rfc2119">may</em> [=publish=] a [=Note=]
			with or without its prior publication as a [=Working Draft=].

		<li>
			<em class="rfc2119">must</em> record the group's decision
			to request publication as a [=Note=], and

		<li>
			<em class="rfc2119">should</em> publish documentation
			of significant changes to the technical report
			since any previous publication.
	</ul>

	Possible next steps:

	<ul>
		<li>
			End state:
			A technical report <em class="rfc2119">may</em> remain
			a Working or Interest Group [=Note=] indefinitely

		<li>
			A [=Working Group=] <em class="rfc2119">may</em> resume work
			on a technical report
			within the scope of its charter
			at any time,
			at the maturity level the specification had before publication as a [=Note=]
	</ul>

	Note: The W3C Patent Policy [[PATENT-POLICY]]
	does not specify any licensing requirements or commitments for Working Group Notes.

<h3 id="further-reading">
Further reading</h3>

	Refer to <a href="https://www.w3.org/Guide/transitions">"How to Organize a Recommendation Track Transition"</a> [[TRANSITION]]
	in the <a href="https://www.w3.org/Guide/">Member guide</a> [[GUIDE]]
	for practical information about preparing for the reviews
	and announcements of the various steps,
	and <a href="https://www.w3.org/2002/05/rec-tips">tips on getting to Recommendation faster</a> [[REC-TIPS]].
	Please see also the <a href="https://www.w3.org/2003/01/republishing/">Requirements for modification of W3C Technical Reports</a> [[REPUBLISHING]].

<h2 id="ReviewAppeal">
Advisory Committee Reviews, Appeals, and Votes</h2>

	This section describes how the [=Advisory Committee=] reviews proposals from the [=Director=]
	and how [=Advisory Committee representatives=] initiate an [=Advisory Committee Appeal=]
	of a [=W3C decision=]
	or [=Director=]'s decision.
	A <dfn id="def-w3c-decision">W3C decision</dfn> is one
	where the [=Director=]
	(or the Director's delegate)
	has exercised the role of assessing consensus after an [=Advisory Committee review=].

<h3 id="ACReview">
Advisory Committee Reviews</h3>

	The Advisory Committee reviews:

	<ul>
		<li>
			<a href="#CharterReview">new and modified Working and Interest Groups</a>,

		<li>
			[=Proposed Recommendations=],
			<a href="#proposed-rescinded-rec">Proposals to Obsolete, Rescind, Supersede, or Restore Recommendations</a>,
			and

		<li>
			<a href="#GAProcess">Proposed changes to the W3C process</a>.

		<li>
			Conversion from a [=Recommendation=] to a [=Expandable Recommendation=], and vice versa.
	</ul>

<h4 id="ACReviewStart">
Start of a Review Period</h4>

	Each <dfn export lt="Advisory Committee review | AC Review"> Advisory Committee review</dfn> period
	begins with a Call for Review from the [=Team=] to the [=Advisory Committee=].
	The <dfn id="reviewform">Call for Review</dfn> describes the proposal,
	raises attention to deadlines,
	estimates when the decision will be available,
	and includes other practical information.
	Each Member organization <em class="rfc2119">may</em> send one review,
	which <em class="rfc2119">must</em> be returned by its [=Advisory Committee representative=].

	The Team <em class="rfc2119">must</em> provide two channels for Advisory Committee review comments:

	<ol>
		<li>
			an archived [=Team-only=] channel;

		<li>
			an archived [=Member-only=] channel.
	</ol>

	The [=Call for Review=] <em class="rfc2119">must</em> specify
	which channel is the default for review comments on that Call.

	Reviewers <em class="rfc2119">may</em> send information
	to either or both channels.
	They <em class="rfc2119">may</em> also share their reviews
	with other Members on the <a href="#ACCommunication">Advisory Committee discussion list</a>.

	A Member organization <em class="rfc2119">may</em> modify its review
	during a review period
	(e.g., in light of comments from other Members).

<h4 id="ACReviewAfter">
After the Review Period</h4>

	After the review period,
	the [=Director=] <em class="rfc2119">must</em> announce
	to the [=Advisory Committee=]
	the level of support for the proposal
	([=consensus=] or [=dissent=]).
	The [=Director=] <em class="rfc2119">must</em> also indicate
	whether there were any [=Formal Objections=],
	with attention to <a href="#confidentiality-change">changing confidentiality level</a>.
	This [=W3C decision=] is generally one of the following:

	<ol>
		<li>
			The proposal is approved,
			possibly with [=editorial changes=] integrated.

		<li>
			The proposal is approved,
			possibly with [=substantive changes=] integrated.
			In this case the Director's announcement <em class="rfc2119">must</em> include rationale
			for the decision to advance the document despite the proposal for a substantive change.

		<li>The proposal is returned for additional work,
			with a request to the initiator to [=formally address=] certain issues.

		<li>
			The proposal is rejected.
	</ol>

	This document does not specify
	time intervals between the end of an [=Advisory Committee review=] period
	and the [=W3C decision=].
	This is to ensure that the Members and [=Team=]
	have sufficient time to consider comments
	gathered during the review.
	The [=Advisory Committee=] <em class="rfc2119">should not</em> expect an announcement
	sooner than <span class="time-interval">two weeks</span> after the end of a review period.
	If, after <span class="time-interval">three weeks</span>, the Director has not announced the outcome,
	the [=Director=] <em class="rfc2119">should</em> provide the [=Advisory Committee=] with an update.

<h3 id="ACAppeal">
Appeal by Advisory Committee Representatives</h3>

	[=Advisory Committee representatives=] <em class="rfc2119">may</em> appeal certain decisions,
	though appeals are only expected to occur in extraordinary circumstances.

	When a [=W3C decision=] is made following an [=Advisory Committee review=],
	[=Advisory Committee representatives=] <em class="rfc2119">may</em> initiate
	an <dfn export lt="Advisory Committee Appeal|AC Appeal| Appeal">Advisory Committee Appeal</dfn>.
	These [=W3C decisions=] include those related to group creation and modification,
	and transitions to new maturity levels for Recommendation Track documents
	and the Process document.

	[=Advisory Committee representatives=] <em class="rfc2119">may</em> also initiate an appeal
	for certain [=Director=]'s decisions that do not involve an [=Advisory Committee review=].
	These cases are identified in the sections
	which describe the requirements for the [=Director=]'s decision
	and include
	additional (non-reviewed) maturity levels of Recommendation Track documents,
	group [=charter extensions=] and closures,
	and [=Memoranda of Understanding=].

	In all cases,
	an [=appeal=] <em class="rfc2119">must</em> be initiated within <span class="time-interval">three weeks</span> of the decision.

	An [=Advisory Committee representative=] initiates an [=appeal=] by sending a request to the [=Team=].
	The request should say “I appeal this Director's Decision”
	and identify the decision.
	Within one week the [=Team=] <em class="rfc2119">must</em> announce the appeal process
	to the [=Advisory Committee=]
	and provide a mechanism for [=Advisory Committee representatives=]
	to respond with a statement of positive support for this appeal.
	The archive of these statements <em class="rfc2119">must</em> be [=member-only=].
	If, within <span class="time-interval">one week</span> of the Team's announcement,
	5% or more of the [=Advisory Committee=] support the appeal request,
	the Team <em class="rfc2119">must</em> organize an appeal vote
	asking the [=Advisory Committee=]
	“Do you approve of the Director's Decision?”
	together with links to the [=Director=]'s decision and the appeal support.

	The ballot <em class="rfc2119">must</em> allow for three possible responses:
	“Approve”,
	“Reject”,
	and “Abstain”,
	together with Comments.

	If the number of votes to reject
	exceeds the number of votes to approve,
	the decision is overturned.
	In that case, there are the following possible next steps:

	<ol>
		<li>
			The proposal is rejected.

		<li>
			The proposal is returned for additional work,
			after which the applicable decision process is re-initiated.
	</ol>

<h3 id="ACVotes">
Advisory Committee Votes</h3>

	The [=Advisory Committee=] votes in <a href="#AB-TAG-elections">elections for seats on the TAG or Advisory Board</a>,
	and in the event of an [=Advisory Committee Appeal=]
	achieving the required support to trigger an appeal vote.
	Whenever the [=Advisory Committee=] votes,
	each Member or group of [=related Members=] has one vote.
	In the case of <a href="#AB-TAG-elections">Advisory Board and TAG elections</a>,
	“one vote” means “one vote per available seat”.

<h2 id="GAEvents">
Workshops and Symposia</h2>

	The Team organizes <dfn id="EventsW" export>Workshops</dfn>
	and <dfn id="EventsS" export lt="Symposium | Symposia">Symposia</dfn>
	to promote early involvement in the development of W3C activities
	from Members and the public.

	The goal of a Workshop is usually
	either to convene experts and other interested parties for an exchange of ideas
	about a technology or policy,
	or to address the pressing concerns of W3C Members.
	Organizers of the first type of Workshop
	<em class="rfc2119">may</em> solicit position papers for the Workshop program
	and <em class="rfc2119">may</em> use those papers
	to choose attendees and/or presenters.

	The goal of a Symposium is usually
	to educate interested parties about a particular subject.

	The Call for Participation in a Workshop or Symposium
	<em class="rfc2119">may</em> indicate participation requirements or limits,
	and expected deliverables
	(e.g., reports and minutes).
	Organization of an event does not guarantee
	further investment by W3C in a particular topic,
	but <em class="rfc2119">may</em> lead to proposals for new activities or groups.

	Workshops and Symposia generally last one to three days.
	If a Workshop is being organized to address the pressing concerns of Members,
	the Team <em class="rfc2119">must</em> issue the Call for Participation
	no later than <span class="time-interval">six weeks</span>
	prior to the Workshop's scheduled start date.
	For other Workshops and Symposia,
	the Team <em class="rfc2119">must</em> issue a Call for Participation
	no later than <span class="time-interval">eight weeks</span>
	prior to the meeting's scheduled start date.
	This helps ensure that speakers and authors
	have adequate time to prepare position papers and talks.

<h2 id="Liaisons">
Liaisons</h2>

	W3C uses the term <dfn export>liaison</dfn>
	to refer to coordination of activities with a variety of organizations,
	through a number of mechanisms
	ranging from very informal
	(e.g., an individual from another organization participates in a W3C Working Group,
	or just follows its work)
	to mutual membership,
	to even more formal agreements.
	Liaisons are not meant to substitute for W3C membership.

	All liaisons <em class="rfc2119">must</em> be coordinated by the [=Team=]
	due to requirements for public communication;
	patent,
	copyright,
	and other IPR policies;
	confidentiality agreements;
	and mutual membership agreements.

	The [=W3C Director=] <em class="rfc2119">may</em> negotiate
	a Memorandum of Understanding with another organization.
	For the purposes of the W3C Process a <dfn id="mou" export lt="Memorandum of Understanding | Memoranda of Understanding | MoU">Memorandum of Understanding</dfn> (<abbr>MoU</abbr>)
	is a formal agreement or similar contractual framework between W3C and another party or parties,
	other than agreements between the [=Hosts=]
	or between [=Hosts=] and W3C members
	for the purposes of membership
	and agreements related to the ordinary provision of services
	for the purposes of running W3C,
	that specifies rights and obligations of each party toward the others.
	These rights and obligations <em class="rfc2119">may</em> include joint deliverables,
	an agreed share of technical responsibilities with due coordination,
	and/or considerations for confidentiality and specific IPR.
	The agreement may be called something other than a “Memorandum of Understanding”,
	and something called a “Memorandum of Understanding”
	may not be an [=MoU=] for the purposes of the Process.

	Before signing the MoU,
	the Team <em class="rfc2119">must</em> inform
	the [=Advisory Committee=] of the intent to sign
	and make the MoU available for [=Advisory Committee review=];
	[=Advisory Committee representatives=] <em class="rfc2119">may</em> initiate an [=Advisory Committee Appeal=]
	of the decision to sign the [=MoU=].
	Unless an [=appeal=] rejects the proposal to sign an MoU,
	the [=Director=] may sign the [=MoU=] on behalf of W3C.
	A signed [=Memorandum of Understanding=] <em class="rfc2119">should</em> be made public.

	Information about <a href="https://www.w3.org/2001/11/StdLiaison">W3C liaisons with other organizations</a> [[LIAISON]]
	and the guidelines W3C follows when creating a liaison is available on the Web.

<h2 id="Submission">
Member Submission Process</h2>

	The Member Submission process allows Members
	to propose technology
	or other ideas
	for consideration by the [=Team=].
	After review,
	the [=Team=] <em class="rfc2119">may</em> make the material available at the W3C Web site.
	The formal process affords Members a record of their contribution
	and gives them a mechanism for disclosing the details of the transaction with the Team
	(including IPR claims).
	The [=Team=] also makes review comments on the Submitted materials available for W3C Members,
	the public,
	and the media.

	A <dfn id="MemberSubmission">Member Submission</dfn> consists of:

	<ul>
		<li>
			One or more documents developed outside of the W3C process, and

		<li>
			Information about the documents,
			provided by the Submitter.
	</ul>

	One or more Members (called the <dfn export lt="submitter">Submitter(s)</dfn>)
	<em class="rfc2119">may</em> participate in a Member Submission.
	Only W3C Members <em class="rfc2119">may</em> be listed as [=Submitters=].

	The Submission process consists of the following steps:

	<ol>
		<li>
			One of the [=Submitters=] sends a request to the Team to acknowledge the Submission request.
			The Team and [=Submitter=](s) communicate to ensure that the [=Member Submission=] is complete.

		<li>
			After [=Team=] review, the [=Director=] <em class="rfc2119">must</em> either
			acknowledge or reject the Submission request.

			<ul>
				<li>
					If <a href="#SubmissionYes">acknowledged</a>,
					the Team <em class="rfc2119">must</em> make the [=Member Submission=] available
					at the public W3C Web site,
					in addition to Team comments about the [=Member Submission=].

				<li>If <a href="#SubmissionNo">rejected</a>,
					the [=Submitter=](s) <em class="rfc2119">may</em> initiate a <a href="#SubmissionNo">Submission Appeal</a>
					to either the [=TAG=]
					or the [=Advisory Board=].
			</ul>
	</ol>

	<div class=note>
		Note: To avoid confusion about the Member Submission process, please note that:

		<ul>
			<li>
				Documents in a [=Member Submission=] are developed outside
				of the W3C <a href="#Reports">technical report development process</a>
				(and therefore are not included in the <a href="https://www.w3.org/TR/">index of W3C technical reports</a> [[TR]]).

			<li>
				The Submission process is <strong>not</strong> a means
				by which Members ask for “ratification” of these documents
				as [=W3C Recommendations=].

			<li>
				There is no requirement or guarantee
				that technology which is part of an acknowledged Submission request
				will receive further consideration by W3C
				(e.g., by a W3C [=Working Group=]).
		</ul>
	</div>

	Making a Member Submission available at the W3C website
	does not imply endorsement by W3C,
	including the W3C Team or Members.
	The acknowledgment of a Submission request
	does not imply that any action will be taken by W3C.
	It merely records publicly
	that the Submission request has been made by the Submitter.
	A Member Submission made available by W3C
	<em class="rfc2119">must not</em> be referred to as “work in progress” of the W3C.

	The list of <a href="https://www.w3.org/Submission/">acknowledged Member Submissions</a> [[SUBMISSION-LIST]]
	is available at the W3C Web site.

<h3 id="SubmissionRights">
Submitter Rights and Obligations</h3>

	When more than one Member jointly participates in a Submission request,
	only one Member formally sends in the request.
	That Member <em class="rfc2119">must</em> copy
	each of the [=Advisory Committee representatives=] of the other participating Members,
	and each of those [=Advisory Committee representatives=] <em class="rfc2119">must</em> confirm
	(by email to the Team)
	their participation in the Submission request.

	At any time prior to acknowledgment,
	any [=Submitter=] <em class="rfc2119">may</em> withdraw support for a Submission request
	(described in "<a href="https://www.w3.org/2000/09/submission">How to send a Submission request</a>" [[SUBMISSION-REQ]]).
	A Submission request is “withdrawn” when no Submitter(s) support it.
	The Team <em class="rfc2119">must not</em> make statements
	about withdrawn Submission requests.

	Prior to acknowledgment,
	the [=Submitter=](s) <em class="rfc2119">must not</em>,
	<strong>under any circumstances</strong>,
	refer to a document as “submitted to the World Wide Web Consortium”
	or “under consideration by W3C” or any similar phrase
	either in public or Member communication.
	The [=Submitter=](s) <em class="rfc2119">must not</em> imply
	in public or Member communication
	that W3C is working (with the [=Submitter=](s)) on the material in the [=Member Submission=].
	The [=Submitter=](s) <em class="rfc2119">may</em> release the documents in the Member Submission to the public
	prior to acknowledgment
	(without reference to the Submission request).

	After acknowledgment,
	the [=Submitter=](s) <em class="rfc2119">must not</em>,
	<strong>under any circumstances</strong>,
	imply W3C investment in the Member Submission
	until, and unless, the material has been adopted as a deliverable
	of a W3C [=Working Group=].

<h4 id="SubmissionScope">
Scope of Member Submissions</h4>

	When a technology overlaps in scope with the work of a chartered Working Group,
	Members <em class="rfc2119">should</em> <a href="#group-participation">participate in the Working Group</a>
	and contribute the technology to the group's process
	rather than seek publication through the Member Submission process.
	The [=Working Group=] <em class="rfc2119">may</em> incorporate the contributed technology into its deliverables.
	If the Working Group does not incorporate the technology,
	it <em class="rfc2119">should not</em> publish the contributed documents
	as [=Working Group Notes=] since [=Working Group Notes=]
	represent group output,
	not input to the group.

	On the other hand,
	while W3C is in the early stages of developing a charter,
	Members <em class="rfc2119">should</em> use the Submission process
	to build consensus around concrete proposals for new work.

	Members <em class="rfc2119">should not</em> submit materials
	covering topics well outside the scope of <a href="https://www.w3.org/Consortium/mission">W3C's mission</a> [[MISSION]].

<h4 id="SubmissionReqs">
Information Required in a Submission Request</h4>

	The [=Submitter=](s)
	and any other authors of the submitted material
	<em class="rfc2119">must</em> agree that,
	if the request is acknowledged,
	the documents in the Member Submission will be subject to the <a href="https://www.w3.org/Consortium/Legal/copyright-documents">W3C Document License</a> [[!DOC-LICENSE]]
	and will include a reference to it.
	The [=Submitter=](s) <em class="rfc2119">may</em> hold the copyright for the documents in a Member Submission.

	The request <em class="rfc2119">must</em> satisfy the Member Submission licensing commitments
	in “Licensing Commitments in W3C Submissions”
	in the W3C Patent Policy [[!PATENT-POLICY]].

	The [=Submitter=](s) <em class="rfc2119">must</em> include the following information:

	<ul>
		<li>
			The list of all submitting Members.

		<li>
			Position statements from all submitting Members (gathered by the Submitter).
			All position statements <em class="rfc2119">must</em> appear in a separate document.

		<li>
			Complete electronic copies of any documents submitted for consideration
			(e.g., a technical specification,
			a position paper,
			etc.)
			If the Submission request is acknowledged,
			these documents will be made available by W3C
			and therefore <em class="rfc2119">must</em> satisfy the Communication Team's
			<a href="https://www.w3.org/pubrules/">Publication Rules</a> [[!PUBRULES]].
			[=Submitters=] <em class="rfc2119">may</em> hold the copyright for the material contained in these documents,
			but when made available by W3C,
			these documents <em class="rfc2119">must</em> be subject to the provisions
			of the <a href="https://www.w3.org/Consortium/Legal/copyright-documents">W3C Document License</a> [[!DOC-LICENSE]].
	</ul>

	The request <em class="rfc2119">must</em> also answer the following questions.

	<ul>
		<li>
			What proprietary technology is required to implement the areas addressed by the request,
			and what terms are associated with its use?
			Again, many answers are possible,
			but the specific answer will affect the Team's decision.

		<li>
			What resources, if any,
			does the Submitter intend to make available
			if the W3C acknowledges the Submission request
			and takes action on it?

		<li>
			What action would the Submitter like W3C to take
			if the Submission request is acknowledged?

		<li>
			What mechanisms are there to make changes to the specification being submitted?
			This includes, but is not limited to,
			stating where change control will reside
			if the request is acknowledged.
	</ul>

	For other administrative requirements related to Submission requests,
	see “<a href="https://www.w3.org/2000/09/submission">How to send a Submission request</a>” [[MEMBER-SUB]].

<h3 id="TeamSubmissionRights">
Team Rights and Obligations</h3>

	Although they are not technical reports,
	the documents in a [=Member Submission=]
	<em class="rfc2119">must</em> fulfil the requirements established by the [=Team=],
	including the Team's <a href="https://www.w3.org/pubrules/">Publication Rules</a> [[!PUBRULES]].

	The [=Team=] sends a <dfn id="validation-notice">validation notice</dfn> to the [=Submitter=](s)
	once the Team has reviewed a Submission request
	and judged it complete and correct.

	Prior to a decision to <a href="#SubmissionYes">acknowledge</a>
	or <a href="#SubmissionNo">reject</a> the request,
	the request is [=Team-only=],
	and the [=Team=] <em class="rfc2119">must</em> hold it in the strictest confidentiality.
	In particular,
	the Team <em class="rfc2119">must not</em> comment to the media
	about the Submission request.

<h3 id="SubmissionYes">
Acknowledgment of a Submission Request</h3>

	The [=Director=] <a href="#SubmissionYes">acknowledges</a> a Submission request
	by sending an announcement to the [=Advisory Committee=].
	Though the announcement <em class="rfc2119">may</em> be made at any time,
	the [=Submitter=](s) can expect an announcement between <span class="time-interval">four to six weeks</span>
	after the [=validation notice=].
	The [=Team=] <em class="rfc2119">must</em> keep the [=Submitter=](s) informed
	of when an announcement is likely to be made.

	Once a Submission request has been acknowledged,
	the [=Team=] <em class="rfc2119">must</em>:

	<ul>
		<li>
			Make the [=Member Submission=] available at the W3C website.

		<li>
			Make the Team comments about the Submission request available at the W3C website.
	</ul>

	If the [=Submitter=](s) wishes to modify
	a document made available as the result of acknowledgment,
	the Submitter(s) <em class="rfc2119">must</em> start the Submission process from the beginning,
	even just to correct [=editorial changes=].

<h3 id="SubmissionNo">
Rejection of a Submission Request, and Submission Appeals</h3>

	The [=Director=] <em class="rfc2119">may</em> reject a Submission request
	for a variety of reasons,
	including any of the following:

	<ul>
		<li>
			The ideas expressed in the request
			overlap in scope with the work of a chartered Working Group,
			and acknowledgment might jeopardize the progress of the group.

		<li>
			The IPR statement made by the [=Submitter=](s) is inconsistent with the W3C's
			Patent Policy [[!PATENT-POLICY]]
			and in particular the “Licensing Commitments in W3C Submissions”,
			<a href="https://www.w3.org/Consortium/Legal/copyright-documents">Document License</a> [[!DOC-LICENSE]],
			or other IPR policies.

		<li>
			The ideas expressed in the request are poor,
			might harm the Web,
			or run counter to <a href="https://www.w3.org/Consortium/mission">W3C's mission</a> [[MISSION]].

		<li>
			The ideas expressed in the request lie well outside the scope of W3C's mission.
	</ul>

	  In case of a rejection,
	  the [=Team=] <em class="rfc2119">must</em> inform the [=Advisory Committee representative=](s)
	  of the [=Submitter=](s).
	  If requested by the [=Submitter=](s),
	  the [=Team=] <em class="rfc2119">must</em> provide rationale
	  to the [=Submitter=](s) about the rejection.
	  Other than to the [=Submitter=](s),
	  the Team <em class="rfc2119">must not</em> make statements about why a Submission request was rejected.

	  The [=Advisory Committee representative=](s) of the [=Submitters=](s)
	  <em class="rfc2119">may</em> initiate a Submission Appeal
	  of the Team's Decision to the [=TAG=]
	  if the reasons are related to Web architecture,
	  or to the [=Advisory Board=]
	  if the request is rejected for other reasons.
	  In this case the [=Team=] <em class="rfc2119">should</em> make available
	  its rationale for the rejection to the appropriate body.
	  The [=Team=] will establish a process for such appeals
	  that ensures the appropriate <a href="#confidentiality-levels">level of confidentiality</a>.

<h2 id="GAProcess">
Evolution of the Governing Documents</h2>

	The <dfn>W3C Governing Documents</dfn> comprise:
	* the W3C Process (this document)
	* the W3C Patent Policy [[!PATENT-POLICY]]
	* the W3C Code of Ethics and Professional Conduct [[!CEPC]]

	Revision of a [=W3C Governing Document=] undergoes similar [=consensus=]-building processes as for [=technical reports=],
	with the [=Advisory Board=]--
	or another group acting under delegation from the [=Advisory Board=]--
	acting as the sponsoring [=Working Group=].
	To help assure fitness for purpose,
	the group should solicit input from the W3C community,
	and in particular the Team.

	The [=Advisory Board=] initiates review of a [=W3C Governing Document=] as follows:

	<ol>
		<li>
			The [=Team=] sends a Call for Review to the [=Advisory Committee=] and other W3C groups.

		<li>
			After comments have been [=formally addressed=] and the document possibly modified,
			the Team seeks endorsement from the Members by initiating an [=Advisory Committee review=].
			The review period <em class="rfc2119">must</em> last at least <span class="time-interval">28 days</span>.

		<li>
			<a href="#ACReviewAfter">After the Advisory Committee review</a>,
			if there is [=consensus=],
			the Team enacts the new process officially by announcing
			the <a href="#def-w3c-decision">W3C decision</a> to the [=Advisory Committee=].
			Advisory Committee representatives <em class="rfc2119">may</em> initiate
			an [=Advisory Committee Appeal=] to the W3C.
	</ol>

<h2 id="acks" class=non-normative>
Acknowledgments</h2>

	<em>This section is non-normative.</em>

	The editors are grateful to the following people,
	who as interested individuals and/or with the affiliation(s) listed,
	have contributed to this proposal for a revised Process:
	Brian Kardell,
	Carine Bournez (W3C),
	Charles McCathie Nevile (ConsenSys),
	Chris Wilson (Google),
	David Singer (Apple),
	Delfí Ramírez,
	Dominique Hazaël-Massieux (W3C),
	Elika J. Etemad <abbr title="also known as">aka</abbr> fantasai,
	Fuqiao Xue (W3C),
	Jeff Jaffe (W3C),
	Kevin Fleming (Bloomberg),
	Léonie Watson (The Paciello Group),
	Michael Champion (Microsoft),
	Nigel Megitt (BBC),
	Philippe Le Hégaret (W3C),
	Ralph Swick (W3C),
	Samuel Weiler (W3C),
	Sandro Hawke (W3C),
	Tantek Çelik (Mozilla),
	Virginia Fournier (Apple),
	Wendy Seltzer (W3C),
	Yves Lafon (W3C).

	The editors are sorry for forgetting any names,
	and grateful to those who have listened patiently to conversations about this document
	without feeling a need to add more.

	The following individuals contributed to the development of earlier versions of the Process:
	Alex Russell (Google),
	Andreas Tai (Institut fuer Rundfunktechnik),
	Andrew Betts (Fastly),
	Ann Bassetti (The Boeing Company),
	Anne van Kesteren,
	Art Barstow (Nokia, unaffiliated),
	Bede McCall (MITRE),
	Ben Wilson,
	Brad Hill (Facebook),
	Brian Kardell (JQuery),
	Carine Bournez (W3C),
	Carl Cargill (Netscape, Sun Microsystems, Adobe),
	Chris Lilley (W3C),
	Chris Wilson (Google),
	Claus von Riegen (SAP AG),
	Coralie Mercier (W3C),
	Cullen Jennings (Cisco),
	Dan Appelquist (Telefonica, Samsung),
	Dan Connolly (W3C),
	Daniel Dardailler (W3C),
	Daniel Glazman (Disruptive Innovations),
	David Baron (Mozilla),
	David Fallside (IBM),
	David Singer (Apple),
	David Singer (IBM),
	Delfí Ramírez,
	Dominique Hazaël-Massieux (W3C),
	Don Brutzman (Web3D),
	Don Deutsch (Oracle),
	Eduardo Gutentag (Sun Microsystems),
	Elika J. Etemad <abbr title="also known as">aka</abbr> fantasai,
	Florian Rivoal,
	Fuqiao Xue (W3C),
	Geoffrey Creighton (Microsoft),
	Geoffrey Snedden,
	Giri Mandyam (Qualcomm),
	Gregg Kellogg,
	Hadley Beeman,
	Helene Workman (Apple),
	Henri Sivonen (Mozilla),
	Håkon Wium Lie (Opera Software),
	Ian Hickson (Google),
	Ian Jacobs (W3C),
	Ivan Herman (W3C),
	J Alan Bird (W3C),
	Jay Kishigami 岸上順一 (NTT),
	Jean-Charles Verdié (MStar),
	Jean-François Abramatic (IBM, ILOG, W3C),
	Jeff Jaffe (W3C),
	Jim Bell (HP),
	Jim Miller (W3C),
	Joe Hall (CDT),
	John Klensin (MCI),
	Josh Soref (BlackBerry, unaffiliated),
	Judy Brewer (W3C),
	Judy Zhu 朱红儒 (Alibaba),
	Kari Laihonen (Ericsson),
	Karl Dubost (Mozilla),
	Ken Laskey (MITRE),
	Kevin Fleming (Bloomberg),
	Klaus Birkenbihl (Fraunhofer Gesellschaft),
	Larry Masinter (Adobe Systems),
	Lauren Wood (unaffiliated),
	Liam Quin (W3C),
	Léonie Watson (The Paciello Group),
	Marcos Cáceres (Mozilla),
	Maria Courtemanche (IBM),
	Mark Crawford (SAP),
	Mark Nottingham,
	Michael Champion (Microsoft),
	Michael Geldblum (Oracle),
	Mike West (Google),
	Mitch Stoltz (EFF),
	Natasha Rooney (GSMA),
	Nigel Megitt (BBC),
	Olle Olsson (SICS),
	Ora Lassila (Nokia),
	Paul Cotton (Microsoft),
	Paul Grosso (Arbortext),
	Peter Linss,
	Peter Patel-Schneider,
	Philippe Le Hégaret (W3C),
	Qiuling Pan (Huawei),
	Ralph Swick (W3C),
	Renato Iannella (IPR Systems),
	Rigo Wenning (W3C),
	Rob Sanderson (J Paul Getty Trust),
	Robin Berjon (W3C),
	Sally Khudairi (W3C),
	Sam Ruby (IBM),
	Sandro Hawke (W3C),
	Sangwhan Moon (Odd Concepts),
	Scott Peterson (Google),
	Steve Holbrook (IBM),
	Steve Zilles (Adobe Systems)
	Steven Pemberton (CWI),
	TV Raman (Google),
	Tantek Çelik (Mozilla),
	Terence Eden (Her Majesty's Government),
	Thomas Reardon (Microsoft),
	Tim Berners-Lee (W3C),
	Tim Krauskopf (Spyglass),
	Travis Leithead (Microsoft),
	Virginia Fournier (Apple),
	Virginie Galindo (Gemalto),
	Wayne Carr (Intel),
	Wendy Fong (Hewlett-Packard),
	Wendy Seltzer (W3C),
	Yves Lafon (W3C).

<h2 id="changes" class=non-normative>
Changes</h2>

	<em>This section is non-normative.</em>

<h3 id=changes-2019 class=no-num>
Changes since the <a href="https://www.w3.org/2019/Process-20190301/">1 March 2019 Process</a>:</h3>


	<p>This document is based on the <a href="https://www.w3.org/2019/Process-20190301/">1 March 2019 Process</a>.
	A <a href="https://w3c.github.io/w3process/issues-20190301">Disposition of Comments</a>,
	as well as
	a <a href="https://github.com/w3c/w3process/compare/process-2019...master">Detailed log of all changes since then</a>
	are available.
	The following is a summary:

	<ul>
		<li>
			Converted the source code of the process to the <a href="https://tabatkins.github.io/bikeshed/">Bikeshed</a> document format,
			improving the ease of maintenance,
			and gaining better cross linking capabilities as well as an [[#index]] in the process.
			Note that while this makes no change to the text of the process,
			it is a large change of the source code,
			and source level diffs are unlikely to be of help to compare the before and after state.

		<li>
			Phrasing clarifications and removal of unnecessary text
			in <a href="#AB-TAG-constraints"></a> and
			<a href="#ACReviewAfter"></a>.

		<li>
			Avoid using the specialized term “[=publish=]” to mean anything other than
			putting documents on TR.

		<li>
			Avoid using the specialized term “[=dissent=]” in situations that do not involve Formal Objections.

		<li>
			Retire the “Team Submissions” (<em>not</em> “Member Submissions”) mechanism,
			as it is unused,
			does not provide the Team with abilities it doesn't already have,
			nor provides meaningful governance of the Team's communications.

		<li>
			Clarify that appeal statements are meant to be member-only.

		<li>
			Clarify that Working Drafts have some, even if limited, standing.

		<li>
			Resolve minor wording conflict as to whether charters "should" or "may" include formal voting procedures.

		<li>
			Delete references to non existing parts of the Patent policy, previously invoked in the TAG participation section.

		<li>
			Remove a spurious link that made it look like AC-Reps were involved when participants in the TAG or AB resign.

		<li>
			Add some sub-section headings to improve readability and cross-section linking

		<li>
			Clarify the rule on vacant seats in [[#AB-TAG-elections]]

		<li>
			Eliminate the use of the undefined “minor changes” term

		<li>
			Define and differentiate between Group Decisions and Chair Decisions.

		<li>
			Define that the process to revise the CEPC and Patent Policy are the same as for revising the Process

		<li>
			Add note clarifying when it is appropriate to use the superseding process, and when not.

		<li>
			Dont require documenting editorial changes since the previous CR.

		<li>
			Add Note about the flexibility of Consensus.

		<li>
			Section 6 (on Technical Report development) has been reogranized,
			to disentangle definitions of the various maturities
			from the steps needed to publish
			and to transition form one maturity to another.

		<li>
			Significant additions were made to the Recommendation Track,
			both to facilitated maintenance,
			and to ease development of technical reports in a more agile fashion.
	</ul>

<h3 id=changes-2018 class=no-num>
Changes since the <a href="https://www.w3.org/2018/Process-20180201/">1 February 2018 Process</a>:</h3>


	<p>This document is based on the <a href="https://www.w3.org/2018/Process-20180201/">1 February 2018 Process</a>.
	A <a href="https://github.com/w3c/w3process/compare/process-20180201...process-2019">Detailed log of all changes since then</a> is available.
	The following is a summary:

	<ul>
		<li>
			Mention the existence of Community Groups and Business Groups
			in <a href="#Intro"></a>.

		<li>
			Change the size of the Advisory Board from a fixed 9 to a range of 9 to 11 elected members
			in <a href="#ABParticipation"></a>,
			and adjust the election mechanics accordingly
			in <a href="#AB-TAG-elections"></a>
			and <a href="#AB-TAG-vacated"></a>.

		<li>
			Clarify that AB and TAG members act in their personal capacity
			in <a href="#AB-TAG-participation"></a>.

		<li>
			Improve phrasing in <a href="#AB-TAG-constraints"></a>
			to avoid suggesting that while member organizations were limited to one seat,
			non member organizations could have several.

		<li>
			Clarify what constitutes “affiliation”
			for the purposes of the “change of affiliation” provisions for TAG and AB members
			in <a href="#AB-TAG-elections"></a>.

		<li>
			In case of resignation from a group,
			require that the team records the resignation notification,
			in <a href="#resignation"></a>.

		<li>
			Allow one person to represent (with disclosure) multiple organizations in a Working Group
			in <a href="#group-participation"></a>.

		<li>
			Various phrasing clarifications
			to <a href="#maturity-levels"></a>,
			and <a href="#transition-reqs"></a>,
			and <a href="#first-wd"></a>.

		<li>
			Added new <a href="#contributor-license"></a>
			to define the requirements for contributions made by non-WG members.

		<li>
			Clarify in <a href="#rec-rescind"></a>
			that a Recommendation can be both superseded and obsolete.

		<li>
			Add an informative diagram about the process of obsoleting, rescinding, or restoring a Recommendation
			in <a href="#rec-rescind"></a>.

		<li>
			Clarify in <a href="#rec-rescind"></a>
			that the process to obsolete, rescind, or restore a Recommendation does not have to be initiated by the Director.

		<li>
			Rephrase part of <a href="#Submission"></a>
			to avoid statement about Member Submissions that appeared to negate the possibility
			of having work originate in Community Groups.

		<li>
			Remove meaningless claim from <a href="#GAProcess"></a>
			that an alternative way to update the Process was to follow the REC track process.
	</ul>

<h3 id=changes-2017 class=no-num>
Changes between the <a href="https://www.w3.org/2017/Process-20170301/">1 March 2017 Process</a>
and the <a href="https://www.w3.org/2018/Process-20180201/">1 February 2018 Process</a>:</h3>

	A log of
	<a href="https://github.com/w3c/w3process/compare/7836a7a539e97fe430a930daa00e7f2151abb2ac...process-20180201">all changes from March 2017 to the 1 February 2018 Processs</a>
	is available,
	as is an earlier <a href="https://dvcs.w3.org/hg/AB/">log of changes in previous versions, from 2005-2014</a>.
	The following is a summary:


	<dl>
		<dt>
			<a href="#ABParticipation"></a>
		<dd>
			The AB has a team contact

		<dt>
			<a href="#tag-participation"></a>
		<dd>
			Tim Berners-Lee is a Life Member of the TAG.
		<dd>
			The TAG chair <em class="rfc2119">must</em> be one of the participants
		<dd>
			The TAG has 6 AC-elected members instead of 5
		<dd>
			The TAG has a team contact

		<dt>
			<a href="#AB-TAG-participation"></a>
		<dd>
			Seats on the AB or TAG cannot be delegated to a proxy

		<dt>
			<a href="#AB-TAG-vacated"></a>
		<dd>
			AB and TAG seats are vacated if the Director removes a participant,
			instead of if the chair asks them to resign
		<dd>
			Clarify that a special election can begin if a candidate resigns as of some future date,
			rather than waiting for that resignation to take effect.

		<dt>
			<a href="#ParticipationCriteria"></a>
		<dd>
			Participants in W3C activity are explicitly required to abide by the terms and spirit
			of the <a href="https://www.w3.org/Consortium/cepc/" title="Code of Ethics and Professional Conduct">CEPC</a>
		<dd>
			The Director can suspend or remove a participant from any Group or activity
			for failure to meet participation criteria.

		<dt>
			<a href="#CharterReview"></a>
		<dd>
			An AC member may request at least 60 days
			to review any new or substantively modified working group charter,
			with such review period occurring between the Call for Review
			and the Call for Participation.


		<dt>
			<a href="#WGCharter"></a>
		<dd>
			Charters <em class="rfc2119">must</em> include documentation of any voting procedures
			additional to those defined in <a href="#Votes"></a>.

		<dt>
			<a href="#revising-rec"></a>
		<dd>
			W3C can produce an <a href="#rec-amended">Amended Recommendation</a> to update a Recommendation,
			without adding new features,
			similar to the way a Working Group produces an Edited Recommendation.

		<dt>
			<a href="#rec-rescind"></a>
		<dd>
			A Recommendation can be declared Superseded.

		<dt>
			<a href="#ACAppeal"></a>
		<dd>
			Clarify that in the event of an Advisory Committee appeal, a Director's decision can be overtuned by more votes for than against

		<dt>
			<a href="#Liaisons"></a>
		<dd>
			Definition of MoU and conditions for approval.
	</dl>

<pre class=biblio>
{
	"JOIN": {
		"href": "https://www.w3.org/Consortium/join",
		"title": "How to Join W3C",
		"publisher": "W3C"
		},
	"MEMBER-AGREEMENT": {
		"href": "https://www.w3.org/Consortium/Agreement/Member-Agreement",
		"title": "W3C Membership Agreement",
		"publisher": "W3C"
	},
	"MEMBER-LIST": {
		"href": "https://www.w3.org/Consortium/Member/List",
		"title": "The list of current W3C Members",
		"publisher": "W3C"
	},
	"SUBMISSION-LIST": {
		"href": "https://www.w3.org/Submission/",
		"title": "The list of acknowledged Member Submissions",
		"publisher": "W3C"
	},
	"SUBMISSION-REQ": {
		"href": "https://www.w3.org/2000/09/submission",
		"title": "Make or Withdraw a Member Submission Request (Member-only access)",
		"publisher": "W3C"
	},
	"TR": {
		"href": "https://www.w3.org/TR/",
		"title": "The W3C technical reports index",
		"publisher": "W3C"
	},
	"MISSION": {
		"href": "https://www.w3.org/Consortium/mission",
		"title": "The W3C Mission statement",
		"publisher": "W3C"
	},
	"COLLABORATORS-AGREEMENT": {
		"href": "https://www.w3.org/Consortium/Legal/collaborators-agreement",
		"title": "Invited expert and collaborators agreement",
		"publisher": "W3C"
	},
	"DOC-LICENSE": {
		"href": "https://www.w3.org/Consortium/Legal/copyright-documents",
		"title": "W3C Document License",
		"publisher": "W3C"
	},
	"TRANSLATION": {
		"href": "https://www.w3.org/Consortium/Translation/",
		"title": "Translations of W3C technical reports",
		"publisher": "W3C"
	},
	"CONFLICT-POLICY": {
		"href": "https://www.w3.org/2000/09/06-conflictpolicy",
		"title": "Conflict of Interest Policy for W3C Team Members Engaged in Outside Professional Activities",
		"publisher": "W3C"
	},
	"TAG-CHARTER": {
		"href": "https://www.w3.org/2004/10/27-tag-charter.html",
		"title": "Technical Architecture Group (TAG) Charter",
		"publisher": "W3C"
	},
	"TAG-HP": {
		"href": "https://www.w3.org/2001/tag/",
		"title": "The TAG home page",
		"publisher": "W3C"
	},
	"REC-TIPS": {
		"href": "https://www.w3.org/2002/05/rec-tips",
		"title": "Tips for Getting to Recommendation Faster",
		"publisher": "W3C"
	},
	"LIAISON": {
		"href": "https://www.w3.org/2001/11/StdLiaison",
		"title": "W3C liaisons with other organizations",
		"publisher": "W3C"
	},
	"AB-HP": {
		"href": "https://www.w3.org/2002/ab/",
		"title": "The Advisory Board home page",
		"publisher": "W3C"
	},
	"PUBRULES": {
		"href": "https://www.w3.org/pubrules/",
		"title": "Publication Rules",
		"publisher": "W3C"
	},
	"FELLOWS": {
		"href": "https://www.w3.org/Consortium/Recruitment/Fellows",
		"title": "W3C Fellows Program",
		"publisher": "W3C"
	},
	"PATENT-POLICY": {
		"href": "https://www.w3.org/Consortium/Patent-Policy/",
		"title": "The W3C Patent Policy",
		"publisher": "W3C"
	},
	"REPUBLISHING": {
		"href": "https://www.w3.org/2003/01/republishing/",
		"title": "In-place modification of W3C Technical Reports",
		"publisher": "W3C"
	},
	"CALENDAR": {
		"href": "https://www.w3.org/participate/eventscal",
		"title": "Calendar of all scheduled official W3C events",
		"publisher": "W3C"
	},
	"GUIDE": {
		"href": "https://www.w3.org/Guide/",
		"title": "The Art of Consensus",
		"publisher": "W3C"
	},
	"CEPC": {
		"href": "https://www.w3.org/Consortium/cepc/",
		"title": "W3C Code of Ethics and Professional Conduct",
		"publisher": "W3C"
	},
	"OBS-RESC": {
		"href": "https://www.w3.org/2016/11/obsoleting-rescinding/",
		"title": "Obsoleting and Rescinding W3C Specifications",
		"publisher": "W3C"
	},
	"BG-CG": {
		"href": "https://www.w3.org/community/about/agreements/",
		"title": "Community and Business Group Process",
		"publisher": "W3C"
	},
	"CURRENT-AC": {
		"href": "https://www.w3.org/Member/ACList",
		"title": "Current Advisory Committee representatives (Member-only access)",
		"publisher": "W3C"
	},
	"GROUP-MAIL": {
		"href": "https://www.w3.org/Member/Groups",
		"title": "Group mailing lists (Member-only access)",
		"publisher": "W3C"
	},
	"INTRO": {
		"href": "https://www.w3.org/Member/Intro",
		"title": "Process, Patent Policy, Finances, Specs management, Strategic vision (Member-only access)",
		"publisher": "W3C"
	},
	"AC-MEETING": {
		"href": "https://www.w3.org/Member/Meeting/",
		"title": "Advisory Committee meetings (Member-only access)",
		"publisher": "W3C"
	},
	"MEMBER-HP": {
		"href": "https://www.w3.org/Member/",
		"title": "Member Web site (Member-only access)",
		"publisher": "W3C"
	},
	"MEMBER-SUB": {
		"href": "https://www.w3.org/2000/09/submission",
		"title": "How to send a Submission request",
		"publisher": "W3C"
	},
	"DISCIPLINARY-GL": {
		"href": "https://www.w3.org/2002/09/discipline",
		"title": "Guidelines for Disciplinary Action",
		"publisher": "W3C"
	},
	"ELECTION-HOWTO": {
		"href": "https://www.w3.org/2002/10/election-howto",
		"title": "How to Organize an Advisory Board or TAG election",
		"publisher": "W3C",
		"id": "MEM15"
	},
	"TEAM-CONTACT": {
		"href": "https://www.w3.org/Guide/teamcontact/role.html",
		"title": "Role of the Team Contact",
		"publisher": "W3C"
	},
	"CHAIR": {
		"href": "https://www.w3.org/Guide/chair/role.html",
		"title": "W3C Working/Interest Group Chair",
		"publisher": "W3C"
	},
	"CHARTER": {
		"href": "https://www.w3.org/Guide/process/charter.html",
		"title": "How to Create a Working Group or Interest Group",
		"publisher": "W3C"
	},
	"TRANSITION": {
		"href": "https://www.w3.org/Guide/transitions",
		"title": "Organize a Technical Report Transition",
		"publisher": "W3C"
	}
}
</pre>
